/*
 *  linux/arch/arm/mach-dvf/arch.c
 *
 *  Copyright (C) 2010-2012 DSPG Technologies GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/delay.h>
#include <linux/export.h>
#include <linux/of.h>
#include <linux/of_address.h>
#include <linux/of_platform.h>
#include <linux/memblock.h>
#include <linux/dma-mapping.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/memblock.h>

#include <mach/hardware.h>

#include "clock.h"

extern void dvf_timer_init(void);
static bool dvf_nodebug = false;
static void __iomem *denali_sync;

int dvf_get_usb11_gcr(void)
{
	return readl(IOMEM(IO_ADDRESS(DVF_SYSCFG_BASE)) + DVF_SYSCFG_USB11_GCR1);
}
EXPORT_SYMBOL(dvf_get_usb11_gcr);

void dvf_set_usb11_gcr(int value)
{
	writel(value, IOMEM(IO_ADDRESS(DVF_SYSCFG_BASE)) + DVF_SYSCFG_USB11_GCR1);
}
EXPORT_SYMBOL(dvf_set_usb11_gcr);

int dvf_get_chip_rev(void)
{
	return readl(IOMEM(IO_ADDRESS(DVF_SYSCFG_BASE)) + DVF_SYSCFG_CHIP_REV);
}
EXPORT_SYMBOL(dvf_get_chip_rev);

static void dvf_denali_sync(void)
{
	/* Perform read-after-write to make sure CPU port on Denali controller
	 * is flushed. */
	if (denali_sync) {
		writel_relaxed(0, denali_sync);
		barrier();
		readl_relaxed(denali_sync);
	}
}

static int __init dvf_denali_sync_init(void)
{
	dma_addr_t paddr;

	denali_sync = dma_alloc_coherent(NULL, 4, &paddr, GFP_KERNEL);
	if (!denali_sync)
		panic("dvf_denali_sync_init failed!");

	soc_mb = dvf_denali_sync;

	return 0;
}
arch_initcall(dvf_denali_sync_init);

bool dvf_debug_disabled(void)
{
	return dvf_nodebug;
}

static int __init nodebug_setup(char *__unused)
{
	dvf_nodebug = true;

	return 1;
}

__setup("nodebug", nodebug_setup);

static struct map_desc dvf_io_desc[] __initdata = {
	{
		.virtual	= IO_ADDRESS(DVF_CMU_BASE),
		.pfn		= __phys_to_pfn(DVF_CMU_BASE),
		.length		= SZ_4K,
		.type		= MT_DEVICE
	}, {
		.virtual	= IO_ADDRESS(DVF_SYSCFG_BASE),
		.pfn		= __phys_to_pfn(DVF_SYSCFG_BASE),
		.length		= SZ_4K,
		.type		= MT_DEVICE
	},
};

static const char *dvf_dt_match[] = {
	"dspg,dvf99",
	NULL
};

static void __init dvf_map_io(void)
{
	iotable_init(dvf_io_desc, ARRAY_SIZE(dvf_io_desc));
}

static void __init dvf_early_init(void)
{
	if (dvf_clk_init())
		panic("Clock init failed!");
}

static void dvf_reset(enum reboot_mode mode, const char *cmd)
{
	unsigned int value;

	/* Clear WDRSTIND and SWRSTIND before SW resetting */
	value = readl(IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_RSTSR));
	value |= (1<<5) | (1<<4) | (1<<3);
	writel(value, IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_RSTSR));

	writel(0x7496398B, IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_SWSYSRST));
	writel(0xB5C9C674, IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_SWSYSRST));
}

extern struct sys_timer dvf_timer;

DT_MACHINE_START(DVF99, "DSPG DVF99")
	.map_io		= dvf_map_io,
	.init_early	= dvf_early_init,
	.init_time	= dvf_timer_init,
	/* .reserve	= dvf_reserve, */
	.dt_compat	= dvf_dt_match,
	.restart	= dvf_reset,
MACHINE_END

