/*
 *  linux/arch/arm/mach-dvf/include/mach/hardware.h
 *
 *  This file contains the hardware definitions of the DVF base band family.
 *
 *  Copyright (C) 2010 DSPG Technologies GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __MACH_HARDWARE_H
#define __MACH_HARDWARE_H

#include <asm/sizes.h>

/*
 * Get virtual address of a device for the static mappings. The virtual address
 * of these mappings is within the vmalloc region. Only the first 16MB starting
 * from DVF_GPIO_BASE can be used.
 */
#define IO_ADDRESS(a)        (0xfe000000 - 0x05000000 + (a))

/*
 * Base address definitions.
 */
#define DVF_DDRC_BASE			0x00100000
#define DVF_USB_OTG_BASE		0x00300000
#define DVF_USB_11_BASE			0x00400000
#define DVF_GPU_BASE			0x00600000
#define DVF_SDMMC_BASE			0x00900000
#define DVF_SEC_BASE			0x00A00000
#define DVF_MPMC_BASE			0x01000000
#define DVF_BOOTROM_BASE		0x01200000

#define DVF_GPIO_BASE			0x05000000
#define DVF_PLICU_BASE			0x05100000
#define DVF_SYSCFG_BASE			0x05200000
#define DVF_CMU_BASE			0x05300000
#define DVF_NAND_LCD_BASE		0x05404000
#define DVF_QSPI_BASE			0x05500000
#define DVF_RTC_BASE			0x05600000
#define DVF_WATCHDOG_BASE		0x05700000
#define DVF_TIMER0_BASE			0x05800000
#define DVF_TIMER1_BASE			0x05800020
#define DVF_TIMER2_BASE			0x05900000
#define DVF_TIMER3_BASE			0x05900020
#define DVF_UART1_BASE			0x05A00000
#define DVF_UART2_BASE			0x05B00000
#define DVF_SPI1_BASE			0x05D00000
#define DVF_SPI2_BASE			0x05E00000
#define DVF_TDM1_BASE			0x05F00000
#define DVF_TDM2_BASE			0x06000000
#define DVF_I2C1_BASE			0x06100000
#define DVF_I2C2_BASE			0x06200000
#define DVF_PMU_DAIF_BASE		0x06300000
#define DVF_GDMAC_BASE			0x06400000
#define DVF_MII_BASE			0x06500000
#define DVF_CIU_BASE			0x06700000
#define DVF_LCDC_BASE			0x06B00000
#define DVF_SECURITY_BASE		0x06C00000
#define DVF_ETH_MAC_BASE		0x06E00000
#define DVF_DAP_BASE			0x07300000

/*
 * The internal SRAM is shared by multiple divices. Define the layout here to
 * have a single, consistent place!
 */
#define DVF_SRAM_BASE			0x00200000
#define DVF_SRAM_SIZE			SZ_32K
#define DVF_SRAM_GDMAC_SIZE		256
#define DVF_SRAM_GDMAC_BASE		(DVF_SRAM_BASE + DVF_SRAM_SIZE - DVF_SRAM_GDMAC_SIZE)
#define DVF_SRAM_WIFI_BASE		DVF_SRAM_BASE
#define DVF_SRAM_WIFI_SIZE		(DVF_SRAM_GDMAC_BASE - DVF_SRAM_BASE)

#define DVF_DRAM0_BASE			0x40000000UL
#define DVF_DRAM1_BASE			0x60000000UL
#define DVF_DRAM_SIZE			0x20000000UL

#define CSS_ITCM_BASE			0x08000000
#define CSS_ITCM_SIZE			(272 * SZ_1K)

#define CSS_AHBRAM_BASE			0x08200000
#define CSS_AHBRAM_SIZE			(82 * SZ_1K)

#define CSS_DTCM_BASE			0x08100000
#define CSS_DTCM_SIZE			SZ_128K

/*
 * Clock management unit
 */
#define DVF_CMU_PLL1CONTROL		0x00
#define DVF_CMU_PLL2CONTROL		0x04
#define DVF_CMU_PLL3CONTROL		0x08
#define DVF_CMU_CPUCLKCNTRL		0x14
#define DVF_CMU_COMCLKSEL		0x18	/* guarded by SEC CMU mutex */
#define DVF_CMU_CLKSWCNTRL		0x1C
#define DVF_CMU_CLKDIVCNTRL1	0x20
#define DVF_CMU_CLKDIVCNTRL2	0x24
#define DVF_CMU_CLKDIVCNTRL3	0x28
#define DVF_CMU_CLKDIVCNTRL4	0x2C
#define DVF_CMU_CLKDIVCNTRL5	0x30
#define DVF_CMU_CLKOSCCR		0x3C
#define DVF_CMU_SWSYSRST		0x40
#define DVF_CMU_SWWRMRST		0x44
#define DVF_CMU_RSTSR			0x48
#define DVF_CMU_SWCHWRST1		0x50	/* guarded by SEC CMU mutex */
#define DVF_CMU_SWCHWRST2		0x54	/* guarded by SEC CMU mutex */
#define DVF_CMU_SWCLKENR1		0x58	/* guarded by SEC CMU mutex */
#define DVF_CMU_SWCLKENR2		0x5C	/* guarded by SEC CMU mutex */
#define DVF_CMU_CUSTATR			0x60
#define DVF_CMU_TEST			0x68

/*
 * System configuration
 */
#define DVF_SYSCFG_GCR1				0x04
#define DVF_SYSCFG_GCR2				0x08
#define DVF_SYSCFG_CORTEX_IR		0x0C
#define DVF_SYSCFG_CHIP_ID			0x1C
#define DVF_SYSCFG_CHIP_REV			0x20
#define DVF_SYSCFG_SPCR1			0x54
#define DVF_SYSCFG_SPCR2			0x58
#define DVF_SYSCFG_USB11_GCR1			0x60
#define DVF_SYSCFG_DRAMCTL_GCR1		0x6c
#define DVF_SYSCFG_DRAMCTL_GCR2		0x70
#define DVF_SYSCFG_MEM_CFG			0x74

/* SysCfg pad enable definitions */
#define DVF_SYSCFG_SPCR1_RFAPU_PAD_EN			(1 << 31)
#define DVF_SYSCFG_SPCR1_MLSE _PAD_EN			(1 << 30)
#define DVF_SYSCFG_SPCR1_PMU_PAD_EN				(1 << 29)
#define DVF_SYSCFG_SPCR1_DIGMIC_PAN_EN			(1 << 28)
#define DVF_SYSCFG_SPCR1_FC_PAD_EN				(1 << 27)
#define DVF_SYSCFG_SPCR1_UART1_PAD_EN			(1 << 26)
#define DVF_SYSCFG_SPCR1_UART2_PAD_EN			(1 << 25)
#define DVF_SYSCFG_SPCR1_TDM1_PAD_EN			(1 << 23)
#define DVF_SYSCFG_SPCR1_TDM2_PAD_EN			(1 << 22)
#define DVF_SYSCFG_SPCR1_SPI1_PAD_EN			(1 << 20)
#define DVF_SYSCFG_SPCR1_SPI2_PAD_EN			(1 << 19)
#define DVF_SYSCFG_SPCR1_I2C_1_PAD_EN			(1 << 17)
#define DVF_SYSCFG_SPCR1_I2C_2_PAD_EN			(1 << 16)
#define DVF_SYSCFG_SPCR1_SD_PAD_EN				(1 << 15)
#define DVF_SYSCFG_SPCR1_EMACA_PAD_EN			(1 << 14)
#define DVF_SYSCFG_SPCR1_BMP_SYNCPORT_PAD_EN	(1 << 12)
#define DVF_SYSCFG_SPCR1_SLVMII_PAD_EN			(1 << 11)
#define DVF_SYSCFG_SPCR1_LCDC_PAD_EN			(1 << 10)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD8_EN			(1 <<  8)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD7_EN			(1 <<  7)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD6_EN			(1 <<  6)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD5_EN			(1 <<  5)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD4_EN			(1 <<  4)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD3_EN			(1 <<  3)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD2_EN			(1 <<  2)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD1_EN			(1 <<  1)
#define DVF_SYSCFG_SPCR1_LCDGP_PAD0_EN			(1 <<  0)

#define DVF_SYSCFG_SPCR2_TRACEPORT_PAD_EN		(1 << 19)
#define DVF_SYSCFG_SPCR2_SDMMC_DATA_LOC			(1 << 18)
#define DVF_SYSCFG_SPCR2_EXT13_LOC				(1 << 17)
#define DVF_SYSCFG_SPCR2_EXT14_LOC				(1 << 16)
#define DVF_SYSCFG_SPCR2_MEMA25_EN				(1 << 14)
#define DVF_SYSCFG_SPCR2_MEMA24_EN				(1 << 13)
#define DVF_SYSCFG_SPCR2_MEMA23_EN				(1 << 12)
#define DVF_SYSCFG_SPCR2_MEMA22_EN				(1 << 11)
#define DVF_SYSCFG_SPCR2_MEMA21_EN				(1 << 10)
#define DVF_SYSCFG_SPCR2_MEMA18_EN				(1 <<  9)
#define DVF_SYSCFG_SPCR2_MEMA17_EN				(1 <<  8)
#define DVF_SYSCFG_SPCR2_DCLASS_PAD_EN			(1 <<  7)
#define DVF_SYSCFG_SPCR2_BMP_PAD_EN				(1 <<  6)
#define DVF_SYSCFG_SPCR2_USB_DRV_VBUS_PAD_EN	(1 <<  5)
#define DVF_SYSCFG_SPCR2_CLK_OUT_PAD_EN			(1 <<  4)
#define DVF_SYSCFG_SPCR2_CIU_PAD_EN				(1 <<  3)
#define DVF_SYSCFG_SPCR2_QSPI_PAD_EN			(1 <<  2)
#define DVF_SYSCFG_SPCR2_REF_CLK_PAD_EN			(1 <<  1)

#endif

