/*
 *  linux/arch/arm/mach-dvf101/arch.c
 *
 *  Copyright (C) 2016 DSPG Technologies GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <linux/delay.h>
#include <linux/export.h>
#include <linux/of.h>
#include <linux/of_address.h>
#include <linux/of_platform.h>
#include <linux/memblock.h>

#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/memblock.h>

#include <mach/hardware.h>

#include <asm/barrier.h>
#include <asm/compiler.h>
#include <asm/cmpxchg.h>
#include <asm/exec.h>
#include <asm/switch_to.h>
#include <asm/system_info.h>
#include <asm/system_misc.h>
#include <asm/system_misc.h>

static bool dvf_nodebug = false;

static __init int uart_locked_setup(char* str)
{
    get_option(&str, &uart_locked);
    return 0;
}
early_param("uart_locked", uart_locked_setup);

int dvf_get_chip_rev(void)
{
	return readl(IOMEM(IO_ADDRESS(DVF_SYSCFG_BASE)) + DVF_SYSCFG_CHIP_REV);
}
EXPORT_SYMBOL(dvf_get_chip_rev);

bool dvf_debug_disabled(void)
{
	return dvf_nodebug;
}

static int __init nodebug_setup(char *__unused)
{
	dvf_nodebug = true;

	return 1;
}
__setup("nodebug", nodebug_setup);

static struct map_desc dvf_io_desc[] __initdata = {
	{
		.virtual	= IO_ADDRESS(DVF_CMU_BASE),
		.pfn		= __phys_to_pfn(DVF_CMU_BASE),
		.length		= SZ_4K,
		.type		= MT_DEVICE
	}, {
		.virtual	= IO_ADDRESS(DVF_SYSCFG_BASE),
		.pfn		= __phys_to_pfn(DVF_SYSCFG_BASE),
		.length		= SZ_4K,
		.type		= MT_DEVICE
	},
};

static const char *dvf_dt_match[] = {
	"dspg,dvf101",
	NULL,
};

static void __init dvf_map_io(void)
{
	iotable_init(dvf_io_desc, ARRAY_SIZE(dvf_io_desc));
}

static void dvf_reset(enum reboot_mode mode, const char *cmd)
{
	unsigned int value;

	/* clear all previous reset indications */
	value = readl(IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_RST_CTRL));
	value |= (1<<16) | (1<<17) | (1<<19);
	writel(value, IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_RST_CTRL));

	switch (mode) {
	case REBOOT_WARM:
		writel(0xEBA7BFC1,
		       IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_WARM_RST));
		writel(0xC41359A7,
		       IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_WARM_RST));
		break;
	case REBOOT_SOFT:
	default:
		writel(0x7496398B,
		       IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_SW_RST));
		writel(0xB5C9C674,
		       IOMEM(IO_ADDRESS(DVF_CMU_BASE) + DVF_CMU_SW_RST));
		break;
	}
}

DT_MACHINE_START(DVF101, "DSPG DVF101")
	.map_io		= dvf_map_io,
	.l2c_aux_val	= 0x0,
	.l2c_aux_mask	= ~0x0,
	.dt_compat	= dvf_dt_match,
	.restart	= dvf_reset,
MACHINE_END
