/*
 *  linux/arch/arm/mach-dvf101/include/mach/hardware.h
 *
 *  This file contains the hardware definitions of the DVF101 base band family.
 *
 *  Copyright (C) 2016 DSPG
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef __MACH_HARDWARE_H
#define __MACH_HARDWARE_H

#include <asm/sizes.h>

/*
 * Base addresses
 */
#define DVF_GPIO_BASE				0x05000000
#define DVF_SYSCFG_BASE				0x05200000
#define DVF_CMU_BASE				0x05300000

/*
 * Get virtual address of a device for the static mappings. The virtual address
 * of these mappings is within the vmalloc region. Only the first 16MB starting
 * from DVF_GPIO_BASE can be used.
 */
#define IO_ADDRESS(a)        (0xfe000000 - DVF_GPIO_BASE + (a))

/*
 * Clock management unit
 */
#define DVF_CMU_RST_CTRL			0x0000
#define DVF_CMU_SW_RST				0x0004
#define DVF_CMU_WARM_RST			0x0008
#define DVF_CMU_OSC_CTRL			0x000C
#define DVF_CMU_PLL1_UNIT_CTRL			0x1000
#define DVF_CMU_PLL1_UNIT_RST_CNT		0x1004
#define DVF_CMU_PLL1_UNIT_LOCK_CNT		0x1008
#define DVF_CMU_PLL1_CFG1			0x100C
#define DVF_CMU_PLL1_CFG2			0x1010
#define DVF_CMU_PLL2_UNIT_CTRL			0x1014
#define DVF_CMU_PLL2_UNIT_RST_CNT		0x1018
#define DVF_CMU_PLL2_UNIT_LOCK_CNT		0x101C
#define DVF_CMU_PLL2_CFG1			0x1020
#define DVF_CMU_PLL2_CFG2			0x1024
#define DVF_CMU_PLL3_UNIT_CTRL			0x1028
#define DVF_CMU_PLL3_UNIT_RST_CNT		0x102C
#define DVF_CMU_PLL3_UNIT_LOCK_CNT		0x1030
#define DVF_CMU_PLL3_CFG1			0x1034
#define DVF_CMU_PLL3_CFG2			0x1038
#define DVF_CMU_PLL4_UNIT_CTRL			0x103C
#define DVF_CMU_PLL4_UNIT_RST_CNT		0x1040
#define DVF_CMU_PLL4_UNIT_LOCK_CNT		0x1044
#define DVF_CMU_PLL4_CFG1			0x1048
#define DVF_CMU_PLL4_CFG2			0x104C
#define DVF_CMU_MCU_CTRL			0x2000
#define DVF_CMU_MCU_DIV_VAL			0x2004
#define DVF_CMU_MCU_AXI_DIV_VAL			0x2008
#define DVF_CMU_CSS_CTRL			0x200C
#define DVF_CMU_CSS_DIV_VAL			0x2010
#define DVF_CMU_TDM1_CTRL			0x2014
#define DVF_CMU_TDM1_DIV_VAL			0x2018
#define DVF_CMU_TDM2_CTRL			0x201C
#define DVF_CMU_TDM2_DIV_VAL			0x2020
#define DVF_CMU_TDM3_CTRL			0x2024
#define DVF_CMU_TDM3_DIV_VAL			0x2028
#define DVF_CMU_UART1_CTRL			0x202C
#define DVF_CMU_UART2_CTRL			0x2030
#define DVF_CMU_UART3_CTRL			0x2034
#define DVF_CMU_UART4_CTRL			0x2038
#define DVF_CMU_NFC_CTRL			0x203C
#define DVF_CMU_BMP_CTRL			0x2040
#define DVF_CMU_BMP_DIV_VAL			0x2044
#define DVF_CMU_CS_CTRL				0x2048
#define DVF_CMU_CS_DIV_VAL			0x204C
#define DVF_CMU_DCLS_CTRL			0x2050
#define DVF_CMU_DCLS_DIV_VAL			0x2054
#define DVF_CMU_DAIF_CTRL			0x2058
#define DVF_CMU_LCDC_CTRL			0x205C
#define DVF_CMU_LCDC_DIV_VAL			0x2060
#define DVF_CMU_PWM_GEN_CTRL			0x2064
#define DVF_CMU_CRYPTO_CTRL			0x2068
#define DVF_CMU_MCU_GDMAC_CTRL			0x206C
#define DVF_CMU_CSS_GDMAC_CTRL			0x2070
#define DVF_CMU_SEC_CTRL			0x2074
#define DVF_CMU_SPI1_CTRL			0x2078
#define DVF_CMU_SPI1_DIV_VAL			0x207C
#define DVF_CMU_SPI2_CTRL			0x2080
#define DVF_CMU_SPI2_DIV_VAL			0x2084
#define DVF_CMU_SPI3_CTRL			0x2088
#define DVF_CMU_SPI3_DIV_VAL			0x208C
#define DVF_CMU_I2C1_CTRL			0x2090
#define DVF_CMU_I2C2_CTRL			0x2094
#define DVF_CMU_GPIO_CTRL			0x2098
#define DVF_CMU_RTC_CTRL			0x209C
#define DVF_CMU_RTC_DIV_VAL			0x20A0
#define DVF_CMU_WD_CTRL				0x20A4
#define DVF_CMU_WD_DIV_VAL			0x20A8
#define DVF_CMU_DRT_CTRL			0x20AC
#define DVF_CMU_ADPCM_CTRL			0x20B0
#define DVF_CMU_ADPCM_DIV_VAL			0x20B4
#define DVF_CMU_REFDIV_CTRL			0x20B8
#define DVF_CMU_REFDIV_DIV_VAL			0x20BC
#define DVF_CMU_MCU_TIMER12_CTRL		0x20C0
#define DVF_CMU_MCU_TIMER34_CTRL		0x20C4
#define DVF_CMU_CSS_TIMER12_CTRL		0x20C8
#define DVF_CMU_APU_CTRL			0x20CC
#define DVF_CMU_APU_DIV_VAL			0x20D0
#define DVF_CMU_CCU_CTRL			0x20D4
#define DVF_CMU_GPU_CTRL			0x20D8
#define DVF_CMU_GPU_DIV_VAL			0x20DC
#define DVF_CMU_ETH_CTRL			0x20E0
#define DVF_CMU_ETH_DIV_VAL			0x20E4
#define DVF_CMU_DDR3_CTRL			0x20E8
#define DVF_CMU_SDIO_CTRL			0x20EC
#define DVF_CMU_SDIO_DIV_VAL			0x20F0
#define DVF_CMU_EMMC_CTRL			0x20F4
#define DVF_CMU_EMMC_DIV_VAL			0x20F8
#define DVF_CMU_CLK_OUT_CTRL			0x20FC
#define DVF_CMU_CLK_OUT_DIV_VAL			0x2100
#define DVF_CMU_DSI_CTRL			0x2104
#define DVF_CMU_USB_PHY_CTRL			0x2108
#define DVF_CMU_USB_PHY_DIV_VAL			0x210C
#define DVF_CMU_USB_OTG_CTRL			0x2110
#define DVF_CMU_USB_HOST_CTRL			0x2114

/*
 * System Configuration
 */
#define DVF_SYSCFG_GCR0				0x00
#define DVF_SYSCFG_GCR1				0x04
#define DVF_SYSCFG_HOSTCPU_IR			0x0C
#define DVF_SYSCFG_CHIP_ID			0x1C
#define DVF_SYSCFG_CHIP_REV			0x20
#define DVF_SYSCFG_TEST_CTRL_EN			0x40
#define DVF_SYSCFG_TEST_CTRL1			0x44
#define DVF_SYSCFG_CA9_CFG			0x48
#define DVF_SYSCFG_GPU_IDLE_CTRL		0x4C
#define DVF_SYSCFG_DSI_CTRL			0x50
#define DVF_SYSCFG_USB_CSR			0x58
#define DVF_SYSCFG_USB_P1S			0x5C
#define DVF_SYSCFG_USB_P2S			0x60
#define DVF_SYSCFG_USB_CTRL			0x64
#define DVF_SYSCFG_DDR_GCR1			0x6C
#define DVF_SYSCFG_DDR_GCR2			0x70
#define DVF_SYSCFG_DDR_GCR3			0x74
#define DVF_SYSCFG_DDR_BSD1_CTRL		0xA0
#define DVF_SYSCFG_DDR_BSD2_CTRL		0xA4
#define DVF_SYSCFG_DDR_BSD3_CTRL		0xA8
#define DVF_SYSCFG_DDR_BSD4_CTRL		0xAC
#define DVF_SYSCFG_DDR_BSD5_CTRL		0xB0
#define DVF_SYSCFG_DDR_BSD6_CTRL		0xB4
#define DVF_SYSCFG_DDR_BSD7_CTRL		0xB8
#define DVF_SYSCFG_DDR_BSD8_CTRL		0xBC
#define DVF_SYSCFG_DDR_BSD9_CTRL		0xC0
#define DVF_SYSCFG_USB_PHY1_CTRL1		0x80
#define DVF_SYSCFG_USB_PHY1_CTRL2		0x84
#define DVF_SYSCFG_USB_PHY2_CTRL1		0x88
#define DVF_SYSCFG_USB_PHY2_CTRL2		0x8C
#define DVF_SYSCFG_IOM1				0x200
#define DVF_SYSCFG_IOM2				0x204
#define DVF_SYSCFG_IOM3				0x208
#define DVF_SYSCFG_IOM4				0x20C
#define DVF_SYSCFG_IOM5				0x210
#define DVF_SYSCFG_IOM6				0x214
#define DVF_SYSCFG_IOM7				0x218
#define DVF_SYSCFG_IOM8				0x21C
#define DVF_SYSCFG_IOM9				0x220
#define DVF_SYSCFG_IOM10			0x224
#define DVF_SYSCFG_IOM11			0x228
#define DVF_SYSCFG_IODS1			0x22C
#define DVF_SYSCFG_IODS2			0x230
#define DVF_SYSCFG_IODS3			0x234
#define DVF_SYSCFG_IODS4			0x238
#define DVF_SYSCFG_IODS5			0x23C
#define DVF_SYSCFG_IODS6			0x240
#define DVF_SYSCFG_IODS7			0x244
#define DVF_SYSCFG_IODS8			0x248
#define DVF_SYSCFG_IODS9			0x24C
#define DVF_SYSCFG_IODS10			0x250
#define DVF_SYSCFG_IODS11			0x254
#define DVF_SYSCFG_IOSR1			0x258
#define DVF_SYSCFG_IOSR2			0x25C
#define DVF_SYSCFG_IOSR3			0x260
#define DVF_SYSCFG_IOSR4			0x264
#define DVF_SYSCFG_IOSR5			0x268
#define DVF_SYSCFG_IOSR6			0x26C
#define DVF_SYSCFG_IOKE1			0x270
#define DVF_SYSCFG_IOKE2			0x274
#define DVF_SYSCFG_IOKE3			0x278
#define DVF_SYSCFG_IOKE4			0x27C
#define DVF_SYSCFG_IOKE5			0x280
#define DVF_SYSCFG_IOKE6			0x284
#define DVF_SYSCFG_IOPE1			0x288
#define DVF_SYSCFG_IOPE2			0x28C
#define DVF_SYSCFG_IOPE3			0x290
#define DVF_SYSCFG_IOPE4			0x294
#define DVF_SYSCFG_IOPE5			0x298
#define DVF_SYSCFG_IOPE6			0x29C
#define DVF_SYSCFG_IOPV1			0x2A0
#define DVF_SYSCFG_IOPV2			0x2A4
#define DVF_SYSCFG_IOPV3			0x2A8
#define DVF_SYSCFG_IOPV4			0x2AC
#define DVF_SYSCFG_IOPV5			0x2B0
#define DVF_SYSCFG_IOPV6			0x2B4
#define DVF_SYSCFG_IOIE1			0x2B8
#define DVF_SYSCFG_IOIE2			0x2BC
#define DVF_SYSCFG_IOIE3			0x2C0
#define DVF_SYSCFG_IOIE4			0x2C4
#define DVF_SYSCFG_IOIE5			0x2C8
#define DVF_SYSCFG_IOIE6			0x2CC

#endif

