/*
 *  Copyright (C) 2016 DSPG Technologies GmbH
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Common Clock Framework support for DSPG platforms
 */
#include <linux/of.h>
#include <linux/clk-provider.h>

int of_dspg_get_mux_flags(struct device_node *node, u8 *flags)
{
	if (of_find_property(node, "mux-index-one", NULL))
		*flags |= CLK_MUX_INDEX_ONE;
	if (of_find_property(node, "mux-index-bit", NULL))
		*flags |= CLK_MUX_INDEX_BIT;
	if (of_find_property(node, "mux-hiword-mask", NULL))
		*flags |= CLK_MUX_HIWORD_MASK;
	if (of_find_property(node, "mux-read-only", NULL))
		*flags |= CLK_MUX_READ_ONLY;
	if (of_find_property(node, "mux-round-closest", NULL))
		*flags |= CLK_MUX_ROUND_CLOSEST;
	return 0;
}

int of_dspg_get_div_flags(struct device_node *node, u8 *flags)
{
	if (of_find_property(node, "div-one-based", NULL))
		*flags |= CLK_DIVIDER_ONE_BASED;
	if (of_find_property(node, "div-power-of-two", NULL))
		*flags |= CLK_DIVIDER_POWER_OF_TWO;
	if (of_find_property(node, "div-allow-zero", NULL))
		*flags |= CLK_DIVIDER_ALLOW_ZERO;
	if (of_find_property(node, "div-hiword-mask", NULL))
		*flags |= CLK_DIVIDER_HIWORD_MASK;
	if (of_find_property(node, "div-round-closest", NULL))
		*flags |= CLK_DIVIDER_ROUND_CLOSEST;
	if (of_find_property(node, "div-read-only", NULL))
		*flags |= CLK_DIVIDER_READ_ONLY;
	return 0;
}

int of_dspg_get_gate_flags(struct device_node *node, u8 *flags)
{
	if (of_find_property(node, "gate-set-to-disable", NULL))
		*flags |= CLK_GATE_SET_TO_DISABLE;
	if (of_find_property(node, "gate-hiword-mask", NULL))
		*flags |= CLK_GATE_HIWORD_MASK;
	return 0;
}

int of_dspg_get_clk_flags(struct device_node *node, unsigned long *flags)
{
	if (of_find_property(node, "clk-set-rate-gate", NULL))
		*flags |= CLK_SET_RATE_GATE;
	if (of_find_property(node, "clk-set-parent-gate", NULL))
		*flags |= CLK_SET_PARENT_GATE;
	if (of_find_property(node, "clk-set-rate-parent", NULL))
		*flags |= CLK_SET_RATE_PARENT;
	if (of_find_property(node, "clk-ignore-unused", NULL))
		*flags |= CLK_IGNORE_UNUSED;
	if (of_find_property(node, "clk-is-basic", NULL))
		*flags |= CLK_IS_BASIC;
	if (of_find_property(node, "clk-get-rate-nocache", NULL))
		*flags |= CLK_GET_RATE_NOCACHE;
	if (of_find_property(node, "clk-set-rate-no-reparent", NULL))
		*flags |= CLK_SET_RATE_NO_REPARENT;
	if (of_find_property(node, "clk-get-accuracy-nocache", NULL))
		*flags |= CLK_GET_ACCURACY_NOCACHE;
	return 0;
}
