/*
* Copyright c                  Realtek Semiconductor Corporation, 2006
* All rights reserved.
*
* Program : Control smi connected RTL8366
* Abstract :
* Author : Yu-Mei Pan (ympan@realtek.com.cn)
*  $Id: smi.c,v 1.2 2008-04-10 03:04:19 shiehyy Exp $
*/
#include "rtk_types.h"
#include "rtk_error.h"
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/mutex.h>
#include "smi.h"

#define MDC_MDIO_CTRL0_REG          31
#define MDC_MDIO_START_REG          29
#define MDC_MDIO_CTRL1_REG          21
#define MDC_MDIO_ADDRESS_REG        23
#define MDC_MDIO_DATA_WRITE_REG     24
#define MDC_MDIO_DATA_READ_REG      25
#define MDC_MDIO_PREAMBLE_LEN       32

#define MDC_MDIO_START_OP          0xFFFF
#define MDC_MDIO_ADDR_OP           0x000E
#define MDC_MDIO_READ_OP           0x0001
#define MDC_MDIO_WRITE_OP          0x0003

static struct mii_bus *g_mbus = NULL;
static struct mutex mlock;
static int mlock_init;
static smi_mdio_read stmmac_mdio_read = NULL;
static smi_mdio_write stmmac_mdio_write = NULL;

rtk_int32 smi_read(rtk_uint32 mAddrs, rtk_uint32 *rData);
rtk_int32 smi_write(rtk_uint32 mAddrs, rtk_uint32 rData);

/*******************************************************************************/
/*  MDC/MDIO porting                                                           */
/*******************************************************************************/
/* define the PHY ID currently used */
#define MDC_MDIO_PHY_ID     29  /* PHY ID 0 or 29 */

/* MDC/MDIO, redefine/implement the following Macro */
#define MDC_MDIO_WRITE(preamableLength, phyID, regID, data)
#define MDC_MDIO_READ(preamableLength, phyID, regID, pData)

static void rtlglue_drvMutexLock(void)
{
	if (!mlock_init) {
		mlock_init = 1;
		mutex_init(&mlock);
	}
	mutex_lock(&mlock);
	return;
}

static void rtlglue_drvMutexUnlock(void)
{
	if (!mlock_init) {
		mlock_init = 1;
		mutex_init(&mlock);
	}
	mutex_unlock(&mlock);
	return;
}

int rtl_init_mii_bus(struct mii_bus *bus, smi_mdio_read read, smi_mdio_write write)
{
	g_mbus = bus;
	stmmac_mdio_read = read;
	stmmac_mdio_write = write;
	return 0;
}

rtk_int32 smi_read(rtk_uint32 mAddrs, rtk_uint32 *rData)
{
	if (mAddrs > 0xFFFF)
		return RT_ERR_INPUT;

	if (rData == NULL)
		return RT_ERR_NULL_POINTER;

	if (!g_mbus) {
		printk("smi_read RT_ERR_NULL_POINTER\n");
		return RT_ERR_NULL_POINTER;
	}

	/* Lock */
	rtlglue_drvMutexLock();

	/* Write address control code to register 31 */
	stmmac_mdio_write(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_CTRL0_REG, MDC_MDIO_ADDR_OP);

	/* Write address to register 23 */
	stmmac_mdio_write(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_ADDRESS_REG, mAddrs);

	/* Write read control code to register 21 */
	stmmac_mdio_write(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_CTRL1_REG, MDC_MDIO_READ_OP);

	/* Read data from register 25 */
	*rData = stmmac_mdio_read(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_DATA_READ_REG);

	/* Unlock */
	rtlglue_drvMutexUnlock();

	return RT_ERR_OK;
}

rtk_int32 smi_write(rtk_uint32 mAddrs, rtk_uint32 rData)
{
	if (mAddrs > 0xFFFF)
		return RT_ERR_INPUT;

	if (rData > 0xFFFF)
		return RT_ERR_INPUT;

	if (!g_mbus) {
		printk("smi_write RT_ERR_NULL_POINTER\n");
		return RT_ERR_NULL_POINTER;
	}

	/* Lock */
	rtlglue_drvMutexLock();

	/* Write address control code to register 31 */
	stmmac_mdio_write(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_CTRL0_REG, MDC_MDIO_ADDR_OP);

	/* Write address to register 23 */
	stmmac_mdio_write(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_ADDRESS_REG, mAddrs);

	/* Write data to register 24 */
	stmmac_mdio_write(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_DATA_WRITE_REG, rData);

	/* Write data control code to register 21 */
	stmmac_mdio_write(g_mbus, MDC_MDIO_PHY_ID, MDC_MDIO_CTRL1_REG, MDC_MDIO_WRITE_OP);

	/* Unlock */
	rtlglue_drvMutexUnlock();

	return RT_ERR_OK;
}
