/*
 * include/linux/coma/coma.h - the cordless manager
 *
 * The cordless manager is responsible for maintaining the main cfifo between
 * Linux and cordless which is used for loading and unloading cordless and for
 * registration of other services (e.g., debug output, VoIP/RTP, etc.)
 *
 * Copyright (C) 2007 NXP Semiconductors
 * Copyright (C) 2008 - 2012 DSP Group Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _LINUX_COMA_COMA_H
#define _LINUX_COMA_COMA_H

#include <linux/types.h>
#include <uapi/linux/coma/coma.h>

struct device;

typedef void (*coma_process_cb_t)(void *arg, struct cmsg *cmsg);
typedef void (*coma_remove_cb_t)(void *arg);
typedef int  (*coma_suspend_cb_t)(void);

/*
 * Register a service with cordless in order to receive messages from and
 * send messages to the cordless domain.
 */
int coma_register(const char *name, unsigned int cfifo_size, void *arg,
		  coma_process_cb_t process_message, coma_remove_cb_t remove,
		  coma_suspend_cb_t suspend);

/*
 * De-register the services (e.g., when unloading the corresponding kernel
 * module).
 */
int coma_deregister(unsigned int service);

/*
 * Data is send as cordless messages (cmsg) through the services. The following
 * functions deal with them...
 */
struct cmsg *coma_cmsg_alloc(unsigned int service, int type,
			     unsigned int params_size,
			     unsigned int payload_size);

int coma_cmsg_commit(unsigned int service);
void coma_cmsg_abort(unsigned int service);

int coma_cmsg_send(unsigned int service, int type,
		   void *params, unsigned int params_size,
		   void *payload, unsigned int payload_size);

static inline int cmsg_type(struct cmsg *cmsg)
{
	return cmsg->type;
}

static inline void *cmsg_params(struct cmsg *cmsg)
{
	return cmsg->body;
}

static inline void *cmsg_payload(struct cmsg *cmsg)
{
	return cmsg->body + cmsg->params_size;
}

/*
 * Coma subsystem (de-)initialization.
 */

int coma_setup(struct device *dev, dma_addr_t *l2c_phys, dma_addr_t *c2l_phys);
void coma_release(struct device *dev);
int coma_suspend(void);

#endif /* _LINUX_COMA_COMA_H */
