/*
 * linux/include/linux/mfd/dspg-crypto.h - DSPG cryptographic security module
 *
 *  Copyright (C) 2013 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA
 *
 */

#ifndef __LINUX_MFD_DSPG_CRYPTO_H
#define __LINUX_MFD_DSPG_CRYPTO_H

#define NUM_OF_TRNG 8

/* Descriptor control: crypto core modes */
enum dspg_crypto_mode {
	DESC_CTL_MODE_AES_CTR_ENCRYPT	= 0x10,
	DESC_CTL_MODE_AES_CTR_DECRYPT	= 0x11,
	DESC_CTL_MODE_AES_CBC_ENCRYPT	= 0x12,
	DESC_CTL_MODE_AES_CBC_DECRYPT	= 0x13,
	DESC_CTL_MODE_AES_ECB_ENCRYPT	= 0x14,
	DESC_CTL_MODE_AES_ECB_DECRYPT	= 0x15,
	DESC_CTL_MODE_AES_F8_ENCRYPT	= 0x16,
	DESC_CTL_MODE_AES_F8_DECRYPT	= 0x17,
	DESC_CTL_MODE_AES_GCM_ENCRYPT	= 0x18,
	DESC_CTL_MODE_AES_GCM_DECRYPT	= 0x19,
	DESC_CTL_MODE_SHA1_160		= 0x20,
	DESC_CTL_MODE_SHA2_224		= 0x21,
	DESC_CTL_MODE_SHA2_256		= 0x22,
	DESC_CTL_MODE_MD5		= 0x23,
	DESC_CTL_MODE_SHA1_160_HMAC	= 0x30,
	DESC_CTL_MODE_SHA2_224_HMAC	= 0x31,
	DESC_CTL_MODE_SHA2_256_HMAC	= 0x32,
	DESC_CTL_MODE_MD5_HMAC		= 0x33,
	DESC_CTL_MODE_3DES_CBC_ENCRYPT	= 0x40,
	DESC_CTL_MODE_3DES_CBC_DECRYPT	= 0x41,
	DESC_CTL_MODE_DES_CBC_ENCRYPT	= 0x42,
	DESC_CTL_MODE_DES_CBC_DECRYPT	= 0x43,
	DESC_CTL_MODE_3DES_ECB_ENCRYPT	= 0x44,
	DESC_CTL_MODE_3DES_ECB_DECRYPT	= 0x45,
	DESC_CTL_MODE_DES_ECB_ENCRYPT	= 0x46,
	DESC_CTL_MODE_DES_ECB_DECRYPT	= 0x47,
	DESC_CTL_MODE_RC4_ENCRYPT	= 0x60,
	DESC_CTL_MODE_RC4_DECRYPT	= 0x61,
	DESC_CTL_MODE_DMA_LOOPBACK	= 0x80,
};

typedef void (*dspg_crypto_callback)(int status, void *context);

struct hw_crypto_desc;

struct crypto_op {
	struct hw_crypto_desc *hw_desc;
	dma_addr_t hw_desc_phys;

	u32 channel;
	void *data_in_virt;  /* 32-bit pointer to input data */
	void *data_out_virt; /* 32-bit pointer to output data */

	void *iv_virt;
	dspg_crypto_callback done;
	void *context;
	struct cryptop *dspg_crp;
};

struct crypto_op *
dspg_crypto_alloc_operation(void *in, size_t in_len, void *out, size_t out_len,
			    u8 mode, int channel, u8 *key, size_t key_len,
			    void *iv, void *a, size_t a_len, size_t c_len,
			    int secret);
void dspg_crypto_free_operation(struct crypto_op *op);
int dspg_crypto_submit_operation(struct crypto_op *op);
void dspg_crypto_generate_trng(void);
int dspg_crypto_get_trng(unsigned int *buf);

#endif
