/*
 * include/uapi/linux/coma/cfifo.h - cordless FIFOs
 *
 * Cordless FIFOs store variable-length messages in a ring buffer. This FIFO
 * implementation is safe for parallel accesses of not more than one reader and
 * one writer. All messages are stored in a consecutive memory area, i.e.,
 * they are never wrapped around at the end of the FIFO as in many other ring
 * buffer implementations.
 *
 * Copyright (C) 2007 NXP Semiconductors
 * Copyright (C) 2008 - 2012 DSP Group Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _UAPI_LINUX_COMA_CFIFO_H
#define _UAPI_LINUX_COMA_CFIFO_H

#define CFIFO_MSGHEADER_SIZE (sizeof(unsigned int))
#define CFIFO_PAD(x)         (((x) + (CFIFO_MSGHEADER_SIZE-1)) & \
                              ~(CFIFO_MSGHEADER_SIZE-1))

#define CFIFO_MAGIC (((unsigned)'C' << 0) | ((unsigned)'F' << 8) | \
			((unsigned)'I' << 16) | ((unsigned)'F' << 24))

/*
 * CFIFO shared data structure between Linux and Cordless domains.
 */
struct cfifo_shm {
	unsigned int magic;	/* CFIFO magic 'CFIF' */
	unsigned int size;	/* the size of the allocated buffer */
	unsigned int in;	/* data is added at offset in */
	unsigned int out;	/* data is extracted from offset out */

	char buffer[0];
};

#endif /* _UAPI_LINUX_COMA_CFIFO_H */
