/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/hardware.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/dvf9a.h>

static int dvf99_detect_chip(void)
{
	int chip = DVF99_CHIP_UNKNOWN;

	switch (readl(DVF99_SYSCFG_BASE + DVF99_SYSCFG_CHIP_ID)) {
	case 0x9700:
		chip = DVF99_CHIP_DVF97;
		break;
	case 0x9910:
		chip = DVF99_CHIP_DVF99;
		break;
	}

	return chip;
}

int
is_dvf97(void)
{
	return (readl(DVF99_SYSCFG_BASE + DVF99_SYSCFG_CHIP_ID) == 0x9700);
}

#ifdef CONFIG_RECOVERY
int is_recovery(void){
	return !strcmp("recovery", (char *)(DVF99_INTRAM_BASE + 0x10000 -
					    DVF99_RECOVERY_INFOSIZE));
}
#endif

int print_cpuinfo(void)
{
	int dvfd9_version, dvfa9_version;
	char str[32];

	dvfd9_version = readl(DVF99_SYSCFG_BASE + DVF99_SYSCFG_CHIP_REV);
	dvfa9_version = daif_read(DAIF_HW_REVISION);

	puts("CPU:   ");

	switch (dvf99_detect_chip()) {
	case DVF99_CHIP_DVF99:
		puts("DVF99 (");
		sprintf(str, "rev %d/%d: ", dvfd9_version, dvfa9_version);
		puts(str);
		switch (dvfd9_version) {
		case 0:
			if (dvfa9_version == 1)
				puts("YBC");
			else
				puts("WBC-U/ABC");
			break;
		case 1:
			if (dvfa9_version == 1)
				puts("XBC");
			else
				puts("WBC");
			break;
		case 16:
			puts("BBC");
			break;
		default:
			puts("unknown?");
			break;
		}
		puts("): ");
		sprintf(str, "%lu MHz (CSS@%lu, PCLK@%lu)",
		        dvf99_get_cpufreq() / 1000000,
		        dvf99_get_cssfreq() / 1000000,
			dvf99_get_hclk() / 1000000);
		puts(str);
		puts("\n");
		break;
	case DVF99_CHIP_DVF97:
		puts("DVF97 (");
		sprintf(str, "rev %d/%d", dvfd9_version, dvfa9_version);
		puts(str);
		puts("): ");
		sprintf(str, "%lu MHz (CSS@%lu, PCLK@%lu)",
		        dvf99_get_cpufreq() / 1000000,
		        dvf99_get_cssfreq() / 1000000,
			dvf99_get_hclk() / 1000000);
		puts(str);
		puts("\n");
		break;
	default: puts("unknown\n");
	}

	return 0;
}

/*
 * Write the CMU S/W reset register to cause reset
 */
void reset_cpu(unsigned long addr)
{
	uint32_t reg;

	/* clear SWSYSRSTIND and SWWRMRSTIND before SW resetting the CPU */
	reg = readl(DVF99_CMU_BASE + DVF99_CMU_RSTSR);
	reg |= ((1<<5) | (1<<4));
	writel(reg, DVF99_CMU_BASE + DVF99_CMU_RSTSR);

	writel(0x7496398b, DVF99_CMU_BASE + DVF99_CMU_SWSYSRST);
	writel(0xb5c9c674, DVF99_CMU_BASE + DVF99_CMU_SWSYSRST);
}

#ifdef CONFIG_ARCH_CPU_INIT
int arch_cpu_init(void)
{
	clk_init();
	dvf9a_init();

	return 0;
}
#endif /* CONFIG_ARCH_CPU_INIT */
