/*
 * (C) Copyright 2015 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/io.h>
#include <asm/arch/hardware.h>

#define IOM1 0x200
#define IOM2 0x204
#define IOM3 0x208
#define IOM4 0x20C
#define IOM5 0x210
#define IOM6 0x214
#define IOM7 0x218

struct iomux {
	char *name;
	int reg;
	int shift;
	int value;
};

struct io {
	char *name;
	int reg;
	int bit;
};

#define __stringify_1(x)	#x
#define __stringify(x)		__stringify_1(x)

#define IOMUX(_name, _reg, _shift, _val)      \
	static struct iomux iomux_##_name = { \
		.name = __stringify(_name),   \
		.reg = _reg,                  \
		.shift = _shift,              \
		.value = _val,                \
	}

IOMUX(gpio0,     IOM5, 20, 1);
IOMUX(gpio1,     IOM5, 18, 0);
IOMUX(gpio2,     IOM5, 16, 0);
IOMUX(gpio3,     IOM4,  4, 2);
IOMUX(gpio4,     IOM3,  0, 0);
IOMUX(gpio5,     IOM6,  6, 2);
IOMUX(gpio6,     IOM6,  8, 1);
IOMUX(gpio7,     IOM5, 28, 1);
IOMUX(gpio8,     IOM5, 22, 1);
IOMUX(gpio9,     IOM4,  0, 1);
IOMUX(gpio10,    IOM4,  2, 1);
IOMUX(gpio11,    IOM4, 20, 3);
IOMUX(gpio12,    IOM4, 18, 2);
IOMUX(gpio13,    IOM4, 16, 2);
IOMUX(gpio14,    IOM4, 14, 2);
IOMUX(gpio15,    IOM4, 12, 2);
IOMUX(gpio16,    IOM4, 10, 2);
IOMUX(gpio17,    IOM4,  8, 2);
IOMUX(gpio18,    IOM4,  6, 2);
IOMUX(gpio19,    IOM5,  6, 1);
IOMUX(gpio20,    IOM5,  4, 1);
IOMUX(gpio21,    IOM5,  2, 1);
IOMUX(gpio22,    IOM5,  0, 1);
IOMUX(gpio23,    IOM4, 28, 0);
IOMUX(gpio24,    IOM4, 26, 0);
IOMUX(gpio25,    IOM4, 24, 0);
IOMUX(gpio26,    IOM4, 22, 3);
IOMUX(gpio27,    IOM5, 14, 2);
IOMUX(gpio28,    IOM5, 12, 2);
IOMUX(gpio29,    IOM5, 10, 1);
IOMUX(gpio30,    IOM5,  8, 2);

IOMUX(u1tx,      IOM5,  6, 0);
IOMUX(u1rx,      IOM5,  4, 0);
IOMUX(u2tx,      IOM5,  2, 0);
IOMUX(u2rx,      IOM5,  0, 0);

IOMUX(iicscl,    IOM4,  2, 0);
IOMUX(iicsda,    IOM4,  0, 0);

IOMUX(spido1,    IOM5, 14, 0);
IOMUX(spido2,    IOM5, 14, 1);
IOMUX(spidi1,    IOM5, 12, 0);
IOMUX(spidi2,    IOM5, 12, 1);
IOMUX(spics,     IOM5, 10, 0);
IOMUX(spick1,    IOM5,  8, 0);
IOMUX(spick2,    IOM5,  8, 1);

IOMUX(key_col0,  IOM4,  8, 1);
IOMUX(key_col1,  IOM4,  6, 1);
IOMUX(key_col2,  IOM4,  4, 1);
IOMUX(key_col3,  IOM4, 22, 1);
IOMUX(key_col4,  IOM4, 24, 1);
IOMUX(key_col5,  IOM4, 26, 1);

IOMUX(key_row0,  IOM4, 20, 1);
IOMUX(key_row1,  IOM4, 18, 1);
IOMUX(key_row2,  IOM4, 16, 1);
IOMUX(key_row3,  IOM4, 14, 1);
IOMUX(key_row4,  IOM4, 12, 1);
IOMUX(key_row5,  IOM4, 10, 1);

IOMUX(tri_kpd0,  IOM4,  4, 0);
IOMUX(tri_kpd1,  IOM4,  6, 0);
IOMUX(tri_kpd2,  IOM4,  8, 0);
IOMUX(tri_kpd3,  IOM4, 10, 0);
IOMUX(tri_kpd4,  IOM4, 12, 0);
IOMUX(tri_kpd5,  IOM4, 14, 0);
IOMUX(tri_kpd6,  IOM4, 16, 0);
IOMUX(tri_kpd7,  IOM4, 18, 0);
IOMUX(tri_kpd8,  IOM4, 20, 0);
IOMUX(tri_kpd9,  IOM4, 22, 0);

IOMUX(tdm_fsync, IOM4, 26, 2);
IOMUX(tdm_tx,    IOM4, 24, 2);
IOMUX(tdm_rx,    IOM4, 22, 2);
IOMUX(tdm_sclk,  IOM4, 20, 2);

IOMUX(mdio,      IOM5, 22, 0);

IOMUX(main_clk,  IOM5, 20, 0);

IOMUX(syncport,  IOM3,  0, 1);

static struct iomux *iomuxes[] = {
	&iomux_gpio0,
	&iomux_gpio1,
	&iomux_gpio2,
	&iomux_gpio3,
	&iomux_gpio4,
	&iomux_gpio5,
	&iomux_gpio6,
	&iomux_gpio7,
	&iomux_gpio8,
	&iomux_gpio9,
	&iomux_gpio10,
	&iomux_gpio11,
	&iomux_gpio12,
	&iomux_gpio13,
	&iomux_gpio14,
	&iomux_gpio15,
	&iomux_gpio16,
	&iomux_gpio17,
	&iomux_gpio18,
	&iomux_gpio19,
	&iomux_gpio20,
	&iomux_gpio21,
	&iomux_gpio22,
	&iomux_gpio23,
	&iomux_gpio24,
	&iomux_gpio25,
	&iomux_gpio26,
	&iomux_gpio27,
	&iomux_gpio28,
	&iomux_gpio29,
	&iomux_gpio30,

	&iomux_key_col0,
	&iomux_key_col1,
	&iomux_key_col2,
	&iomux_key_col3,
	&iomux_key_col4,
	&iomux_key_col5,

	&iomux_key_row0,
	&iomux_key_row1,
	&iomux_key_row2,
	&iomux_key_row3,
	&iomux_key_row4,
	&iomux_key_row5,

	&iomux_u1tx,
	&iomux_u1rx,
	&iomux_u2tx,
	&iomux_u2rx,

	&iomux_spido1,
	&iomux_spido2,
	&iomux_spidi1,
	&iomux_spidi2,
	&iomux_spics,
	&iomux_spick1,
	&iomux_spick2,

	&iomux_tri_kpd0,
	&iomux_tri_kpd1,
	&iomux_tri_kpd2,
	&iomux_tri_kpd3,
	&iomux_tri_kpd4,
	&iomux_tri_kpd5,
	&iomux_tri_kpd6,
	&iomux_tri_kpd7,
	&iomux_tri_kpd8,
	&iomux_tri_kpd9,

	&iomux_tdm_fsync,
	&iomux_tdm_tx,
	&iomux_tdm_rx,
	&iomux_tdm_sclk,

	&iomux_iicscl,
	&iomux_iicsda,

	&iomux_mdio,

	&iomux_main_clk,

	&iomux_syncport,
};

void
io_mux(const char *id)
{
	int i, tmp;
	struct iomux *mux = NULL;

	for (i = 0; i < ARRAY_SIZE(iomuxes); i++) {
		if (!strcmp(id, iomuxes[i]->name)) {
			mux = iomuxes[i];
			break;
		}
	}

	if (!mux)
		return;

	tmp = readl(DVF97_SYSCFG_BASE + mux->reg);
	tmp &= ~(3 << mux->shift);
	tmp |= mux->value << mux->shift;
	writel(tmp, DVF97_SYSCFG_BASE + mux->reg);
}

#define IOREG1 0x0
#define IOREG2 0x4
#define IOREG3 0x8
#define IOREG4 0xc

#define IODS_OFFSET  0x21C
#define IOSR_OFFSET  0x238
#define IOKE_OFFSET  0x248
#define IOPE_OFFSET  0x258
#define IOV_OFFSET   0x268
#define IOSMT_OFFSET 0x278

#define IO(_name, _reg, _bit)               \
	static struct io io_##_name = {     \
		.name = __stringify(_name), \
		.reg = _reg,                \
		.bit = _bit,                \
	}

IO(gpio0,     IOREG3, 26);
IO(gpio1,     IOREG3, 25);
IO(gpio2,     IOREG3, 24);
IO(gpio3,     IOREG2,  2);
IO(gpio4,     IOREG2, 16);
IO(gpio5,     IOREG3,  3);
IO(gpio6,     IOREG3,  4);
IO(gpio7,     IOREG3, 30);
IO(gpio8,     IOREG3, 27);
IO(gpio9,     IOREG2,  0);
IO(gpio10,    IOREG2,  1);
IO(gpio11,    IOREG2, 10);
IO(gpio12,    IOREG2,  9);
IO(gpio13,    IOREG2,  8);
IO(gpio14,    IOREG2,  7);
IO(gpio15,    IOREG2,  6);
IO(gpio16,    IOREG2,  5);
IO(gpio17,    IOREG2,  4);
IO(gpio18,    IOREG2,  3);
IO(gpio19,    IOREG3, 19);
IO(gpio20,    IOREG3, 18);
IO(gpio21,    IOREG3, 17);
IO(gpio22,    IOREG3, 16);
IO(gpio23,    IOREG2, 14);
IO(gpio24,    IOREG2, 13);
IO(gpio25,    IOREG2, 12);
IO(gpio26,    IOREG2, 11);
IO(gpio27,    IOREG3, 23);
IO(gpio28,    IOREG3, 22);
IO(gpio29,    IOREG3, 21);
IO(gpio30,    IOREG3, 20);

IO(qd0,       IOREG3,  5);
IO(qd1,       IOREG3,  6);
IO(qd2,       IOREG3,  7);
IO(qd3,       IOREG3,  8);
IO(qclk,      IOREG3,  9);
IO(qcs,       IOREG3, 10);

IO(key_col_0, IOREG2,  4);
IO(key_col_1, IOREG2,  3);
IO(key_col_2, IOREG2,  2);
IO(key_col_3, IOREG2, 11);
IO(key_col_4, IOREG2, 12);
IO(key_col_5, IOREG2, 13);
IO(key_row_0, IOREG2, 10);
IO(key_row_1, IOREG2,  9);
IO(key_row_2, IOREG2,  8);
IO(key_row_3, IOREG2,  7);
IO(key_row_4, IOREG2,  6);
IO(key_row_5, IOREG2,  5);

IO(iicscl,    IOREG2,  1);
IO(iicsda,    IOREG2,  0);

IO(u1tx,      IOREG3, 19);
IO(u1rx,      IOREG3, 18);
IO(u2tx,      IOREG3, 17);
IO(u2rx,      IOREG3, 16);

IO(spido,     IOREG3, 23);
IO(spidi,     IOREG3, 22);
IO(spics,     IOREG3, 21);
IO(spick,     IOREG3, 20);

static struct io *ios[] = {
	&io_gpio0,
	&io_gpio1,
	&io_gpio2,
	&io_gpio3,
	&io_gpio4,
	&io_gpio5,
	&io_gpio6,
	&io_gpio7,
	&io_gpio8,
	&io_gpio9,
	&io_gpio10,
	&io_gpio11,
	&io_gpio12,
	&io_gpio13,
	&io_gpio14,
	&io_gpio15,
	&io_gpio16,
	&io_gpio17,
	&io_gpio18,
	&io_gpio19,
	&io_gpio20,
	&io_gpio21,
	&io_gpio22,
	&io_gpio23,
	&io_gpio24,
	&io_gpio25,
	&io_gpio26,
	&io_gpio27,
	&io_gpio28,
	&io_gpio29,
	&io_gpio30,
	&io_qd0,
	&io_qd1,
	&io_qd2,
	&io_qd3,
	&io_qclk,
	&io_qcs,
	&io_key_col_0,
	&io_key_col_1,
	&io_key_col_2,
	&io_key_col_3,
	&io_key_col_4,
	&io_key_col_5,
	&io_key_row_0,
	&io_key_row_1,
	&io_key_row_2,
	&io_key_row_3,
	&io_key_row_4,
	&io_key_row_5,
	&io_iicscl,
	&io_iicsda,
	&io_u1tx,
	&io_u1rx,
	&io_u2tx,
	&io_u2rx,
	&io_spido,
	&io_spidi,
	&io_spics,
	&io_spick,
};

static struct io *
io_get(const char *id)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(ios); i++) {
		if (!strcmp(id, ios[i]->name))
			return ios[i];
	}

	return NULL;
}

void
io_drive_strength(const char *id, int strength)
{
	int tmp, bit, reg;
	struct io *io = io_get(id);

	if (!io || strength > 3)
		return;

	reg = io->reg * 2;
	bit = io->bit * 2;

	if (io->bit > 15) {
		reg += 4;
		bit -= 32;
	}

	tmp = readl(DVF97_SYSCFG_BASE + reg + IODS_OFFSET);
	tmp &= ~(3 << bit);
	tmp |= strength << bit;
	writel(tmp, DVF97_SYSCFG_BASE + reg + IODS_OFFSET);
}

void
io_pull(const char *id, int enable, int up)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF97_SYSCFG_BASE + io->reg + IOV_OFFSET);
	if (up)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF97_SYSCFG_BASE + io->reg + IOV_OFFSET);

	tmp = readl(DVF97_SYSCFG_BASE + io->reg + IOPE_OFFSET);
	if (enable)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF97_SYSCFG_BASE + io->reg + IOPE_OFFSET);
}

void
io_slew_rate(const char *id, int slow)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF97_SYSCFG_BASE + io->reg + IOSR_OFFSET);
	if (slow)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF97_SYSCFG_BASE + io->reg + IOSR_OFFSET);
}

void
io_keeper(const char *id, int enable)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF97_SYSCFG_BASE + io->reg + IOKE_OFFSET);
	if (enable)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF97_SYSCFG_BASE + io->reg + IOKE_OFFSET);
}

void
io_schmitt(const char *id, int enable)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF97_SYSCFG_BASE + io->reg + IOSMT_OFFSET);
	if (enable)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF97_SYSCFG_BASE + io->reg + IOSMT_OFFSET);
}
