#ifndef __CLOCK_INTERNAL_H
#define __CLOCK_INTERNAL_H

#include <common.h>
#include <asm/arch/clock.h>

extern struct clk *clocks[];

static inline uint32_t
readl_cmu(uint32_t addr)
{
	return readl(DVF101_CMU_BASE + addr);
}

static inline void
writel_cmu(uint32_t val, uint32_t addr)
{
	writel(val, DVF101_CMU_BASE + addr);
}

enum clk_flags {
	/* no parent */
	OSC_IN		= (1 << 0),
	/* only even divider values allowed; no bypass */
	EVEN_DIV	= (1 << 1),
	/* divider not adjustable */
	FIXED_DIV	= (1 << 2),
	HAS_EN		= (1 << 3),
	HAS_AHB		= (1 << 4),
	HAS_APB		= (1 << 5),
	HAS_AXI		= (1 << 6),
	IS_LEAF		= (1 << 7),
};

struct clk_div {
	uint32_t reg;
	unsigned shift;
	unsigned length;
};

struct clk_bit {
	uint32_t reg;
	unsigned shift;
};

struct clk {
	char *name;
	uint32_t flags;

	struct clk_div div;
	struct clk_bit mux;
	uint32_t en_reg;
	uint32_t en_mask;
	uint32_t en_stat_mask;

	int (*is_enabled)(const struct clk *clk);
	int (*enable)(const struct clk *clk, int en);

	unsigned long (*get_rate)(const struct clk *clk);
	int (*set_rate)(const struct clk *clk, unsigned long rate);

	struct clk *(*get_parent)(const struct clk *clk);
	struct clk *parents[4];
};

#endif	/* __CLOCK_INTERNAL_H */
