/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/hardware.h>
#include <asm/arch/clock.h>
#include <asm/arch/dvf10a.h>

const char *logo_dvf101[] = {
"oooooooooo. oooooo     oooo oooooooooooo   .o    .oooo.     .o\n",
"`888'   `Y8b `888.     .8'  `888'     `8 o888   d8P'`Y8b  o888\n",
" 888      888 `888.   .8'    888          888  888    888  888\n",
" 888      888  `888. .8'     888oooo8     888  888    888  888\n",
" 888      888   `888.8'      888    \"     888  888    888  888\n",
" 888     d88'    `888'       888          888  `88b  d88'  888\n",
"o888bood8P'       `8'       o888o        o888o  `Y8bd8P'  o888o\n\n",
};

static int dvf101_detect_chip(void)
{
	int chip = DVF101_CHIP_UNKNOWN;

	switch (readl(DVF101_SYSCFG_BASE + DVF101_SYSCFG_CHIP_ID)) {
	case 0x1010:
		chip = DVF101_CHIP_DVF101;
		break;
	}

	return chip;
}

#ifdef CONFIG_RECOVERY
int is_recovery(void){
	return !strcmp("recovery", (char *)(DVF101_INTRAM_BASE + 0x10000 -
					    DVF101_RECOVERY_INFOSIZE));
}
#endif

int print_cpuinfo(void)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(logo_dvf101); i++)
		puts(logo_dvf101[i]);

	puts("CPU:   ");

	switch (dvf101_detect_chip()) {
	case DVF101_CHIP_DVF101:
		puts("DVF101\n");
		break;
	default: puts("unknown\n");
	}

	return 0;
}

#define GCU_RST_CNTRL 0x0000
#define GCU_SW_RST    0x0004
void reset_cpu(unsigned long addr)
{
	uint32_t reg;

	/* Clear JTAGRSTIND, SWSYSRSTIND and WDRSTIND before
	 * resetting the CPU. */
	reg = readl(DVF101_CMU_BASE + GCU_RST_CNTRL);
	reg |= ((1<<19) | (1<<17) | (1<<16));
	writel(reg, DVF101_CMU_BASE + GCU_RST_CNTRL);

	writel(0x7496398b, DVF101_CMU_BASE + GCU_SW_RST);
	writel(0xb5c9c674, DVF101_CMU_BASE + GCU_SW_RST);
}

#ifdef CONFIG_ARCH_CPU_INIT
int arch_cpu_init(void)
{
	clk_init();
	//dvf10a_init();

	return 0;
}
#endif /* CONFIG_ARCH_CPU_INIT */
