/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>

int do_cfg_mac_addr(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char mac[20];
	unsigned long long addr = 0x009696969600ULL + simple_strtol(argv[1], NULL, 10);

	sprintf(mac, "%02llX:%02llX:%02llX:%02llX:%02llX:%02llX",
		(addr >> 40) & 0xff, (addr >> 32) & 0xff, (addr >> 24) & 0xff,
		(addr >> 16) & 0xff, (addr >>  8) & 0xff,  addr        & 0xff);
	setenv ("ethaddr", mac);

	return 0;
}

U_BOOT_CMD(cfg_mac_addr, 2, 0, do_cfg_mac_addr,
	"cfg_mac_addr - Configure Ethernet MAC address.\n",
	""
);

#ifdef CONFIG_RANDOM_MACADDR
#include <asm/arch/clock.h>
#include <asm/arch/hardware.h>
#include "secure_boot/crypto_block.h"

#define CRYPTO_REG_TRNG_CFG		0x0008
#define CRYPTO_REG_TRNG_KICK		0x000c
#define CRYPTO_REG_TRNG_STATUS		0x0010
#define CRYPTO_REG_TRNG_CLR_STATUS	0x0014

#define CRYPTO_TRNG_STATUS_BUSY		0x0001
#define CRYPTO_TRNG_STATUS_READY	0x0002
#define CRYPTO_TRNG_STATUS_READY_CLR	0x0001

#define CRYPTO_TRNG_CONFIG_HW_TUNE	0x0000
#define CRYPTO_TRNG_CONFIG_KEY_INDEX	0x0000004f

#define CRYPTO_TRNG_MAGIC_SEQ		0xaa55bb11
#define CRYPTO_KS_KEY_256_START		0x1200

int do_random_mac_addr(cmd_tbl_t *cmdtp, int flag, int argc,
		       char *const argv[])
{
	char mac[18] = {0};
	unsigned long long addr;

	clk_enable("crypto_clk");

	while ((crypto_readl(CRYPTO_REG_TRNG_STATUS) & CRYPTO_TRNG_STATUS_BUSY)
	       == CRYPTO_TRNG_STATUS_BUSY)
		;

	crypto_writel(CRYPTO_REG_TRNG_CFG, CRYPTO_TRNG_CONFIG_HW_TUNE |
					   CRYPTO_TRNG_CONFIG_KEY_INDEX);

	/* start the random number generation and wait until finished */
	crypto_writel(CRYPTO_REG_TRNG_KICK, CRYPTO_TRNG_MAGIC_SEQ);
	while ((crypto_readl(CRYPTO_REG_TRNG_STATUS) &
		CRYPTO_TRNG_STATUS_READY) != CRYPTO_TRNG_STATUS_READY)
		;
	crypto_writel(CRYPTO_REG_TRNG_CLR_STATUS,
		      CRYPTO_TRNG_STATUS_READY_CLR);

	addr = crypto_readl(CRYPTO_KS_KEY_256_START +
			    0x20 * (CRYPTO_TRNG_CONFIG_KEY_INDEX - 32));
	addr = (addr << 32) |
		crypto_readl(CRYPTO_KS_KEY_256_START +
			     0x20 * (CRYPTO_TRNG_CONFIG_KEY_INDEX - 32) + 4);
	/*
	 * setting the 2nd LSB in the most significant byte of
	 * the address makes it a locally administered ethernet
	 * address
	 */
	addr = (addr & 0xfeffffffffff) | 0x020000000000;

	sprintf(mac, "%2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X",
		(unsigned)(addr >> 40) & 0xff, (unsigned)(addr >> 32) & 0xff,
		(unsigned)(addr >> 24) & 0xff, (unsigned)(addr >> 16) & 0xff,
		(unsigned)(addr >>  8) & 0xff, (unsigned)(addr >>  0) & 0xff);
	setenv("ethaddr", mac);

	return 0;
}

U_BOOT_CMD(cfg_random_mac_addr, 1, 0, do_random_mac_addr,
	   "cfg_unique_mac_addr - Assign random ethernet MAC address.\n",
	   ""
);
#endif
