/*
 * (C) Copyright 2015 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/io.h>
#include <asm/arch/hardware.h>

#define IOM1  0x200
#define IOM2  0x204
#define IOM3  0x208
#define IOM4  0x20C
#define IOM5  0x210
#define IOM6  0x214
#define IOM7  0x218
#define IOM8  0x21C
#define IOM9  0x220
#define IOM10 0x224
#define IOM11 0x228

struct iomux {
	char *name;
	int reg;
	int shift;
	int value;
};

struct io {
	char *name;
	int reg;
	int bit;
};

#define IOMUX(_name, _reg, _shift, _val)      \
	static struct iomux iomux_##_name = { \
		.name = __stringify(_name),   \
		.reg = _reg,                  \
		.shift = _shift,              \
		.value = _val,                \
	}

IOMUX(agpio0,                 IOM4,   0, 0);
IOMUX(agpio1,                 IOM5,  30, 0);
IOMUX(agpio2,                 IOM5,  28, 0);
IOMUX(agpio3,                 IOM5,  26, 0);
IOMUX(agpio4,                 IOM5,  24, 0);
IOMUX(agpio5,                 IOM5,  22, 0);
IOMUX(agpio6,                 IOM5,  20, 0);
IOMUX(agpio7,                 IOM5,  18, 0);
IOMUX(agpio8,                 IOM5,  16, 0);
IOMUX(agpio9,                 IOM5,  14, 0);
IOMUX(agpio10,                IOM5,  12, 0);
IOMUX(agpio11,                IOM5,  10, 0);
IOMUX(agpio12,                IOM5,   8, 0);
IOMUX(agpio13,                IOM5,   6, 0);
IOMUX(agpio14,                IOM8,  30, 1);
IOMUX(agpio15,                IOM8,  28, 1);
IOMUX(agpio16,                IOM5,   4, 0);
IOMUX(agpio17,                IOM5,   2, 0);
IOMUX(agpio18,                IOM5,   0, 0);
IOMUX(agpio19,                IOM6,  30, 0);
IOMUX(agpio20,                IOM10,  0, 0);
IOMUX(agpio21,                IOM11, 30, 0);
IOMUX(agpio22,                IOM11, 28, 0);
IOMUX(agpio23,                IOM11, 26, 0);
IOMUX(agpio26,                IOM6,  28, 0);
IOMUX(agpio27,                IOM6,  26, 0);
IOMUX(agpio28,                IOM6,  24, 0);
IOMUX(agpio29,                IOM6,  22, 0);
IOMUX(agpio30,                IOM8,  26, 1);
IOMUX(agpio31,                IOM8,  24, 1);
IOMUX(bgpio0,                 IOM10, 22, 0);
IOMUX(bgpio1,                 IOM10, 20, 0);
IOMUX(bgpio2,                 IOM10, 18, 0);
IOMUX(bgpio3,                 IOM10, 16, 0);
IOMUX(bgpio4,                 IOM8,  22, 1);
IOMUX(bgpio5,                 IOM8,  18, 1);
IOMUX(bgpio6,                 IOM8,  20, 1);
IOMUX(bgpio7,                 IOM8,  16, 1);
IOMUX(bgpio8,                 IOM10, 14, 0);
IOMUX(bgpio9,                 IOM10, 12, 0);
IOMUX(bgpio10,                IOM10, 10, 0);
IOMUX(bgpio11,                IOM9,   4, 0);
IOMUX(bgpio12,                IOM10,  8, 0);
IOMUX(bgpio13,                IOM10,  6, 0);
IOMUX(bgpio14,                IOM10,  4, 0);
IOMUX(bgpio16,                IOM10,  2, 0);
IOMUX(bgpio17,                IOM9,  26, 0);
IOMUX(bgpio18,                IOM9,  24, 0);
IOMUX(bgpio19,                IOM9,  22, 0);
IOMUX(bgpio20,                IOM9,  20, 0);
IOMUX(bgpio21,                IOM9,  18, 0);
IOMUX(bgpio22,                IOM9,  16, 0);
IOMUX(bgpio23,                IOM9,  14, 0);
IOMUX(bgpio24,                IOM9,  12, 0);
IOMUX(bgpio25,                IOM9,  10, 0);
IOMUX(bgpio26,                IOM1,  16, 0);
IOMUX(bgpio27,                IOM7,   0, 1);
IOMUX(bgpio28,                IOM8,  14, 1);
IOMUX(bgpio29,                IOM8,  10, 1);
IOMUX(bgpio30,                IOM8,  12, 1);
IOMUX(bgpio31,                IOM9,   2, 0);
IOMUX(cgpio0,                 IOM2,  28, 0);
IOMUX(cgpio1,                 IOM3,  22, 0);
IOMUX(cgpio2,                 IOM2,  12, 0);
IOMUX(cgpio3,                 IOM3,  12, 0);
IOMUX(cgpio4,                 IOM2,   6, 0);
IOMUX(cgpio5,                 IOM2,  22, 0);
IOMUX(cgpio6,                 IOM3,  26, 0);
IOMUX(cgpio7,                 IOM3,  30, 0);
IOMUX(cgpio8,                 IOM1,  12, 0);
IOMUX(cgpio9,                 IOM1,   4, 0);
IOMUX(cgpio10,                IOM1,   0, 0);
IOMUX(cgpio11,                IOM2,  20, 0);
IOMUX(cgpio12,                IOM3,  18, 0);
IOMUX(cgpio13,                IOM1,  22, 0);
IOMUX(cgpio14,                IOM1,  20, 0);
IOMUX(cgpio15,                IOM2,  16, 0);
IOMUX(cgpio16,                IOM1,  10, 0);
IOMUX(cgpio17,                IOM1,   8, 0);
IOMUX(cgpio18,                IOM1,  26, 0);
IOMUX(cgpio19,                IOM1,  18, 0);
IOMUX(cgpio20,                IOM3,  10, 0);
IOMUX(cgpio21,                IOM2,  18, 0);
IOMUX(cgpio22,                IOM2,   8, 0);
IOMUX(cgpio23,                IOM3,  16, 0);
IOMUX(cgpio24,                IOM1,  14, 0);
IOMUX(cgpio25,                IOM3,  14, 0);
IOMUX(cgpio26,                IOM2,  30, 0);
IOMUX(cgpio27,                IOM1,   2, 0);
IOMUX(cgpio28,                IOM2,   4, 0);
IOMUX(cgpio29,                IOM3,  24, 0);
IOMUX(cgpio30,                IOM2,  24, 0);
IOMUX(cgpio31,                IOM1,  24, 0);
IOMUX(dgpio0,                 IOM1,  30, 0);
IOMUX(dgpio1,                 IOM3,  20, 0);
IOMUX(dgpio2,                 IOM1,   6, 0);
IOMUX(dgpio3,                 IOM2,  14, 0);
IOMUX(dgpio4,                 IOM2,  10, 0);
IOMUX(dgpio5,                 IOM2,  26, 0);
IOMUX(dgpio6,                 IOM1,  28, 0);
IOMUX(dgpio7,                 IOM2,   2, 0);
IOMUX(dgpio8,                 IOM3,   8, 0);
IOMUX(dgpio9,                 IOM3,   6, 0);
IOMUX(dgpio10,                IOM2,   0, 0);
IOMUX(dgpio11,                IOM3,  28, 0);
IOMUX(dgpio12,                IOM7,  18, 0);
IOMUX(dgpio13,                IOM7,  16, 0);
IOMUX(dgpio14,                IOM7,  24, 0);
IOMUX(dgpio15,                IOM7,  14, 0);
IOMUX(dgpio16,                IOM7,  12, 0);
IOMUX(dgpio17,                IOM7,  10, 0);
IOMUX(dgpio18,                IOM7,  22, 0);
IOMUX(dgpio19,                IOM7,  20, 0);
IOMUX(dgpio20,                IOM8,   4, 0);
IOMUX(dgpio21,                IOM8,   2, 0);
IOMUX(dgpio22,                IOM8,   0, 0);
IOMUX(dgpio23,                IOM9,  30, 0);
IOMUX(dgpio24,                IOM9,   8, 0);
IOMUX(dgpio25,                IOM9,   6, 0);
IOMUX(dgpio26,                IOM9,  28, 0);
IOMUX(dgpio30,                IOM7,   8, 0);
IOMUX(dgpio31,                IOM7,   6, 0);
IOMUX(egpio0,                 IOM3,   4, 0);
IOMUX(egpio1,                 IOM3,   2, 0);
IOMUX(egpio2,                 IOM3,   0, 0);
IOMUX(egpio3,                 IOM4,  30, 0);
IOMUX(egpio4,                 IOM4,  28, 0);
IOMUX(egpio5,                 IOM4,  26, 0);
IOMUX(egpio6,                 IOM4,  24, 0);
IOMUX(egpio7,                 IOM4,  22, 0);
IOMUX(egpio8,                 IOM4,  20, 0);
IOMUX(egpio9,                 IOM4,  18, 0);
IOMUX(egpio10,                IOM4,  16, 0);
IOMUX(egpio11,                IOM4,  14, 0);
IOMUX(egpio12,                IOM4,  12, 0);
IOMUX(egpio13,                IOM4,  10, 0);
IOMUX(egpio14,                IOM4,   8, 0);
IOMUX(egpio15,                IOM4,   6, 0);
IOMUX(egpio16,                IOM4,   4, 0);
IOMUX(egpio17,                IOM4,   2, 0);
IOMUX(egpio18,                IOM6,  20, 0);
IOMUX(egpio19,                IOM6,  18, 0);
IOMUX(egpio20,                IOM6,  16, 0);
IOMUX(egpio21,                IOM6,  14, 0);
IOMUX(egpio22,                IOM6,  12, 0);
IOMUX(egpio23,                IOM6,  10, 0);
IOMUX(egpio24,                IOM6,   8, 0);
IOMUX(egpio25,                IOM6,   6, 0);
IOMUX(egpio26,                IOM6,   4, 0);
IOMUX(egpio27,                IOM6,   2, 0);
IOMUX(egpio28,                IOM6,   0, 0);
IOMUX(egpio29,                IOM7,  30, 0);
IOMUX(egpio30,                IOM7,  28, 0);
IOMUX(egpio31,                IOM7,  26, 0);

IOMUX(clk_out,                IOM5,   0, 2);
IOMUX(clk_ref_out,            IOM6,  30, 2);
IOMUX(main_clk_out,           IOM9,   2, 1);

IOMUX(daif_apu_if_bat_ok,     IOM8,  30, 0);
IOMUX(daif_apu_if_cdc_dat_ad, IOM8,  28, 0);
IOMUX(daif_apu_if_cdc_dat_da, IOM8,  26, 0);
IOMUX(daif_apu_if_cdc_dir,    IOM8,  24, 0);
IOMUX(daif_apu_if_clk_if,     IOM8,   8, 0);
IOMUX(daif_apu_if_clk_pll,    IOM8,  22, 0);
IOMUX(daif_apu_if_data_ad,    IOM8,  20, 0);
IOMUX(daif_apu_if_data_da,    IOM8,  18, 0);
IOMUX(daif_apu_if_en_ad,      IOM8,  16, 0);
IOMUX(daif_apu_if_en_da,      IOM8,  14, 0);
IOMUX(daif_apu_if_int_ana,    IOM8,  12, 0);
IOMUX(daif_apu_if_nares,      IOM8,  10, 0);

IOMUX(digmic_clk,             IOM8,   0, 2);
IOMUX(digmic_data,            IOM9,  30, 2);

IOMUX(drv_vbus,               IOM9,  28, 2);

IOMUX(d_class_clk,            IOM7,   2, 0);
IOMUX(d_class_pwm,            IOM7,   0, 0);
IOMUX(d_class_pwmn,           IOM7,   4, 0);

IOMUX(efuse_csb,              IOM6,  20, 2);
IOMUX(efuse_din,              IOM6,  10, 2);
IOMUX(efuse_dout,             IOM6,   8, 2);
IOMUX(efuse_mem_sel_1,        IOM6,  18, 2);
IOMUX(efuse_mem_sel_2,        IOM6,  16, 2);
IOMUX(efuse_pgm,              IOM6,  14, 2);
IOMUX(efuse_sclk,             IOM6,  12, 2);

IOMUX(extint0,                IOM5,   4, 1);
IOMUX(extint1,                IOM5,   2, 1);
IOMUX(extint2,                IOM5,   0, 1);
IOMUX(extint3,                IOM6,  30, 1);
IOMUX(extint4,                IOM1,  16, 1);
IOMUX(extint5,                IOM2,   4, 1);
IOMUX(extint6,                IOM3,   8, 1);
IOMUX(extint7,                IOM3,   6, 1);
IOMUX(extint8,                IOM9,  28, 1);
IOMUX(extint9,                IOM9,   8, 1);
IOMUX(extint10,               IOM9,   6, 1);
IOMUX(extint11,               IOM9,   2, 2);
IOMUX(extint12,               IOM4,  24, 2);
IOMUX(extint13,               IOM1,  24, 1);
IOMUX(extint13a,              IOM7,   8, 1);
IOMUX(extint14,               IOM1,  26, 1);
IOMUX(extint14a,              IOM7,   6, 1);
IOMUX(extint15,               IOM9,   4, 1);

IOMUX(d1rx,                   IOM10, 16, 1);
IOMUX(d1tx,                   IOM10, 18, 1);
IOMUX(f1sync,                 IOM10, 20, 1);
IOMUX(t1sclk,                 IOM10, 22, 1);
IOMUX(d2rx,                   IOM9,  20, 1);
IOMUX(d2tx,                   IOM9,  22, 1);
IOMUX(f2sync,                 IOM9,  24, 1);
IOMUX(t2sclk,                 IOM9,  26, 1);
IOMUX(d3rx,                   IOM9,  12, 1);
IOMUX(d3tx,                   IOM9,  14, 1);
IOMUX(f3sync,                 IOM9,  16, 1);
IOMUX(t3sclk,                 IOM9,  18, 1);

IOMUX(iic1scl,                IOM5,   2, 2);
IOMUX(iic1sda,                IOM5,   4, 2);
IOMUX(iic2scl,                IOM7,  28, 2);
IOMUX(iic2sda,                IOM7,  26, 2);

IOMUX(key_col0,               IOM6,  10, 1);
IOMUX(key_col1,               IOM6,   8, 1);
IOMUX(key_col2,               IOM6,   6, 1);
IOMUX(key_col3,               IOM6,   4, 1);
IOMUX(key_col4,               IOM6,   2, 1);
IOMUX(key_col5,               IOM6,   0, 1);
IOMUX(key_col6,               IOM6,  28, 1);
IOMUX(key_col7,               IOM6,  26, 1);
IOMUX(key_row0,               IOM6,  20, 1);
IOMUX(key_row1,               IOM6,  18, 1);
IOMUX(key_row2,               IOM6,  16, 1);
IOMUX(key_row3,               IOM6,  14, 1);
IOMUX(key_row4,               IOM6,  12, 1);
IOMUX(key_row5,               IOM7,  30, 1);
IOMUX(key_row6,               IOM7,  28, 1);
IOMUX(key_row7,               IOM7,  26, 1);
IOMUX(key_row8,               IOM6,  24, 1);
IOMUX(key_row9,               IOM6,  22, 1);

IOMUX(lcdd0,                  IOM1,  28, 1);
IOMUX(lcdd1,                  IOM3,  20, 1);
IOMUX(lcdd2,                  IOM2,  28, 1);
IOMUX(lcdd3,                  IOM3,  22, 1);
IOMUX(lcdd4,                  IOM2,  12, 1);
IOMUX(lcdd5,                  IOM3,  12, 1);
IOMUX(lcdd6,                  IOM2,   6, 1);
IOMUX(lcdd7,                  IOM2,  22, 1);
IOMUX(lcdd8,                  IOM1,   6, 1);
IOMUX(lcdd9,                  IOM2,  14, 1);
IOMUX(lcdd10,                 IOM3,  26, 1);
IOMUX(lcdd11,                 IOM3,  30, 1);
IOMUX(lcdd12,                 IOM1,  12, 1);
IOMUX(lcdd13,                 IOM1,   4, 1);
IOMUX(lcdd14,                 IOM1,   0, 1);
IOMUX(lcdd15,                 IOM2,  20, 1);
IOMUX(lcdd16,                 IOM2,  10, 1);
IOMUX(lcdd17,                 IOM2,  26, 1);
IOMUX(lcdd18,                 IOM3,  18, 1);
IOMUX(lcdd19,                 IOM1,  22, 1);
IOMUX(lcdd20,                 IOM1,  20, 1);
IOMUX(lcdd21,                 IOM2,  16, 1);
IOMUX(lcdd22,                 IOM1,  10, 1);
IOMUX(lcdd23,                 IOM1,   8, 1);
IOMUX(lcdgp0,                 IOM1,  18, 1);
IOMUX(lcdgp1,                 IOM3,  10, 1);
IOMUX(lcdgp2,                 IOM2,  18, 1);
IOMUX(lcdgp3,                 IOM2,   8, 1);
IOMUX(lcdgp4,                 IOM3,  16, 1);
IOMUX(lcdgp5,                 IOM1,  14, 1);
IOMUX(lcdgp6,                 IOM3,  14, 1);
IOMUX(lcdgp7,                 IOM2,  30, 1);
IOMUX(lcdgp8,                 IOM1,   2, 1);
IOMUX(lpclk,                  IOM2,   2, 1);

IOMUX(mdc,                    IOM4,  18, 1);
IOMUX(mdio,                   IOM4,  20, 1);
IOMUX(miicol,                 IOM4,   2, 1);
IOMUX(miicrs_dv,              IOM4,  26, 1);
IOMUX(miirefck,               IOM4,  22, 1);
IOMUX(miirxck,                IOM4,   6, 1);
IOMUX(miirxcrs,               IOM4,   4, 1);
IOMUX(miirxd0,                IOM4,  30, 1);
IOMUX(miirxd1,                IOM4,  28, 1);
IOMUX(miirxd2,                IOM4,  10, 1);
IOMUX(miirxd3,                IOM4,   8, 1);
IOMUX(miirxer,                IOM4,  24, 1);
IOMUX(miitxd0,                IOM3,   4, 1);
IOMUX(miitxd1,                IOM3,   2, 1);
IOMUX(miitxd2,                IOM4,  14, 1);
IOMUX(miitxd3,                IOM4,  12, 1);
IOMUX(miitxen,                IOM3,   0, 1);
IOMUX(miitxer,                IOM4,  16, 1);

IOMUX(nflale,                 IOM5,  22, 1);
IOMUX(nflcle,                 IOM5,  24, 1);
IOMUX(nflcs0,                 IOM5,  20, 1);
IOMUX(nfld0,                  IOM5,  18, 1);
IOMUX(nfld1,                  IOM5,  16, 1);
IOMUX(nfld2,                  IOM5,  14, 1);
IOMUX(nfld3,                  IOM5,  12, 1);
IOMUX(nfld4,                  IOM5,  10, 1);
IOMUX(nfld5,                  IOM5,   8, 1);
IOMUX(nfld6,                  IOM5,   6, 1);
IOMUX(nfld7,                  IOM4,   0, 1);
IOMUX(nflrd,                  IOM5,  26, 1);
IOMUX(nflready1,              IOM5,  30, 1);
IOMUX(nflwr,                  IOM5,  28, 1);

IOMUX(rf_radio_en,            IOM11, 30, 2);
IOMUX(rf_rstn,                IOM9,  10, 1);
IOMUX(rf_sclk,                IOM9,  24, 2);
IOMUX(rf_sdata_in,            IOM9,  22, 2);
IOMUX(rf_sdata_out,           IOM9,  26, 2);
IOMUX(rf_sen,                 IOM9,  20, 2);
IOMUX(rf_slot_ctrl,           IOM10,  0, 2);
IOMUX(rf_sync_match,          IOM11, 26, 2);
IOMUX(rf_tr_data,             IOM11, 28, 2);

IOMUX(rgmiird0,               IOM4, 30, 2);
IOMUX(rgmiird1,               IOM4, 28, 2);
IOMUX(rgmiird2,               IOM4, 10, 2);
IOMUX(rgmiird3,               IOM4,  8, 2);
IOMUX(rgmiirxc,               IOM4,  6, 2);
IOMUX(rgmiirx_ctl,            IOM4, 26, 2);
IOMUX(rgmiitd0,               IOM3,  4, 2);
IOMUX(rgmiitd1,               IOM3,  2, 2);
IOMUX(rgmiitd2,               IOM4, 14, 2);
IOMUX(rgmiitd3,               IOM4, 12, 2);
IOMUX(rgmiitxc,               IOM4, 22, 2);
IOMUX(rgmiitx_ctl,            IOM3,  0, 2);

IOMUX(sd_clk,                 IOM7, 22, 1);
IOMUX(sd_cmd,                 IOM7, 20, 1);
IOMUX(sd_d0,                  IOM7, 24, 1);
IOMUX(sd_d1,                  IOM7, 18, 1);
IOMUX(sd_d2,                  IOM7, 16, 1);
IOMUX(sd_d3,                  IOM7, 14, 1);
IOMUX(sd_detect,              IOM7, 10, 1);
IOMUX(sd_wr_prot,             IOM7, 12, 1);

IOMUX(spi1ck,                 IOM10,  8, 1);
IOMUX(spi1cs0,                IOM10, 10, 1);
IOMUX(spi1cs1,                IOM2,  24, 1);
IOMUX(spi1cs1a,               IOM4,  16, 2);
IOMUX(spi1di,                 IOM10, 12, 1);
IOMUX(spi1do,                 IOM10, 14, 1);
IOMUX(spi2ck,                 IOM10,  2, 1);
IOMUX(spi2cs,                 IOM1,  30, 1);
IOMUX(spi2di,                 IOM10,  4, 1);
IOMUX(spi2do,                 IOM10,  6, 1);
IOMUX(spi3ck,                 IOM11, 26, 1);
IOMUX(spi3cs0,                IOM11, 28, 1);
IOMUX(spi3cs1,                IOM3,  24, 1);
IOMUX(spi3di,                 IOM11, 30, 1);
IOMUX(spi3do,                 IOM10,  0, 1);

IOMUX(syncport,               IOM9,   8, 2);

IOMUX(traceclk,               IOM6,  28, 3);
IOMUX(tracectl,               IOM6,   0, 3);
IOMUX(tracedata0,             IOM6,   4, 3);
IOMUX(tracedata1,             IOM6,   8, 3);
IOMUX(tracedata2,             IOM6,  12, 3);
IOMUX(tracedata3,             IOM6,  16, 3);
IOMUX(tracedata4,             IOM6,  20, 3);
IOMUX(tracedata5,             IOM6,  22, 3);
IOMUX(tracedata6,             IOM6,  24, 3);
IOMUX(tracedata7,             IOM6,  26, 3);
IOMUX(tracedata8,             IOM7,  26, 3);
IOMUX(tracedata9,             IOM7,  28, 3);
IOMUX(tracedata10,            IOM7,  30, 3);
IOMUX(tracedata11,            IOM6,   2, 3);
IOMUX(tracedata12,            IOM6,   6, 3);
IOMUX(tracedata13,            IOM6,  10, 3);
IOMUX(tracedata14,            IOM6,  14, 3);
IOMUX(tracedata15,            IOM6,  18, 3);

IOMUX(u1rx,                   IOM8,   2, 1);
IOMUX(u1tx,                   IOM8,   4, 1);
IOMUX(u1cts,                  IOM9,  30, 1);
IOMUX(u1rts,                  IOM8,   0, 1);
IOMUX(u2rx,                   IOM4,   2, 2);
IOMUX(u2tx,                   IOM4,   4, 2);
IOMUX(u3rx,                   IOM3,  28, 1);
IOMUX(u3tx,                   IOM2,   0, 1);
IOMUX(u4rx,                   IOM9,   6, 2);
IOMUX(u4tx,                   IOM9,   4, 2);
IOMUX(u4rxa,                  IOM9,  14, 2);
IOMUX(u4txa,                  IOM9,  12, 2);

IOMUX(emmc_clk,               IOM5,  28, 2);
IOMUX(emmc_cmd,               IOM5,  30, 2);
IOMUX(emmc_d0,                IOM5,  18, 2);
IOMUX(emmc_d1,                IOM5,  16, 2);
IOMUX(emmc_d2,                IOM5,  14, 2);
IOMUX(emmc_d3,                IOM5,  12, 2);
IOMUX(emmc_d4,                IOM5,  10, 2);
IOMUX(emmc_d5,                IOM5,   8, 2);
IOMUX(emmc_d6,                IOM5,   6, 2);
IOMUX(emmc_d7,                IOM4,   0, 2);

static struct iomux *iomuxes[] = {
	&iomux_agpio0,
	&iomux_agpio1,
	&iomux_agpio2,
	&iomux_agpio3,
	&iomux_agpio4,
	&iomux_agpio5,
	&iomux_agpio6,
	&iomux_agpio7,
	&iomux_agpio8,
	&iomux_agpio9,
	&iomux_agpio10,
	&iomux_agpio11,
	&iomux_agpio12,
	&iomux_agpio13,
	&iomux_agpio14,
	&iomux_agpio15,
	&iomux_agpio16,
	&iomux_agpio17,
	&iomux_agpio18,
	&iomux_agpio19,
	&iomux_agpio20,
	&iomux_agpio21,
	&iomux_agpio22,
	&iomux_agpio23,
	&iomux_agpio26,
	&iomux_agpio27,
	&iomux_agpio28,
	&iomux_agpio29,
	&iomux_agpio30,
	&iomux_agpio31,
	&iomux_bgpio0,
	&iomux_bgpio1,
	&iomux_bgpio2,
	&iomux_bgpio3,
	&iomux_bgpio4,
	&iomux_bgpio5,
	&iomux_bgpio6,
	&iomux_bgpio7,
	&iomux_bgpio8,
	&iomux_bgpio9,
	&iomux_bgpio10,
	&iomux_bgpio11,
	&iomux_bgpio12,
	&iomux_bgpio13,
	&iomux_bgpio14,
	&iomux_bgpio16,
	&iomux_bgpio17,
	&iomux_bgpio18,
	&iomux_bgpio19,
	&iomux_bgpio20,
	&iomux_bgpio21,
	&iomux_bgpio22,
	&iomux_bgpio23,
	&iomux_bgpio24,
	&iomux_bgpio25,
	&iomux_bgpio26,
	&iomux_bgpio27,
	&iomux_bgpio28,
	&iomux_bgpio29,
	&iomux_bgpio30,
	&iomux_bgpio31,
	&iomux_cgpio0,
	&iomux_cgpio1,
	&iomux_cgpio2,
	&iomux_cgpio3,
	&iomux_cgpio4,
	&iomux_cgpio5,
	&iomux_cgpio6,
	&iomux_cgpio7,
	&iomux_cgpio8,
	&iomux_cgpio9,
	&iomux_cgpio10,
	&iomux_cgpio11,
	&iomux_cgpio12,
	&iomux_cgpio13,
	&iomux_cgpio14,
	&iomux_cgpio15,
	&iomux_cgpio16,
	&iomux_cgpio17,
	&iomux_cgpio18,
	&iomux_cgpio19,
	&iomux_cgpio20,
	&iomux_cgpio21,
	&iomux_cgpio22,
	&iomux_cgpio23,
	&iomux_cgpio24,
	&iomux_cgpio25,
	&iomux_cgpio26,
	&iomux_cgpio27,
	&iomux_cgpio28,
	&iomux_cgpio29,
	&iomux_cgpio30,
	&iomux_cgpio31,
	&iomux_dgpio0,
	&iomux_dgpio1,
	&iomux_dgpio2,
	&iomux_dgpio3,
	&iomux_dgpio4,
	&iomux_dgpio5,
	&iomux_dgpio6,
	&iomux_dgpio7,
	&iomux_dgpio8,
	&iomux_dgpio9,
	&iomux_dgpio10,
	&iomux_dgpio11,
	&iomux_dgpio12,
	&iomux_dgpio13,
	&iomux_dgpio14,
	&iomux_dgpio15,
	&iomux_dgpio16,
	&iomux_dgpio17,
	&iomux_dgpio18,
	&iomux_dgpio19,
	&iomux_dgpio20,
	&iomux_dgpio21,
	&iomux_dgpio22,
	&iomux_dgpio23,
	&iomux_dgpio24,
	&iomux_dgpio25,
	&iomux_dgpio26,
	&iomux_dgpio30,
	&iomux_dgpio31,
	&iomux_egpio0,
	&iomux_egpio1,
	&iomux_egpio2,
	&iomux_egpio3,
	&iomux_egpio4,
	&iomux_egpio5,
	&iomux_egpio6,
	&iomux_egpio7,
	&iomux_egpio8,
	&iomux_egpio9,
	&iomux_egpio10,
	&iomux_egpio11,
	&iomux_egpio12,
	&iomux_egpio13,
	&iomux_egpio14,
	&iomux_egpio15,
	&iomux_egpio16,
	&iomux_egpio17,
	&iomux_egpio18,
	&iomux_egpio19,
	&iomux_egpio20,
	&iomux_egpio21,
	&iomux_egpio22,
	&iomux_egpio23,
	&iomux_egpio24,
	&iomux_egpio25,
	&iomux_egpio26,
	&iomux_egpio27,
	&iomux_egpio28,
	&iomux_egpio29,
	&iomux_egpio30,
	&iomux_egpio31,

	&iomux_key_col0,
	&iomux_key_col1,
	&iomux_key_col2,
	&iomux_key_col3,
	&iomux_key_col4,
	&iomux_key_col5,
	&iomux_key_col6,
	&iomux_key_col7,

	&iomux_key_row0,
	&iomux_key_row1,
	&iomux_key_row2,
	&iomux_key_row3,
	&iomux_key_row4,
	&iomux_key_row5,
	&iomux_key_row6,
	&iomux_key_row7,
	&iomux_key_row8,
	&iomux_key_row9,

	&iomux_u1tx,
	&iomux_u1rx,
	&iomux_u1rts,
	&iomux_u1cts,
	&iomux_u2tx,
	&iomux_u2rx,
	&iomux_u3tx,
	&iomux_u3rx,
	&iomux_u4tx,
	&iomux_u4rx,
	&iomux_u4txa,
	&iomux_u4rxa,

	&iomux_lcdd0,
	&iomux_lcdd1,
	&iomux_lcdd2,
	&iomux_lcdd3,
	&iomux_lcdd4,
	&iomux_lcdd5,
	&iomux_lcdd6,
	&iomux_lcdd7,
	&iomux_lcdd8,
	&iomux_lcdd9,
	&iomux_lcdd10,
	&iomux_lcdd11,
	&iomux_lcdd12,
	&iomux_lcdd13,
	&iomux_lcdd14,
	&iomux_lcdd15,
	&iomux_lcdd16,
	&iomux_lcdd17,
	&iomux_lcdd18,
	&iomux_lcdd19,
	&iomux_lcdd20,
	&iomux_lcdd21,
	&iomux_lcdd22,
	&iomux_lcdd23,
	&iomux_lcdgp0,
	&iomux_lcdgp1,
	&iomux_lcdgp2,
	&iomux_lcdgp3,
	&iomux_lcdgp4,
	&iomux_lcdgp5,
	&iomux_lcdgp6,
	&iomux_lcdgp7,
	&iomux_lcdgp8,
	&iomux_lpclk,

	&iomux_spi1di,
	&iomux_spi1do,
	&iomux_spi1ck,
	&iomux_spi1cs0,
	&iomux_spi1cs1,
	&iomux_spi1cs1a,
	&iomux_spi2di,
	&iomux_spi2do,
	&iomux_spi2ck,
	&iomux_spi2cs,
	&iomux_spi3di,
	&iomux_spi3do,
	&iomux_spi3ck,
	&iomux_spi3cs0,
	&iomux_spi3cs1,

	&iomux_iic1scl,
	&iomux_iic1sda,
	&iomux_iic2scl,
	&iomux_iic2sda,

	&iomux_extint0,
	&iomux_extint1,
	&iomux_extint2,
	&iomux_extint3,
	&iomux_extint4,
	&iomux_extint5,
	&iomux_extint6,
	&iomux_extint7,
	&iomux_extint8,
	&iomux_extint9,
	&iomux_extint10,
	&iomux_extint11,
	&iomux_extint12,
	&iomux_extint13,
	&iomux_extint13a,
	&iomux_extint14,
	&iomux_extint14a,
	&iomux_extint15,

	&iomux_d1rx,
	&iomux_d1tx,
	&iomux_f1sync,
	&iomux_t1sclk,
	&iomux_d2rx,
	&iomux_d2tx,
	&iomux_f2sync,
	&iomux_t2sclk,
	&iomux_d3rx,
	&iomux_d3tx,
	&iomux_f3sync,
	&iomux_t3sclk,

	&iomux_clk_out,
	&iomux_clk_ref_out,
	&iomux_main_clk_out,

	&iomux_syncport,

	&iomux_mdc,
	&iomux_mdio,
	&iomux_miicol,
	&iomux_miicrs_dv,
	&iomux_miirefck,
	&iomux_miirxck,
	&iomux_miirxcrs,
	&iomux_miirxd0,
	&iomux_miirxd1,
	&iomux_miirxd2,
	&iomux_miirxd3,
	&iomux_miirxer,
	&iomux_miitxd0,
	&iomux_miitxd1,
	&iomux_miitxd2,
	&iomux_miitxd3,
	&iomux_miitxen,
	&iomux_miitxer,

	&iomux_nflale,
	&iomux_nflcle,
	&iomux_nflcs0,
	&iomux_nfld0,
	&iomux_nfld1,
	&iomux_nfld2,
	&iomux_nfld3,
	&iomux_nfld4,
	&iomux_nfld5,
	&iomux_nfld6,
	&iomux_nfld7,
	&iomux_nflrd,
	&iomux_nflready1,
	&iomux_nflwr,

	&iomux_rf_radio_en,
	&iomux_rf_rstn,
	&iomux_rf_sclk,
	&iomux_rf_sdata_in,
	&iomux_rf_sdata_out,
	&iomux_rf_sen,
	&iomux_rf_slot_ctrl,
	&iomux_rf_sync_match,
	&iomux_rf_tr_data,

	&iomux_rgmiird0,
	&iomux_rgmiird1,
	&iomux_rgmiird2,
	&iomux_rgmiird3,
	&iomux_rgmiirxc,
	&iomux_rgmiirx_ctl,
	&iomux_rgmiitd0,
	&iomux_rgmiitd1,
	&iomux_rgmiitd2,
	&iomux_rgmiitd3,
	&iomux_rgmiitxc,
	&iomux_rgmiitx_ctl,

	&iomux_traceclk,
	&iomux_tracectl,
	&iomux_tracedata0,
	&iomux_tracedata1,
	&iomux_tracedata2,
	&iomux_tracedata3,
	&iomux_tracedata4,
	&iomux_tracedata5,
	&iomux_tracedata6,
	&iomux_tracedata7,
	&iomux_tracedata8,
	&iomux_tracedata9,
	&iomux_tracedata10,
	&iomux_tracedata11,
	&iomux_tracedata12,
	&iomux_tracedata13,
	&iomux_tracedata14,
	&iomux_tracedata15,

	&iomux_daif_apu_if_bat_ok,
	&iomux_daif_apu_if_cdc_dat_ad,
	&iomux_daif_apu_if_cdc_dat_da,
	&iomux_daif_apu_if_cdc_dir,
	&iomux_daif_apu_if_clk_if,
	&iomux_daif_apu_if_clk_pll,
	&iomux_daif_apu_if_data_ad,
	&iomux_daif_apu_if_data_da,
	&iomux_daif_apu_if_en_ad,
	&iomux_daif_apu_if_en_da,
	&iomux_daif_apu_if_int_ana,
	&iomux_daif_apu_if_nares,

	&iomux_digmic_clk,
	&iomux_digmic_data,

	&iomux_drv_vbus,

	&iomux_d_class_clk,
	&iomux_d_class_pwm,
	&iomux_d_class_pwmn,

	&iomux_efuse_csb,
	&iomux_efuse_din,
	&iomux_efuse_dout,
	&iomux_efuse_mem_sel_1,
	&iomux_efuse_mem_sel_2,
	&iomux_efuse_pgm,
	&iomux_efuse_sclk,

	&iomux_sd_clk,
	&iomux_sd_cmd,
	&iomux_sd_d0,
	&iomux_sd_d1,
	&iomux_sd_d2,
	&iomux_sd_d3,
	&iomux_sd_detect,
	&iomux_sd_wr_prot,

	&iomux_emmc_clk,
	&iomux_emmc_cmd,
	&iomux_emmc_d0,
	&iomux_emmc_d1,
	&iomux_emmc_d2,
	&iomux_emmc_d3,
	&iomux_emmc_d4,
	&iomux_emmc_d5,
	&iomux_emmc_d6,
	&iomux_emmc_d7,
};

void
io_mux(const char *id)
{
	int i, tmp;
	struct iomux *mux = NULL;

	for (i = 0; i < ARRAY_SIZE(iomuxes); i++) {
		if (!strcmp(id, iomuxes[i]->name)) {
			mux = iomuxes[i];
			break;
		}
	}

	if (!mux)
		return;

	tmp = readl(DVF101_SYSCFG_BASE + mux->reg);
	tmp &= ~(3 << mux->shift);
	tmp |= mux->value << mux->shift;
	writel(tmp, DVF101_SYSCFG_BASE + mux->reg);
}

#define IOREG1 0x00
#define IOREG2 0x04
#define IOREG3 0x08
#define IOREG4 0x0c
#define IOREG5 0x10
#define IOREG6 0x14

#define IODS_OFFSET 0x22C
#define IOSR_OFFSET 0x258
#define IOKE_OFFSET 0x270
#define IOPE_OFFSET 0x288
#define IOPV_OFFSET 0x2a0
#define IOIE_OFFSET 0x2b8

#define IO(_name, _reg, _bit)               \
	static struct io io_##_name = {     \
		.name = __stringify(_name), \
		.reg = _reg,                \
		.bit = _bit,                \
	}

IO(agpio0, IOREG2, 0);
IO(agpio1, IOREG3, 31);
IO(agpio2, IOREG3, 30);
IO(agpio3, IOREG3, 29);
IO(agpio4, IOREG3, 28);
IO(agpio5, IOREG3, 27);
IO(agpio6, IOREG3, 26);
IO(agpio7, IOREG3, 25);
IO(agpio8, IOREG3, 24);
IO(agpio9, IOREG3, 23);
IO(agpio10, IOREG3, 22);
IO(agpio11, IOREG3, 21);
IO(agpio12, IOREG3, 20);
IO(agpio13, IOREG3, 19);
IO(agpio14, IOREG4, 15);
IO(agpio15, IOREG4, 14);
IO(agpio16, IOREG3, 18);
IO(agpio17, IOREG3, 17);
IO(agpio18, IOREG3, 16);
IO(agpio19, IOREG3, 15);
IO(agpio20, IOREG5, 0);
IO(agpio21, IOREG6, 31);
IO(agpio22, IOREG6, 30);
IO(agpio23, IOREG6, 29);
IO(agpio26, IOREG3, 14);
IO(agpio27, IOREG3, 13);
IO(agpio28, IOREG3, 12);
IO(agpio29, IOREG3, 11);
IO(agpio30, IOREG4, 13);
IO(agpio31, IOREG4, 12);

IO(bgpio0, IOREG5, 11);
IO(bgpio1, IOREG5, 10);
IO(bgpio2, IOREG5, 9);
IO(bgpio3, IOREG5, 8);
IO(bgpio4, IOREG4, 11);
IO(bgpio5, IOREG4, 9);
IO(bgpio6, IOREG4, 10);
IO(bgpio7, IOREG4, 8);
IO(bgpio8, IOREG5, 7);
IO(bgpio9, IOREG5, 6);
IO(bgpio10, IOREG5, 5);
IO(bgpio11, IOREG5, 18);
IO(bgpio12, IOREG5, 4);
IO(bgpio13, IOREG5, 3);
IO(bgpio14, IOREG5, 2);
IO(bgpio16, IOREG5, 1);
IO(bgpio17, IOREG5, 29);
IO(bgpio18, IOREG5, 28);
IO(bgpio19, IOREG5, 27);
IO(bgpio20, IOREG5, 26);
IO(bgpio21, IOREG5, 25);
IO(bgpio22, IOREG5, 24);
IO(bgpio23, IOREG5, 23);
IO(bgpio24, IOREG5, 22);
IO(bgpio25, IOREG5, 21);
IO(bgpio26, IOREG1, 24);
IO(bgpio27, IOREG4, 16);
IO(bgpio28, IOREG4, 7);
IO(bgpio29, IOREG4, 5);
IO(bgpio30, IOREG4, 6);
IO(bgpio31, IOREG5, 17);

IO(cgpio0, IOREG1, 14);
IO(cgpio1, IOREG2, 27);
IO(cgpio2, IOREG1, 6);
IO(cgpio3, IOREG2, 22);
IO(cgpio4, IOREG1, 3);
IO(cgpio5, IOREG1, 11);
IO(cgpio6, IOREG2, 29);
IO(cgpio7, IOREG2, 31);
IO(cgpio8, IOREG1, 22);
IO(cgpio9, IOREG1, 18);
IO(cgpio10, IOREG1, 16);
IO(cgpio11, IOREG1, 10);
IO(cgpio12, IOREG2, 25);
IO(cgpio13, IOREG1, 27);
IO(cgpio14, IOREG1, 26);
IO(cgpio15, IOREG1, 8);
IO(cgpio16, IOREG1, 21);
IO(cgpio17, IOREG1, 20);
IO(cgpio18, IOREG1, 29);
IO(cgpio19, IOREG1, 25);
IO(cgpio20, IOREG2, 21);
IO(cgpio21, IOREG1, 9);
IO(cgpio22, IOREG1, 4);
IO(cgpio23, IOREG2, 24);
IO(cgpio24, IOREG1, 23);
IO(cgpio25, IOREG2, 23);
IO(cgpio26, IOREG1, 15);
IO(cgpio27, IOREG1, 17);
IO(cgpio28, IOREG1, 2);
IO(cgpio29, IOREG2, 28);
IO(cgpio30, IOREG1, 12);
IO(cgpio31, IOREG1, 28);

IO(d_class_clk, IOREG4, 17);
IO(d_class_pwmn, IOREG4, 18);

IO(dgpio0, IOREG1, 31);
IO(dgpio1, IOREG2, 26);
IO(dgpio2, IOREG1, 19);
IO(dgpio3, IOREG1, 7);
IO(dgpio4, IOREG1, 5);
IO(dgpio5, IOREG1, 13);
IO(dgpio6, IOREG1, 30);
IO(dgpio7, IOREG1, 1);
IO(dgpio8, IOREG2, 20);
IO(dgpio9, IOREG2, 19);
IO(dgpio10, IOREG1, 0);
IO(dgpio11, IOREG2, 30);
IO(dgpio12, IOREG4, 25);
IO(dgpio13, IOREG4, 24);
IO(dgpio14, IOREG4, 28);
IO(dgpio15, IOREG4, 23);
IO(dgpio16, IOREG4, 22);
IO(dgpio17, IOREG4, 21);
IO(dgpio18, IOREG4, 27);
IO(dgpio19, IOREG4, 26);
IO(dgpio20, IOREG4, 2);
IO(dgpio21, IOREG4, 1);
IO(dgpio22, IOREG4, 0);
IO(dgpio23, IOREG5, 31);
IO(dgpio24, IOREG5, 20);
IO(dgpio25, IOREG5, 19);
IO(dgpio26, IOREG5, 30);
IO(dgpio30, IOREG4, 20);
IO(dgpio31, IOREG4, 19);


IO(egpio0, IOREG2, 18);
IO(egpio1, IOREG2, 17);
IO(egpio2, IOREG2, 16);
IO(egpio3, IOREG2, 15);
IO(egpio4, IOREG2, 14);
IO(egpio5, IOREG2, 13);
IO(egpio6, IOREG2, 12);
IO(egpio7, IOREG2, 11);
IO(egpio8, IOREG2, 10);
IO(egpio9, IOREG2, 9);
IO(egpio10, IOREG2, 8);
IO(egpio11, IOREG2, 7);
IO(egpio12, IOREG2, 6);
IO(egpio13, IOREG2, 5);
IO(egpio14, IOREG2, 4);
IO(egpio15, IOREG2, 3);
IO(egpio16, IOREG2, 2);
IO(egpio17, IOREG2, 1);
IO(egpio18, IOREG3, 10);
IO(egpio19, IOREG3, 9);
IO(egpio20, IOREG3, 8);
IO(egpio21, IOREG3, 7);
IO(egpio22, IOREG3, 6);
IO(egpio23, IOREG3, 5);
IO(egpio24, IOREG3, 4);
IO(egpio25, IOREG3, 3);
IO(egpio26, IOREG3, 2);
IO(egpio27, IOREG3, 1);
IO(egpio28, IOREG3, 0);
IO(egpio29, IOREG4, 31);
IO(egpio30, IOREG4, 30);
IO(egpio31, IOREG4, 29);

IO(if_clk_if, IOREG4, 4);

static struct io *ios[] = {
	&io_agpio0,
	&io_agpio1,
	&io_agpio2,
	&io_agpio3,
	&io_agpio4,
	&io_agpio5,
	&io_agpio6,
	&io_agpio7,
	&io_agpio8,
	&io_agpio9,
	&io_agpio10,
	&io_agpio11,
	&io_agpio12,
	&io_agpio13,
	&io_agpio14,
	&io_agpio15,
	&io_agpio16,
	&io_agpio17,
	&io_agpio18,
	&io_agpio19,
	&io_agpio20,
	&io_agpio21,
	&io_agpio22,
	&io_agpio23,
	&io_agpio26,
	&io_agpio27,
	&io_agpio28,
	&io_agpio29,
	&io_agpio30,
	&io_agpio31,

	&io_bgpio0,
	&io_bgpio1,
	&io_bgpio2,
	&io_bgpio3,
	&io_bgpio4,
	&io_bgpio5,
	&io_bgpio6,
	&io_bgpio7,
	&io_bgpio8,
	&io_bgpio9,
	&io_bgpio10,
	&io_bgpio11,
	&io_bgpio12,
	&io_bgpio13,
	&io_bgpio14,
	&io_bgpio16,
	&io_bgpio17,
	&io_bgpio18,
	&io_bgpio19,
	&io_bgpio20,
	&io_bgpio21,
	&io_bgpio22,
	&io_bgpio23,
	&io_bgpio24,
	&io_bgpio25,
	&io_bgpio26,
	&io_bgpio27,
	&io_bgpio28,
	&io_bgpio29,
	&io_bgpio30,
	&io_bgpio31,

	&io_cgpio0,
	&io_cgpio1,
	&io_cgpio2,
	&io_cgpio3,
	&io_cgpio4,
	&io_cgpio5,
	&io_cgpio6,
	&io_cgpio7,
	&io_cgpio8,
	&io_cgpio9,
	&io_cgpio10,
	&io_cgpio11,
	&io_cgpio12,
	&io_cgpio13,
	&io_cgpio14,
	&io_cgpio15,
	&io_cgpio16,
	&io_cgpio17,
	&io_cgpio18,
	&io_cgpio19,
	&io_cgpio20,
	&io_cgpio21,
	&io_cgpio22,
	&io_cgpio23,
	&io_cgpio24,
	&io_cgpio25,
	&io_cgpio26,
	&io_cgpio27,
	&io_cgpio28,
	&io_cgpio29,
	&io_cgpio30,
	&io_cgpio31,

	&io_d_class_clk,
	&io_d_class_pwmn,

	&io_dgpio0,
	&io_dgpio1,
	&io_dgpio2,
	&io_dgpio3,
	&io_dgpio4,
	&io_dgpio5,
	&io_dgpio6,
	&io_dgpio7,
	&io_dgpio8,
	&io_dgpio9,
	&io_dgpio10,
	&io_dgpio11,
	&io_dgpio12,
	&io_dgpio13,
	&io_dgpio14,
	&io_dgpio15,
	&io_dgpio16,
	&io_dgpio17,
	&io_dgpio18,
	&io_dgpio19,
	&io_dgpio20,
	&io_dgpio21,
	&io_dgpio22,
	&io_dgpio23,
	&io_dgpio24,
	&io_dgpio25,
	&io_dgpio26,
	&io_dgpio30,
	&io_dgpio31,

	&io_egpio0,
	&io_egpio1,
	&io_egpio2,
	&io_egpio3,
	&io_egpio4,
	&io_egpio5,
	&io_egpio6,
	&io_egpio7,
	&io_egpio8,
	&io_egpio9,
	&io_egpio10,
	&io_egpio11,
	&io_egpio12,
	&io_egpio13,
	&io_egpio14,
	&io_egpio15,
	&io_egpio16,
	&io_egpio17,
	&io_egpio18,
	&io_egpio19,
	&io_egpio20,
	&io_egpio21,
	&io_egpio22,
	&io_egpio23,
	&io_egpio24,
	&io_egpio25,
	&io_egpio26,
	&io_egpio27,
	&io_egpio28,
	&io_egpio29,
	&io_egpio30,
	&io_egpio31,

	&io_if_clk_if,
};

static struct io *
io_get(const char *id)
{
	int i;

	for (i = 0; i < ARRAY_SIZE(ios); i++) {
		if (!strcmp(id, ios[i]->name))
			return ios[i];
	}

	return NULL;
}

void
io_drive_strength(const char *id, int strength)
{
	int tmp, bit, reg;
	struct io *io = io_get(id);

	if (!io || strength > 3)
		return;

	reg = io->reg * 2;
	bit = io->bit * 2;

	/* Changed by Grandsteam */
	if (io->bit > 15) {
		bit -= 32;
	} else {
		reg += 4;
	}

	tmp = readl(DVF101_SYSCFG_BASE + reg + IODS_OFFSET);
	tmp &= ~(3 << bit);
	tmp |= strength << bit;
	writel(tmp, DVF101_SYSCFG_BASE + reg + IODS_OFFSET);
}

void
io_pull(const char *id, int enable, int up)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF101_SYSCFG_BASE + io->reg + IOPV_OFFSET);
	if (up)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF101_SYSCFG_BASE + io->reg + IOPV_OFFSET);

	tmp = readl(DVF101_SYSCFG_BASE + io->reg + IOPE_OFFSET);
	if (enable)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF101_SYSCFG_BASE + io->reg + IOPE_OFFSET);
}

void
io_slew_rate(const char *id, int slow)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF101_SYSCFG_BASE + io->reg + IOSR_OFFSET);
	if (slow)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF101_SYSCFG_BASE + io->reg + IOSR_OFFSET);
}

void
io_keeper(const char *id, int enable)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF101_SYSCFG_BASE + io->reg + IOKE_OFFSET);
	if (enable)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF101_SYSCFG_BASE + io->reg + IOKE_OFFSET);
}

void
io_input_enable(const char *id, int enable)
{
	int tmp;
	struct io *io = io_get(id);

	if (!io)
		return;

	tmp = readl(DVF101_SYSCFG_BASE + io->reg + IOIE_OFFSET);
	if (enable)
		tmp |=   1 << io->bit;
	else
		tmp &= ~(1 << io->bit);
	writel(tmp, DVF101_SYSCFG_BASE + io->reg + IOIE_OFFSET);
}
