/*
 * (C) Copyright 2009
 * Shlomi Mor, DSP Group, shlomi.mor@dspg.com
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <video_fb.h>
#include <lcd.h>
#include <asm/io.h>
#include <asm/arch/hardware.h>
#include <asm/arch/clock.h>
#include <asm/arch/lcd.h>
/*#include "dspg_logo.h"*/

DECLARE_GLOBAL_DATA_PTR;

GraphicDevice gdev;

static const struct dvf101_lcdc_panel *dvf101_panel = NULL;

extern const struct dvf101_lcdc_panel *dvf101_panels[];

static ushort colormap[256];

vidinfo_t panel_info = {
	.vl_col = 800,
	.vl_row = 480,
	.vl_bpix = LCD_COLOR16,
	.cmap = colormap,
};

void *dvf101_lcd_base;		/* Start of framebuffer memory	*/
void *lcd_console_address;	/* Start of console buffer	*/

int lcd_color_fg;
int lcd_color_bg;

short console_col;
short console_row;

static struct dvf101_lcdc_seq dvf101_lcdc_hw_init_seq[] = {
	{ LCDC_REG_GCER,        0x0000 }, /* disable gamma correction */
	{ LCDC_REG_CLPER,       0x0000 }, /* disable clipping */
	{ LCDC_REG_INTMR,       0x0000 }, /* 5:6:5 rgb input */
	{ LCDC_REG_OSDMCR,      0x0000 },
	{ LCDC_REG_DISPIDXR,    0x0000 },

	{ LCDC_REG_BACKCPR,     0x001F }, /* blue background */
	{ LCDC_REG_RBCR,        0x0080 }, /* no color clipping */
	{ LCDC_REG_GBCR,        0x0080 },
	{ LCDC_REG_BBCR,        0x0080 },

	{ LCDC_REG_PALR0,       0x0000 },
	{ LCDC_REG_PALR1,       0x0000 },
	{ LCDC_REG_PALR2,       0x0000 },
	{ LCDC_REG_PALR3,       0x0000 },

	{ LCDC_REG_DISPYSPOS2R, 0x0000 }, /* only used for CCIR656 */
	{ LCDC_REG_DISPYEPOS2R, 0x0000 }, /* only used for CCIR656 */

	{ LCDC_REG_GPSELR,      0x0000 },

	{ LCDC_REG_DISPCR,      0x0001 }, /* enable display */

	{ LCDC_SEQ_END,         0x0000 },
};

static inline void dvf101_lcdc_writel(unsigned long addr, unsigned long val)
{
	writel(val, addr);
}

static void dvf101_lcdc_run_sequence(const struct dvf101_lcdc_seq *seq)
{
	struct dvf101_lcdc_seq *cmd = (struct dvf101_lcdc_seq *)seq;

	while (cmd->addr != LCDC_SEQ_END) {
		writel(cmd->value, cmd->addr);
		cmd++;
	}
}

static void dvf101_lcdc_hw_disable(void)
{
	dvf101_lcdc_writel(LCDC_REG_LCDCCR, 0);
}

static void dvf101_lcdc_hw_enable(void)
{
	dvf101_lcdc_writel(LCDC_REG_LCDCCR, 1);
}

static void dvf101_lcdc_hw_param_update(void)
{
	dvf101_lcdc_writel(LCDC_REG_PARUP, 1);
}

static void dvf101_lcdc_set_clock_rate(const struct dvf101_lcdc_panel *panel)
{
	if (panel->clock_rate)
		clk_set_rate("lcdc_clk", panel->clock_rate);
}

static void dvf101_lcdc_hw_init_panel(const struct dvf101_lcdc_panel *panel)
{
	/* input data transfer sizes */
	dvf101_lcdc_writel(LCDC_REG_INDTR, ((panel->xres * NBITS(panel->bits_per_pixel))/32) - 1);
	dvf101_lcdc_writel(LCDC_REG_INDXSR, panel->xres - 1);
	dvf101_lcdc_writel(LCDC_REG_INDYSR, panel->yres - 1);

	dvf101_lcdc_writel(LCDC_REG_VATR,   panel->yres - 1);
	dvf101_lcdc_writel(LCDC_REG_HADSTR, panel->xres - 1);
	dvf101_lcdc_writel(LCDC_REG_HAPWR,  panel->xres - 1);

	/* display position */
	dvf101_lcdc_writel(LCDC_REG_DISPXSPOSR,  0);
	dvf101_lcdc_writel(LCDC_REG_DISPXEPOSR,  panel->xres - 1);
	dvf101_lcdc_writel(LCDC_REG_DISPYSPOS1R, 0);
	dvf101_lcdc_writel(LCDC_REG_DISPYEPOS1R, panel->yres - 1);

	/* input buffer */
	dvf101_lcdc_writel(LCDC_REG_MSBAHBA0R, ((unsigned long)dvf101_lcd_base) >> 16);
	dvf101_lcdc_writel(LCDC_REG_LSBAHBA0R, ((unsigned long)dvf101_lcd_base) & 0xffff);
	dvf101_lcdc_writel(LCDC_REG_OFFAR0, (panel->xres * NBITS(panel->bits_per_pixel))/8);

	dvf101_lcdc_writel(LCDC_REG_INTER, 0x4);

	dvf101_lcdc_writel(LCDC_REG_MSBAHBA1R, 0);
	dvf101_lcdc_writel(LCDC_REG_LSBAHBA1R, 0);
	dvf101_lcdc_writel(LCDC_REG_OFFAR1,    0);

	dvf101_lcdc_run_sequence(panel->init_seq);

	if (panel->init_fn)
		panel->init_fn();
}

void dvf101_lcdc_init_colors(const struct dvf101_lcdc_panel *panel)
{
	int i, num_words;
	unsigned long *buf = dvf101_lcd_base;

	num_words = (panel->xres * panel->yres *
		     (NBITS(panel->bits_per_pixel) / 8)) /
		    sizeof(unsigned long);

#ifdef DSPG_SPLASH
	for (i = 0; i < num_words; i++)
		*buf++ = ((unsigned long *)splash_raw_data)[i];
#else
	for (i = 0; i < num_words / 3; i++)
		*buf++ = 0xf800f800; /* red */

	for (i = 0; i < num_words / 3; i++)
		*buf++ = 0x07e007e0; /* green */

	for (i = 0; i < num_words / 3; i++)
		*buf++ = 0x001f001f; /* blue */
#endif
}

void lcd_initcolregs(void)
{
}

void lcd_setcolreg(ushort regno, ushort red, ushort green, ushort blue)
{
}

void lcd_enable(void)
{
	if (dvf101_panel) {
		unsigned long image_size = dvf101_panel->xres * dvf101_panel->yres *
		                           (NBITS(dvf101_panel->bits_per_pixel) / 8);

		memset(dvf101_lcd_base, 0, image_size);

		dvf101_lcdc_init_colors(dvf101_panel);

		dvf101_lcdc_set_clock_rate(dvf101_panel);

		dvf101_lcdc_hw_disable();

		dvf101_lcdc_run_sequence(dvf101_lcdc_hw_init_seq);

		dvf101_lcdc_hw_init_panel(dvf101_panel);

		dvf101_lcdc_hw_enable();

		dvf101_lcdc_hw_param_update();
	}
}

void lcd_ctrl_init(void *lcdbase)
{
	int i = 0;
	char *lcdc_type;

	if (!lcdbase)
		dvf101_lcd_base = (void *)CONFIG_DSPG_FB_ADDR;
	else
		dvf101_lcd_base = lcdbase;

	lcdc_type = getenv("lcdctype");
	if (!lcdc_type)
		return;

	do {
		dvf101_panel = dvf101_panels[i++];
		if (!strcmp(lcdc_type, dvf101_panel->name)) {
			panel_info.vl_col  = dvf101_panel->xres;
			panel_info.vl_row  = dvf101_panel->yres;
			panel_info.vl_bpix = dvf101_panel->bits_per_pixel;

			return;
		}
	} while (dvf101_panels[i]);
}

ulong calc_fbsize(void)
{
	return ((panel_info.vl_col * panel_info.vl_row *
		NBITS(panel_info.vl_bpix)) / 8) + PAGE_SIZE;
}

void *video_hw_init(void)
{
	if (dvf101_panel) {
		printf("LCD:   %s\n", dvf101_panel->name);

		gdev.frameAdrs = (unsigned long)dvf101_lcd_base;
		gdev.winSizeX = dvf101_panel->xres;
		gdev.winSizeY = dvf101_panel->yres;
		gdev.gdfBytesPP = NBITS(dvf101_panel->bits_per_pixel) / 8;
		gdev.gdfIndex = GDF_16BIT_565RGB;

		return (void *)&gdev;
	}

	return NULL;
}

void video_set_lut(unsigned int index, /* color number */
                   unsigned char r,    /* red */
                   unsigned char g,    /* green */
                   unsigned char b)    /* blue */
{
	/* nothing to do here */
}
