#ifndef _CRYPTO_BLOCK_H
#define _CRYPTO_BLOCK_H

#include <config.h>
#include <asm/io.h>
#include <asm/arch/io.h>
#include <asm/arch/hardware.h>

#define PMU_DAIF_EFUSE_CTRL	0x06300100

#define CRYPTO_CFG			0x000
#define CRYPTO_DINPTR			0x038
#define CRYPTO_DOUTPTR			0x03c
#define CRYPTO_DINLEN			0x040
#define CRYPTO_DOUTLEN			0x044
#define CRYPTO_OTP_STATE		0x304
#define CRYPTO_OTP_MINSWVER		0x308
#define CRYPTO_OTP_DEVID		0x300
#define CRYPTO_OTP_PUBKEY_DIGEST_0	0x320
#define CRYPTO_OTP_PUBKEY_DIGEST_1	0x324
#define CRYPTO_OTP_PUBKEY_DIGEST_2	0x328
#define CRYPTO_OTP_PUBKEY_DIGEST_3	0x32c
#define CRYPTO_OTP_PUBKEY_DIGEST_4	0x330
#define CRYPTO_OTP_PUBKEY_DIGEST_5	0x334
#define CRYPTO_OTP_PUBKEY_DIGEST_6	0x338
#define CRYPTO_OTP_PUBKEY_DIGEST_7	0x33c
#define CRYPTO_OTP_PROGREQ		0x340
#define CRYPTO_OTP_PROGKICK		0x344
#define CRYPTO_OTP_PROGSTAT		0x348
#define CRYPTO_OTP_DEBUG		0x34c

#define CRYPTO_OTP_REQ_BANKSEL_0	(0 << 0) /* state bits */
#define CRYPTO_OTP_REQ_BANKSEL_1	(1 << 0) /* public key 0 */
#define CRYPTO_OTP_REQ_BANKSEL_3	(3 << 0) /* public key 1 */
#define CRYPTO_OTP_REQ_BANKSEL_4	(4 << 0) /* public key 2 */
#define CRYPTO_OTP_REQ_BANKSEL_5	(5 << 0) /* secret key 1 */
#define CRYPTO_OTP_REQ_GEN_SECKEY	(1 << 3)
#define CRYPTO_OTP_REQ_B0_OFFSET_DEVICE_ID	(0 << 4)
#define CRYPTO_OTP_REQ_B0_OFFSET_STATE	(1 << 4)
#define CRYPTO_OTP_REQ_B0_OFFSET_SW_VER	(2 << 4)
#define CRYPTO_OTP_REQ_UPDATE		(1 << 8)
#define CRYPTO_OTP_REQ_PROGRAM		(0 << 8)

#define CRYPTO_OTP_KICK_MAGIC		0xabcdef12

#define CRYPTO_OTP_STAT_COMPLETE	(1 << 1)
#define CRYPTO_OTP_STAT_SUCCESS		(1 << 0)

#define CRYPTO_STATE_SECRET_BURNED	(1 << 0)
#define CRYPTO_STATE_PUBLIC_KEY_BURNED	(1 << 1)
#define CRYPTO_STATE_RE_ENABLE		(1 << 2)
#define CRYPTO_STATE_SW_ROLLBACK_PROT	(1 << 3)
#define CRYPTO_STATE_ENFORCE_SB		(1 << 4)
#define CRYPTO_STATE_DISABLE_JTAG_IN_SB	(1 << 5)
#define CRYPTO_STATE_PUBLIC_KEY1_BURNED	(1 << 6)
#define CRYPTO_STATE_PUBLIC_KEY2_BURNED	(1 << 7)
#define CRYPTO_STATE_SECRET1_BURNED	(1 << 8)
#define CRYPTO_STATE_PUBLIC_KEY_SELECT	(3 << 9)
#define CRYPTO_STATE_PK_SELECT_BURN_DIS	(1 << 11)

#define CRYPTO_OTP_DEBUG_CNTL_READY	(1 << 4)
#define CRYPTO_OTP_DEBUG_BANK0_BURNED	(1 << 8)
#define CRYPTO_OTP_DEBUG_BANK1_BURNED	(1 << 9)
#define CRYPTO_OTP_DEBUG_SECKEY_BURNED	(1 << 10)

#define CRYPTO_KEYSTORE_79		0x17e0

struct cryptohw_desc {
	uint32_t desc_ctl;		/* Descriptor control */
	uint32_t link_ptr;		/* 32-bit pointer to next descriptor */
	uint32_t data_in_phys;		/* 32-bit pointer to input data */
	uint32_t data_out_phys;		/* 32-bit pointer to output data */
	uint32_t data_in_len;		/* 31-bit length of input data */
	uint32_t data_out_len;		/* 31-bit length of output data */
	uint32_t iv[8];			/* Initialization vector */
	uint32_t state_index;
	uint32_t data_in_total_len;	/* Full packet data length if last descriptor */
					/* of multiple-descriptor packet of HASH/HMAC */
};

#define CRYPTO_DESC_CTL_DESC_OWNER_HW	0x80000000
#define CRYPTO_DESC_CTL_MODE_SHA2_256	0x22
#define CRYPTO_REG_CHANNEL_HEADPTR	0x0200
#define CRYPTO_REG_HEADPTR_SET		0x0284

static inline void crypto_writel(unsigned long reg, unsigned long val)
{
	writel(val, DVF101_CRYPTO_BASE + reg);
}

static inline unsigned long crypto_readl(unsigned long reg)
{
	return readl(DVF101_CRYPTO_BASE + reg);
}

int crypto_sha256(const unsigned char* message, unsigned long len,
		  unsigned char *digest);

int crypto_is_dev_secured(void);

int crypto_otp_update_regs(void);

void crypto_otp_get_pubkey_digest(unsigned long pubkey_digest[8]);
int crypto_otp_set_pubkey_digest(unsigned long pubkey_digest[8]);

unsigned long crypto_otp_get_state(void);
int crypto_otp_set_state(unsigned long val);

unsigned long crypto_otp_get_minver(void);
int crypto_otp_set_minver(unsigned long val);

unsigned long crypto_otp_get_devid(void);
int crypto_otp_set_devid(unsigned long val);

unsigned long crypto_otp_get_debug(void);

int crypto_otp_gen_seckey(void);

#endif /* _CRYPTO_BLOCK_H */
