#ifndef __DSPG_IMAGE_H__
#define __DSPG_IMAGE_H__

#include <config.h>
#include <linux/types.h>

struct pubkey_t {
	uint8_t n[256];			/* public modulus (0x24)*/
	uint32_t e;			/* public exponent (0x124)*/
} __attribute__ ((packed));

struct secureboot_header {
	struct pubkey_t pubkey;		/* public key (0x24)*/
	uint8_t pre_sha256[32];		/* preloader sha256 digest (0x128)*/
	uint8_t load_sha256[32];	/* bootloader sha256 digest (0x148)*/
	uint8_t signature[256];		/* header signature (0x168)*/
} __attribute__ ((packed));

struct dspg_image_header {
	uint32_t size;			/* size of this header + 4byte crc32 (0x0)*/
	uint32_t flags;			/* optional flags (0x4)*/
	uint32_t pre_offset;		/* preloader: starts at offset (0x8)*/
	uint32_t pre_size;		/* preloader: size of preloader (0xc)*/
	uint32_t pre_crc32;		/* preloader: crc32 checksum (0x10)*/
	uint32_t load_offset;		/* bootloader: start at offset (0x14)*/
	uint32_t load_size;		/* bootloader: size of bootloader (0x18)*/
	uint32_t load_addr;		/* bootloader: load to address (0x1c)*/
	uint32_t load_crc32;		/* bootloader: crc32 checksum (0x20)*/
	struct secureboot_header sb_hdr;/* secureboot header (0x24)*/
	uint32_t crc32;			/* crc32 of all above fields */
} __attribute__ ((packed));

#define HDRFLAG_CRC32	0x00000001
#define HDRFLAG_SECURE	0x00000002

enum sboot_errors {
	ERR_NONE = 0,
	ERR_SBOOT_NOSIG,
	ERR_SBOOT_INVPUBKEY,
	ERR_SBOOT_INVSIG,
	ERR_SBOOT_INVIMG,
	NUM_SBOOT_ERRORS,
};

#endif
