/*
 * (C) Copyright 2016 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

#include <syndwc_mmc.h>
#include <mmc.h>
#include <part.h>

#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/io.h>
#include <asm/arch/hardware.h>

static int
mmc_get_card_detect(void)
{
	return 1;
}

static struct syndwc_pdata syndwc_pdata = {
	.card_num    = 0,
	.mclk        = 25000000,
	.f_max       = 25000000,
	.host_caps   = MMC_MODE_4BIT,
	.voltages    = MMC_VDD_32_33 | MMC_VDD_33_34,
	.card_detect = &mmc_get_card_detect,
};

void
mmc_max_host_caps(struct mmc *mmc)
{
	syndwc_pdata.mclk = 50000000,
	syndwc_pdata.f_max = 50000000,
	syndwc_pdata.host_caps |= MMC_MODE_HS | MMC_MODE_HS_52MHz |
				  MMC_MODE_DDR_52MHz | MMC_MODE_8BIT;

	syndwc_set_conf(&syndwc_pdata);

	clk_disable("emmc_clk");
	clk_set_rate("emmc_pre", 500000000);
	clk_set_rate("emmc_clk", syndwc_pdata.mclk);
	clk_enable("emmc_clk");

	mmc->has_init = 0;
	mmc_init(mmc);
}

#ifdef CONFIG_EMMC
int
mmc_get_env_addr(struct mmc *mmc, int copy, u32 *env_addr)
{
	int ret, i = 1;
	unsigned boot_part;
	disk_partition_t info;

	/* search bootable partition first */
	boot_part = (mmc->part_config >> 3) & 0x7;
	if (boot_part == 1 || boot_part == 2) {
		printf("switching to boot partition %u\n", boot_part);
		mmc_select_hwpart(mmc->block_dev.dev, boot_part);
	}

	/* try to find the u-boot environment partition */
	do {
		ret = get_partition_info(&mmc->block_dev, i++, &info);
		if (!ret && !strcmp((const char *)info.type, BOOT_PART_ENV)) {
			if (info.size * 512 < CONFIG_ENV_SIZE) {
				printf("ERROR: Environment partition too small!\n");
				return 1;
			}
			*env_addr = info.start * 512;
			printf("found evironment partition at %8.8x\n", *env_addr);
			return 0;
		}
	} while (ret == 0);

	printf("ERROR: No environment partition found! "
	       "Use gptpart to create one...\n");

	return 1;
}
#endif /* CONFIG_EMMC */

int board_mmc_init(bd_t *bis)
{
	clk_disable("emmc_clk");
	/* set drive clock shift to 1 */
	sdmmc_set_phase(1, 0, 1);
	/* set sample clock shift to 4 */
	sdmmc_set_phase(0, 0, 4);
	clk_set_rate("emmc_pre", 100000000);
	clk_set_rate("emmc_clk", syndwc_pdata.mclk);
	reset_release("emmc_clk");
	clk_enable("emmc_clk");

	/* configure pins */
	io_pull("agpio1", 1, 1);
	io_pull("agpio7", 1, 1);
	io_slew_rate("agpio7", 1);
	io_slew_rate("agpio7", 1);
	io_mux("emmc_clk");
	io_mux("emmc_cmd");
	io_mux("emmc_d0");

	io_pull("agpio8", 1, 1);
	io_pull("agpio9", 1, 1);
	io_pull("agpio10", 1, 1);
	io_pull("agpio11", 1, 1);
	io_pull("agpio12", 1, 1);
	io_pull("agpio13", 1, 1);
	io_pull("agpio0", 1, 1);
	io_slew_rate("agpio8", 1);
	io_slew_rate("agpio9", 1);
	io_slew_rate("agpio10", 1);
	io_slew_rate("agpio11", 1);
	io_slew_rate("agpio12", 1);
	io_slew_rate("agpio13", 1);
	io_slew_rate("agpio0", 1);

	io_mux("emmc_d1");
	io_mux("emmc_d2");
	io_mux("emmc_d3");
	io_mux("emmc_d4");
	io_mux("emmc_d5");
	io_mux("emmc_d6");
	io_mux("emmc_d7");

	return syndwc_initialize(&syndwc_pdata);
}
