/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <config.h>
#include <asm/io.h>
#include <asm/arch/ccu.h>
#include <asm/arch/clock.h>
#include <asm/arch/lcd.h>
#include <asm/arch/i2c.h>
#include <asm/arch/spi.h>
#include <asm/arch/tdm.h>
#include <asm/arch/apu.h>
#include <asm/arch/serial.h>
#include <asm/arch/hardware.h>
#include <asm/arch/keypad.h>
#include <asm/arch/bmp.h>
#include <asm/arch/rf_spif.h>
#include <i2c.h>
#include <lcd.h>

DECLARE_GLOBAL_DATA_PTR;

int board_early_init_f(void)
{
	/* DVF101's data cache is not in a clean state after power cycle */
	invalidate_dcache_all();

	board_serial_init();
	board_apu_init();

	return 0;
}

int checkboard(void)
{
	puts("BOARD:" CONFIG_IDENT_STRING "\n");

	return 0;
}

int board_init(void)
{
	ccu_init();

	icache_enable();

	gd->bd->bi_boot_params = gd->bd->bi_dram[0].start + 0x100;

	board_spi_init();
	board_i2c_init(gd->fdt_blob);
	board_keypad_init();
	board_lcd_init();
	board_tdm_init();
	board_bmp_init();
	board_rf_spif_init();

	return 0;
}

int board_late_init(void)
{
	lcd_ctrl_init(0);
	lcd_enable();
	dvf_serial_late_init();

	return 0;
}

#if defined(CONFIG_OF_LIBFDT) && defined(CONFIG_OF_BOARD_SETUP)
#include <fdt_support.h>

static void
evb101_sdio_setup(void *blob, bd_t *bd)
{
	int s_shift = -1, d_shift = -1;
	char *s_shift_env, *d_shift_env;

	s_shift_env = getenv("sdio_s_shift");
	d_shift_env = getenv("sdio_d_shift");

	if (s_shift_env)
		s_shift = (int)simple_strtoul(s_shift_env, NULL, 10);
	if (d_shift_env)
		d_shift = (int)simple_strtoul(d_shift_env, NULL, 10);

	if (s_shift >= 0)
		do_fixup_by_compat_u32(blob, "dspg,dw-mmc-evb101",
				       "dspg,shift-s", s_shift, 1);
	if (d_shift >= 0)
		do_fixup_by_compat_u32(blob, "dspg,dw-mmc-evb101",
				       "dspg,shift-d", d_shift, 1);
}

int
ft_board_setup(void *blob, bd_t *bd)
{
	evb101_sdio_setup(blob, bd);

	return 0;
}
#endif
