/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>
#ifdef CONFIG_RTL83XX_SWITCH
#include <rtk_api_ext.h>
#include <rtl8367b_asicdrv_port.h>
#endif

#if defined(CONFIG_PHY8710) && defined (CONFIG_RTL83XX_SWITCH)
	#error Both switch and PHY are enabled!
#endif

#if defined(CONFIG_SLVMII)
	#define ETH_SYSCFG_MODE		0
	#define ETH_CLK_MODE		ETH_SLVMII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_MII_MAC /* ? */
#elif defined(CONFIG_RMII)
	#ifdef CONFIG_CHIP_RTL8365MB
		#error RTL8365MB does not support RMII!
	#endif
	#define ETH_SYSCFG_MODE		4
	#define ETH_CLK_MODE		ETH_RMII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_RMII_MAC
#elif defined(CONFIG_SLVRMII)
	#ifdef CONFIG_CHIP_RTL8365MB
		#error RTL8365MB does not support SLVRMII!
	#endif
	#define ETH_SYSCFG_MODE		4
	#define ETH_CLK_MODE		ETH_SLVRMII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_RMII_MAC /* ? */
#elif defined(CONFIG_RGMII)
	#define ETH_SYSCFG_MODE		1
	#define ETH_CLK_MODE		ETH_RGMII
	#define ETH_CLK_SPEED		ETH_SPEED_1000
	#define RTK_SPEED		SPD_1000M
	#define RTK_MODE		MODE_EXT_RGMII /* MODE_EXT_RGMII_33V */
#else
	/* Use MII by default (CONFIG_MII is used for other purposes so we don't check it) */
	#define ETH_SYSCFG_MODE		0
	#define ETH_CLK_MODE		ETH_MII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_MII_PHY
#endif

/* optional: override default ETH speed */
#ifdef CONFIG_ETH_SPEED
	#undef ETH_CLK_SPEED
	#undef RTK_SPEED
	#if (CONFIG_ETH_SPEED == 10)
		#define ETH_CLK_SPEED	ETH_SPEED_10
		#define RTK_SPEED	SPD_10M
	#elif (CONFIG_ETH_SPEED == 100)
		#define ETH_CLK_SPEED	ETH_SPEED_100
		#define RTK_SPEED	SPD_100M
	#elif (CONFIG_ETH_SPEED == 1000)
		#define ETH_CLK_SPEED	ETH_SPEED_1000
		#define RTK_SPEED	SPD_1000M
	#else
		#error Wrong CONFIG_ETH_SPEED value
	#endif
#endif

int gmac_eth_register(int id, void *regs, int ignore_phy, u32 speed, u32 clk_csr_i);

static void init_eth_gpios(enum eth_mode mode)
{
	/* all modes */
	gpio_disable(EGPIO(0));  /* TXD0 */
	gpio_disable(EGPIO(1));  /* TXD1 */
	gpio_disable(EGPIO(2));  /* TXEN */
	gpio_disable(EGPIO(3));  /* RXD0 */
	gpio_disable(EGPIO(4));  /* RXD1 */
	gpio_disable(EGPIO(5));  /* CRS_DV */
	gpio_disable(EGPIO(7));  /* REFCK */
	gpio_disable(EGPIO(8));  /* MDIO */
	gpio_disable(EGPIO(9));  /* MDC */

	/* MII/RGMII (not RMII/SLVRMII) */
	if (mode != ETH_RMII && mode != ETH_SLVRMII) {
		gpio_disable(EGPIO(11)); /* TXD2 */
		gpio_disable(EGPIO(12)); /* TXD3 */
		gpio_disable(EGPIO(13)); /* RXD2 */
		gpio_disable(EGPIO(14)); /* RXD3 */
		gpio_disable(EGPIO(15)); /* RXCLK */
	}

	/* only MII/SLVMII */
	if (mode == ETH_MII || mode == ETH_SLVRMII) {
		gpio_disable(EGPIO(6));  /* RXER */
		gpio_disable(EGPIO(10)); /* TXER */
		gpio_disable(EGPIO(16)); /* RXCRS */
		gpio_disable(EGPIO(17)); /* COL */
	}

	if (mode == ETH_RMII) {
		/* RMII restrictions */
		gpio_enable(EGPIO(6));
		gpio_direction_input(EGPIO(6)); /* RXER */

		gpio_enable(EGPIO(11));
		gpio_enable(EGPIO(12));
		gpio_direction_output(EGPIO(11), 0); /* TXD2 */
		gpio_direction_output(EGPIO(12), 0); /* TXD3 */
	}

	pads_enable("emaca");
}

#ifdef CONFIG_PHY8710
static void reset_eth_phy_8710(void)
{
	/* Reset the PHY */
	gpio_enable(CONFIG_ETH_RESET_GPIO);
	gpio_direction_output(CONFIG_ETH_RESET_GPIO, 0);

	/* Enable the strap pins as GPIOs */
	gpio_enable(EGPIO(3));
	gpio_enable(EGPIO(4));
	gpio_enable(EGPIO(6));
	gpio_enable(EGPIO(13));
	gpio_enable(EGPIO(14));
	gpio_enable(EGPIO(15));
	gpio_enable(EGPIO(5));

	/* Configure strap pins for SMSC 8710A on EVB */
#ifdef CONFIG_RMII
	gpio_direction_output(EGPIO(13), 1); /* MIISEL = 1 (RMII) */
#else
	gpio_direction_output(EGPIO(13), 0); /* MIISEL = 0 (MII) */
#endif

	gpio_direction_output(EGPIO(3),  1); /* MODE0  = 1 (Enable autoneg) */
	gpio_direction_output(EGPIO(4),  1); /* MODE1  = 1 (Enable autoneg) */
	gpio_direction_output(EGPIO(5),  1); /* MODE2  = 1 (Enable autoneg) */

	gpio_direction_output(EGPIO(6),  1); /* PHYAD0 = 1 (PHY address 3) */
	gpio_direction_output(EGPIO(15), 1); /* PHYAD1 = 1 (PHY address 3) */
	gpio_direction_output(EGPIO(14), 0); /* PHYAD2 = 0 (PHY address 3) */

	udelay(10000);

	/* Take the PHY out of reset */
	gpio_set_value(CONFIG_ETH_RESET_GPIO, 1);

	udelay(1000);

	/* Re-configure strap pins as MII signals (and not GPIOs) */
	gpio_disable(EGPIO(3));
	gpio_disable(EGPIO(4));
	gpio_disable(EGPIO(6));
	gpio_disable(EGPIO(13));
	gpio_disable(EGPIO(14));
	gpio_disable(EGPIO(15));
	gpio_disable(EGPIO(5));
}
#endif

int board_eth_init(bd_t *bis)
{
#ifdef CONFIG_RTL83XX_SWITCH
	rtk_api_ret_t rtk_ret;
	rtk_port_mac_ability_t rtk_macability;
	rtk_uint32 data;
#endif
	int ignore_phy = 0;
	int rc = 0;
	uint32_t val;
	const u32 eth_speed[] = {10, 100, 1000};

	/* select mode: MII/RMII/RGMII - must be done before reset_release */
	val = readl(DVF99_SYSCFG_BASE + DVF99_SYSCFG_GCR1);
	val &= ~(0x7 << 5);
	val |= (ETH_SYSCFG_MODE) << 5;
	writel(val, DVF99_SYSCFG_BASE + DVF99_SYSCFG_GCR1);

	/* configure clocks for requested mode/speed */
	rc = clk_eth_config(ETH_CLK_MODE, ETH_CLK_SPEED);
	if (rc) {
		printf("error: can't configure ethernet clocks\n");
		return -1;
	}

#ifdef CONFIG_RMII
	reset_release("rmii_if");
	clk_enable("rmii_if");
#endif

	reset_release("ethmac");
	clk_enable("ethmac");

	init_eth_gpios(ETH_CLK_MODE);

#ifdef CONFIG_PHY8710
	reset_eth_phy_8710();
#endif

#ifdef CONFIG_RTL83XX_SWITCH
	ignore_phy = 1;
#endif

	/* TODO: configure GMAC to use selected interface (MII/RMII/RGMII..) */
	/* third parameter tells the mac to ignore PHY, since PHYs are handled
	 * by the external switch */
	rc = gmac_eth_register(0, (void *)DVF99_ETHERMAC_BASE, ignore_phy,
			       eth_speed[ETH_CLK_SPEED], dvf99_get_hclk());
	if (rc < 0) {
		printf("error: can't register MAC\n");
		return -1;
	}

#ifdef CONFIG_RTL83XX_SWITCH
	/* Reset the switch (nRESET) */
	/* According to the ProgrammingGiude - we must wait one second after reset */
	gpio_direction_output(CONFIG_ETH_RESET_GPIO, 0);
	gpio_enable(CONFIG_ETH_RESET_GPIO);
	mdelay(10);
	gpio_set_value(CONFIG_ETH_RESET_GPIO, 1);
	mdelay(100);

	rtk_ret = rtl8367b_setAsicReg(RTL8367B_REG_CHIP_RESET, 1);
	if (rtk_ret != RT_ERR_OK)
		printf("error: chip reset failed\n");
	mdelay(1000);
	rc = 0;
	do {
		rc++;
		rtk_ret = rtl8367b_getAsicReg(RTL8367B_REG_CHIP_RESET, &data);
		if (rtk_ret != RT_ERR_OK)
			break;
		if (rc > 3000) {
			printf("error: timeout during initialization of RTL switch\n");
			return -1;
		}
	} while (data & 1);

	mdelay(10);

	/* TODO: initialize realtek switch, and configure it to selected mode  */
	rtk_ret = rtk_switch_init();
	if (rtk_ret != RT_ERR_OK) {
		printf("error: rtk_switch_init() returned %d\n", (int)rtk_ret);
		return -1;
	}

	/* On RTL8365MB there is only one external link, and it is Ext1 */
	rtk_macability.forcemode	= MAC_FORCE;
	rtk_macability.speed		= RTK_SPEED;
	rtk_macability.duplex		= FULL_DUPLEX;
	rtk_macability.link		= PORT_LINKUP;
	rtk_macability.nway		= DISABLED;
	rtk_macability.txpause		= ENABLED;
	rtk_macability.rxpause		= ENABLED;

	rtk_ret = rtk_port_macForceLinkExt_set(CONFIG_RTK_EXT_PORT, RTK_MODE, &rtk_macability);
	if (rtk_ret != RT_ERR_OK) {
		printf("error: rtk_port_macForceLinkExt1_set() returned %d\n", (int)rtk_ret);
		return -1;
	}

#if defined(CONFIG_RGMII)
	/* Set RGMII Interface 0 TX delay to 2ns and RX to step 0 */
	rtk_port_rgmiiDelayExt0_set(1, 0);
#endif
#endif

	return 0;
}

#ifdef CONFIG_RTL83XX_SWITCH
int do_print_phy_status(void)
{
	rtk_api_ret_t ret;
	int i;
	rtk_port_linkStatus_t stat;
	rtk_port_speed_t speed;
	rtk_port_duplex_t dup;

	for (i = 0; i < 5; i++) {
		ret = rtk_port_phyStatus_get(i, &stat, &speed, &dup);
		if (ret != RT_ERR_OK) {
			printf("error: rtk_port_phyStatus_get() for port %d returned %d\n", i, (int)ret);
		}

		printf("port %d:\n	Link is %s\n	Speed = %d\n	%s duplex\n\n", i,
			stat == PORT_LINKUP ? "UP" : "DOWN",
			speed == PORT_SPEED_10M ? 10 : (speed == PORT_SPEED_100M ? 100 : 1000),
			dup == PORT_HALF_DUPLEX ? "HALF" : "FULL");
	}

	return 0;
}

int do_dump_phy_regs(void)
{
	rtk_api_ret_t rtk_ret;
	rtk_port_phy_data_t d;
	int i, j;

	for (i = 0; i < 5; i++) {
		for (j=0 ; j<16 ; j++) {
			rtk_ret = rtk_port_phyReg_get(i, j, &d);
			if (rtk_ret != RT_ERR_OK) {
				printf("error: rtk_port_phyReg_get() for port,reg %d,%d returned %d\n", i, j, (int)rtk_ret);
			}
			printf("phyReg_get: port=%d reg=%d  ==> 0x%x\n", i, j, (int)d);
		}
		printf("\n");
	}
	return 0;
}

int do_read_phy(int argc, char * const argv[])
{
	rtk_api_ret_t rtk_ret;
	rtk_port_phy_data_t d;
	int i, j;

	if (argc != 4)
		return CMD_RET_USAGE;

	i = simple_strtoul(argv[2], NULL, 10);
	j = simple_strtoul(argv[3], NULL, 10);

	rtk_ret = rtk_port_phyReg_get(i, j, &d);

	printf("phyReg_get: port=%d reg=%d ==> 0x%x\n", i, j, (unsigned int)&d);

	return rtk_ret;
}

int do_write_phy(int argc, char * const argv[])
{
	rtk_api_ret_t rtk_ret;
	rtk_port_phy_data_t d;
	int i, j;

	if (argc != 5)
		return CMD_RET_USAGE;

	i = simple_strtoul(argv[2], NULL, 10);
	j = simple_strtoul(argv[3], NULL, 10);
	d = simple_strtoul(argv[4], NULL, 10);

	rtk_ret = rtk_port_phyReg_set(i, j, d);

	return rtk_ret;
}

int do_phy(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char *cmd = argv[1];

	if (strcmp(cmd, "regs") == 0)
		return do_dump_phy_regs();

	if (strcmp(cmd, "status") == 0)
		return do_print_phy_status();

	if (strcmp(cmd, "read") == 0)
		return do_read_phy(argc, argv);

	if (strcmp(cmd, "write") == 0)
		return do_write_phy(argc, argv);

	return CMD_RET_USAGE;
}

U_BOOT_CMD(
	phy, 5, 1, do_phy,
	"PHY sub-system",
	"regs   - dump registers\n"
	"phy status - print status\n"
	"phy read <port> <reg> - read register\n"
	"phy write <port> <reg> <value> - write register\n"
);
#endif

