/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <i2c.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>
#include <asm/arch/i2c.h>

unsigned long board_i2c_pnx_get_pclk(void)
{
	return dvf99_get_hclk();
}

static void
board_check_cpld(void)
{
#if defined(CONFIG_HARD_I2C) || defined(CONFIG_SOFT_I2C)
	unsigned char version;

	i2c_set_bus_num(0);

	if (!i2c_read(0x37, 0, 7, &version, 1))
		printf("CPLD:  version %d\n", version);
#endif
}

void board_i2c_init(const void *blob)
{
	gpio_disable(AGPIO(16)); /* IIC1SDA */
	gpio_disable(AGPIO(17)); /* IIC1SCL */

	gpio_pull_ctrl(AGPIO(16), 0, 1);
	gpio_pull_ctrl(AGPIO(17), 0, 1);

	reset_release("i2c1");
	clk_enable("i2c1");
	pads_enable("i2c1");

	gpio_disable(EGPIO(30)); /* IIC2SCL */
	gpio_disable(EGPIO(31)); /* IIC2SDA */

	gpio_pull_ctrl(EGPIO(30), 0, 1);
	gpio_pull_ctrl(EGPIO(31), 0, 1);

	reset_release("i2c2");
	clk_enable("i2c2");
	pads_enable("i2c2");

	board_check_cpld();

#if defined(CONFIG_DBMD2)
    gpio_enable(CONFIG_DBMD2_RSTN);
#endif
}

