/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

#include <syndwc_mmc.h>
#include <mmc.h>
#include <part.h>

#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>

#ifdef CONFIG_SDMMC_DETECT_GPIO
static int
mmc_get_card_detect(void)
{
	return !gpio_get_value(CONFIG_SDMMC_DETECT_GPIO);
}
#endif

static void
mmc_init_gpio(int gpio)
{
	gpio_disable(gpio);
	gpio_pull_ctrl(gpio, 1, 1);
}

static struct syndwc_pdata syndwc_pdata = {
	.card_num    = 0,
	.mclk        = 25000000,
	.f_max       = 25000000,
	.host_caps   = MMC_MODE_HS,
	.voltages    = MMC_VDD_32_33 | MMC_VDD_33_34,
#ifdef CONFIG_SDMMC_DETECT_GPIO
	.card_detect = &mmc_get_card_detect,
#endif
};

#ifdef CONFIG_EMMC
int
mmc_get_env_addr(struct mmc *mmc, int copy, u32 *env_addr)
{
	int ret, i = 1;
	unsigned boot_part;
	disk_partition_t info;

	/* search bootable partition first */
	boot_part = (mmc->part_config >> 3) & 0x7;
	if (boot_part == 1 || boot_part == 2) {
		printf("switching to boot partition %u\n", boot_part);
		mmc_select_hwpart(mmc->block_dev.dev, boot_part);
	}

	/* try to find the u-boot environment partition */
	do {
		ret = get_partition_info(&mmc->block_dev, i++, &info);
		if (!ret && !strcmp((const char *)info.type, BOOT_PART_ENV)) {
			if (info.size * 512 < CONFIG_ENV_SIZE) {
				printf("ERROR: Environment partition too small!\n");
				return 1;
			}
			*env_addr = info.start * 512;
			printf("found evironment partition at %8.8x\n", *env_addr);
			return 0;
		}
	} while (ret == 0);

	printf("ERROR: No environment partition found! "
	       "Use gptpart to create one...\n");

	return 1;
}
#endif /* CONFIG_EMMC */

void board_mmc_set_width(int width)
{
#ifdef CONFIG_SDMMC_4BIT
	if (width > 1)
#if CONFIG_SDMMC_LOC_VAL == 0
		mmc_init_gpio(BGPIO(23));
#else
		mmc_init_gpio(DGPIO(15));
#endif
#endif
}

int board_mmc_init(bd_t *bis)
{
	clk_set_rate("sdmmc", syndwc_pdata.mclk);
	reset_release("sdmmc");
	clk_enable("sdmmc");

	/* configure GPIOs */
	pads_location("sdmmc", CONFIG_SDMMC_LOC_VAL);

#ifdef CONFIG_SDMMC_4BIT
	syndwc_pdata.host_caps |= MMC_MODE_4BIT;
#if CONFIG_SDMMC_LOC_VAL == 0
	mmc_init_gpio(BGPIO(21)); /* D1 */
	mmc_init_gpio(BGPIO(22)); /* D2 */
	/* set CS high to avoid switching to SPI mode */
	gpio_direction_output(BGPIO(23), 1);
	gpio_enable(BGPIO(23));   /* D3 */
#else
	mmc_init_gpio(DGPIO(12)); /* D1 */
	mmc_init_gpio(DGPIO(13)); /* D2 */
	/* set CS high to avoid switching to SPI mode */
	gpio_direction_output(DGPIO(15), 1);
	gpio_enable(DGPIO(15));   /* D3 */
#endif
#endif

#ifdef CONFIG_SDMMC_WP
#if CONFIG_SDMMC_LOC_VAL == 0
	mmc_init_gpio(BGPIO(24)); /* WP */
#else
	mmc_init_gpio(DGPIO(16)); /* WP */
#endif
#endif

	mmc_init_gpio(DGPIO(14)); /* D0 */
	mmc_init_gpio(DGPIO(18)); /* CKL */
	mmc_init_gpio(DGPIO(19)); /* CMD */

#ifdef CONFIG_SDMMC_DETECT_GPIO
	gpio_direction_input(CONFIG_SDMMC_DETECT_GPIO);
	gpio_pull_ctrl(CONFIG_SDMMC_DETECT_GPIO, 1, 1);
	gpio_enable(CONFIG_SDMMC_DETECT_GPIO);
#else
#if CONFIG_SDMMC_LOC_VAL == 0
	gpio_direction_input(BGPIO(25));
	gpio_pull_ctrl(BGPIO(25), 1, 1);
	gpio_enable(BGPIO(25));
#else
	gpio_direction_input(DGPIO(17));
	gpio_pull_ctrl(DGPIO(17), 1, 1);
	gpio_enable(DGPIO(17));
#endif
#endif

	pads_enable("sdmmc");

	return syndwc_initialize(&syndwc_pdata);
}
