/*
 * (C) Copyright 2012 - 2014 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>
#ifdef CONFIG_QCA833X_SWITCH
#include <qca/athrs17_phy.h>
#endif

#if defined(CONFIG_PHY8710) && defined (CONFIG_QCA833X_SWITCH)
	#error Both switch and PHY are enabled!
#endif

#if defined(CONFIG_SLVMII)
	#define ETH_SYSCFG_MODE		0
	#define ETH_CLK_MODE		ETH_SLVMII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define ETH_MODE		"SLVMII"
#elif defined(CONFIG_RMII)
	#define ETH_SYSCFG_MODE		4
	#define ETH_CLK_MODE		ETH_RMII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define ETH_MODE		"RMII"
#elif defined(CONFIG_SLVRMII)
	#define ETH_SYSCFG_MODE		4
	#define ETH_CLK_MODE		ETH_SLVRMII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define ETH_MODE		"SLVRMII"
#elif defined(CONFIG_RGMII)
	#define ETH_SYSCFG_MODE		1
	#define ETH_CLK_MODE		ETH_RGMII
	#define ETH_CLK_SPEED		ETH_SPEED_1000
	#define ETH_MODE		"RGMII"
#else
	/* Use MII by default (CONFIG_MII is used for other purposes so we don't check it) */
	#define ETH_SYSCFG_MODE		0
	#define ETH_CLK_MODE		ETH_MII
	#define ETH_CLK_SPEED		ETH_SPEED_100
	#define ETH_MODE		"MII"
#endif

/* optional: override default ETH speed */
#ifdef CONFIG_ETH_SPEED
	#undef ETH_CLK_SPEED
	#if (CONFIG_ETH_SPEED == 10)
		#define ETH_CLK_SPEED	ETH_SPEED_10
	#elif (CONFIG_ETH_SPEED == 100)
		#define ETH_CLK_SPEED	ETH_SPEED_100
	#elif (CONFIG_ETH_SPEED == 1000)
		#define ETH_CLK_SPEED	ETH_SPEED_1000
	#else
		#error Wrong CONFIG_ETH_SPEED value
	#endif
#endif

int gmac_eth_register(int id, void *regs, int ignore_phy, u32 speed, u32 clk_csr_i);

static void init_eth_gpios(enum eth_mode mode)
{
	/* all modes */
	gpio_disable(EGPIO(0));  /* TXD0 */
	gpio_disable(EGPIO(1));  /* TXD1 */
	gpio_disable(EGPIO(2));  /* TXEN */
	gpio_disable(EGPIO(3));  /* RXD0 */
	gpio_disable(EGPIO(4));  /* RXD1 */
	gpio_disable(EGPIO(5));  /* CRS_DV */
	gpio_disable(EGPIO(7));  /* REFCK */
	gpio_disable(EGPIO(8));  /* MDIO */
	gpio_disable(EGPIO(9));  /* MDC */

	/* MII/RGMII (not RMII/SLVRMII) */
	if (mode != ETH_RMII && mode != ETH_SLVRMII) {
		gpio_disable(EGPIO(11)); /* TXD2 */
		gpio_disable(EGPIO(12)); /* TXD3 */
		gpio_disable(EGPIO(13)); /* RXD2 */
		gpio_disable(EGPIO(14)); /* RXD3 */
		gpio_disable(EGPIO(15)); /* RXCLK */
	}

	/* only MII/SLVMII */
	if (mode == ETH_MII || mode == ETH_SLVRMII) {
		gpio_disable(EGPIO(6));  /* RXER */
		gpio_disable(EGPIO(10)); /* TXER */
		gpio_disable(EGPIO(16)); /* RXCRS */
		gpio_disable(EGPIO(17)); /* COL */
	}

	if (mode == ETH_RMII) {
		/* RMII restrictions */
		gpio_enable(EGPIO(6));
		gpio_direction_input(EGPIO(6)); /* RXER */

		gpio_enable(EGPIO(11));
		gpio_enable(EGPIO(12));
		gpio_direction_output(EGPIO(11), 0); /* TXD2 */
		gpio_direction_output(EGPIO(12), 0); /* TXD3 */
	}

	pads_enable("emaca");
}

#ifdef CONFIG_PHY8710
static void reset_eth_phy_8710(void)
{
	/* Reset the PHY */
	gpio_enable(CONFIG_ETH_RESET_GPIO);
	gpio_direction_output(CONFIG_ETH_RESET_GPIO, 0);

	/* Enable the strap pins as GPIOs */
	gpio_enable(EGPIO(3));
	gpio_enable(EGPIO(4));
	gpio_enable(EGPIO(6));
	gpio_enable(EGPIO(13));
	gpio_enable(EGPIO(14));
	gpio_enable(EGPIO(15));
	gpio_enable(EGPIO(5));

	/* Configure strap pins for SMSC 8710A on EVB */
#ifdef CONFIG_RMII
	gpio_direction_output(EGPIO(13), 1); /* MIISEL = 1 (RMII) */
#else
	gpio_direction_output(EGPIO(13), 0); /* MIISEL = 0 (MII) */
#endif

	gpio_direction_output(EGPIO(3),  1); /* MODE0  = 1 (Enable autoneg) */
	gpio_direction_output(EGPIO(4),  1); /* MODE1  = 1 (Enable autoneg) */
	gpio_direction_output(EGPIO(5),  1); /* MODE2  = 1 (Enable autoneg) */

	gpio_direction_output(EGPIO(6),  1); /* PHYAD0 = 1 (PHY address 3) */
	gpio_direction_output(EGPIO(15), 1); /* PHYAD1 = 1 (PHY address 3) */
	gpio_direction_output(EGPIO(14), 0); /* PHYAD2 = 0 (PHY address 3) */

	udelay(10000);

	/* Take the PHY out of reset */
	gpio_set_value(CONFIG_ETH_RESET_GPIO, 1);

	udelay(1000);

	/* Re-configure strap pins as MII signals (and not GPIOs) */
	gpio_disable(EGPIO(3));
	gpio_disable(EGPIO(4));
	gpio_disable(EGPIO(6));
	gpio_disable(EGPIO(13));
	gpio_disable(EGPIO(14));
	gpio_disable(EGPIO(15));
	gpio_disable(EGPIO(5));
}
#endif

int board_eth_init(bd_t *bis)
{
	int ignore_phy = 0;
	int rc = 0;
	uint32_t val;
	const u32 eth_speed[] = {10, 100, 1000};

	/* select mode: MII/RMII/RGMII - must be done before reset_release */
	val = readl(DVF99_SYSCFG_BASE + DVF99_SYSCFG_GCR1);
	val &= ~(0x7 << 5);
	val |= (ETH_SYSCFG_MODE) << 5;
#ifdef CONFIG_MAIN_CLK_OUT
	gpio_disable(BGPIO(31));
	val |= (1 << 1);
#endif
	writel(val, DVF99_SYSCFG_BASE + DVF99_SYSCFG_GCR1);

	/* configure clocks for requested mode/speed */
	rc = clk_eth_config(ETH_CLK_MODE, ETH_CLK_SPEED);
	if (rc) {
		printf("error: can't configure ethernet clocks\n");
		return -1;
	}

	puts(ETH_MODE);
	puts(", ");

#ifdef CONFIG_RMII
	reset_release("rmii_if");
	clk_enable("rmii_if");
#endif

	reset_release("ethmac");
	clk_enable("ethmac");

	init_eth_gpios(ETH_CLK_MODE);

#ifdef CONFIG_PHY8710
	reset_eth_phy_8710();
#endif

#ifdef CONFIG_QCA833X_SWITCH
	ignore_phy = 1;
#endif

	/* TODO: configure GMAC to use selected interface (MII/RMII/RGMII..) */
	/* third parameter tells the mac to ignore PHY, since PHYs are handled
	 * by the external switch */
	rc = gmac_eth_register(0, (void *)DVF99_ETHERMAC_BASE, ignore_phy,
			       eth_speed[ETH_CLK_SPEED], dvf99_get_hclk());

#ifdef CONFIG_QCA833X_SWITCH
	/* Reset the switch (nRESET) */
	/* According to the ProgrammingGiude - we must wait one second after reset */
	gpio_enable(CONFIG_ETH_RESET_GPIO);
	gpio_direction_output(CONFIG_ETH_RESET_GPIO, 0);
	mdelay(15);
	gpio_set_value(CONFIG_ETH_RESET_GPIO, 1);
	mdelay(150);

	/* Initialize ethernet switch */
	athrs17_reg_init();
#endif

	return rc;
}

#ifdef CONFIG_QCA833X_SWITCH
int do_read_ath_reg(int argc, char * const argv[])
{
	int reg;

	if (argc != 3)
		return CMD_RET_USAGE;

	reg = simple_strtoul(argv[2], NULL, 16);
	printf("0x%x\n", athrs17_reg_read(reg));

	return 0;
}

int do_write_ath_reg(int argc, char * const argv[])
{
	int reg, val;

	if (argc != 4)
		return CMD_RET_USAGE;

	reg = simple_strtoul(argv[2], NULL, 16);
	val = simple_strtoul(argv[3], NULL, 16);
	athrs17_reg_write(reg, val);

	return 0;
}

int do_read_phy_reg(int argc, char * const argv[])
{
	int phy, reg;

	if (argc != 4)
		return CMD_RET_USAGE;

	phy = simple_strtoul(argv[2], NULL, 16);
	reg = simple_strtoul(argv[3], NULL, 16);
	printf("0x%x\n", phy_reg_read(0, phy, reg));

	return 0;
}

int do_write_phy_reg(int argc, char * const argv[])
{
	int phy, reg, val;

	if (argc != 5)
		return CMD_RET_USAGE;

	phy = simple_strtoul(argv[2], NULL, 16);
	reg = simple_strtoul(argv[3], NULL, 16);
	val = simple_strtoul(argv[4], NULL, 16);
	phy_reg_write(0, phy, reg, val);

	return 0;
}

int do_atheros(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char *cmd = argv[1];

	if (strcmp(cmd, "phy_read") == 0)
		return do_read_phy_reg(argc, argv);

	if (strcmp(cmd, "phy_write") == 0)
		return do_write_phy_reg(argc, argv);

	if (strcmp(cmd, "read") == 0)
		return do_read_ath_reg(argc, argv);

	if (strcmp(cmd, "write") == 0)
		return do_write_ath_reg(argc, argv);

	return CMD_RET_USAGE;
}

U_BOOT_CMD(
	atheros, 5, 1, do_atheros,
	"Atheros switch sub-system",
	"read      - read register\n"
	"atheros write     - write register\n"
	"atheros phy_read  - read PHY register\n"
	"atheros phy_write - write PHY register\n"
);
#endif
