/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <spi.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>
#include <asm/arch/spi.h>

void
board_spi_init(void)
{
	reset_release("spi2");
	clk_enable("spi2");

	gpio_disable(BGPIO(13)); /* SPI2DO */
	gpio_pull_ctrl(BGPIO(13), 1, 1);
	gpio_disable(BGPIO(14)); /* SPI2DI */
	gpio_pull_ctrl(BGPIO(14), 1, 0);
	gpio_disable(BGPIO(16)); /* SPI2CK */
	gpio_pull_ctrl(BGPIO(16), 1, 1);

	gpio_direction_output(AGPIO(27), 1);
	gpio_enable(AGPIO(27)); /* SPI2CS */

	pads_enable("spi2");

	clk_set_rate("spi2", 2000000);
}

void spi_3wire(unsigned int bus, int read) { }

void spi_cs_activate(struct spi_slave *slave)
{
	if (slave->bus == 1)
		gpio_set_value(AGPIO(27), 0);
}

void spi_cs_deactivate(struct spi_slave *slave)
{
	if (slave->bus == 1)
		gpio_set_value(AGPIO(27), 1);
}

int do_spi_test(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	int i, ret, bus, count, tmp;
	struct spi_slave *spi;

	if (argc < 3)
		return CMD_RET_USAGE;

	bus = (int)simple_strtoul(argv[1], NULL, 16);
	if ((bus < 0) || (bus > 1))
		return CMD_RET_USAGE;

	count = (int)simple_strtoul(argv[2], NULL, 16);

	spi = spi_setup_slave(bus, 0, 2000000, SPI_MODE_3);
	if (!spi)
		return -1;

	ret = spi_claim_bus(spi);
	if (ret) {
		debug("unable to claim SPI bus\n");
		return ret;
	}

	ret = spi_xfer(spi, 8, &tmp, NULL, SPI_XFER_BEGIN);
	if (ret)
		return -1;

	for (i = 0; i < count; i++) {
		ret = spi_xfer(spi, 8, NULL, &tmp, 0);
		if (ret)
			return -1;
	}

	/* deactivate CS */
	spi_xfer(spi, 0, NULL, NULL, SPI_XFER_END);

	spi_release_bus(spi);

	return 0;
}

U_BOOT_CMD(spi_test, 3, 0, do_spi_test,
		"test SPI driver",
		"<bus> <number of transactions>"
	  );

