/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <lcd.h>
#include <spi.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/lcd.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>

static void
lcd_init_gpio(int gpio)
{
	gpio_disable(gpio);
	gpio_pull_ctrl(gpio, 1, 1);
}

void
board_lcd_init(void)
{
	clk_set_rate("lcdc", 10000000);
	reset_release("lcdc");
	clk_enable("lcdc");

	lcd_init_gpio(DGPIO(6));  /* LCDD0 */
	lcd_init_gpio(DGPIO(1));  /* LCDD1 */
	lcd_init_gpio(CGPIO(0));  /* LCDD2 */
	lcd_init_gpio(CGPIO(1));  /* LCDD3 */
	lcd_init_gpio(CGPIO(2));  /* LCDD4 */
	lcd_init_gpio(CGPIO(3));  /* LCDD5 */
	lcd_init_gpio(CGPIO(4));  /* LCDD6 */
	lcd_init_gpio(CGPIO(5));  /* LCDD7 */
	lcd_init_gpio(DGPIO(2));  /* LCDD8 */
	lcd_init_gpio(DGPIO(3));  /* LCDD9 */
	lcd_init_gpio(CGPIO(6));  /* LCDD10 */
	lcd_init_gpio(CGPIO(7));  /* LCDD11 */
	lcd_init_gpio(CGPIO(8));  /* LCDD12 */
	lcd_init_gpio(CGPIO(9));  /* LCDD13 */
	lcd_init_gpio(CGPIO(10)); /* LCDD14 */
	lcd_init_gpio(CGPIO(11)); /* LCDD15 */

	lcd_init_gpio(CGPIO(19)); /* LCDGP0 */
	lcd_init_gpio(CGPIO(20)); /* LCDGP1 */
	lcd_init_gpio(CGPIO(21)); /* LCDGP2 */
	lcd_init_gpio(CGPIO(22)); /* LCDGP3 */

	pads_enable("lcdc");
	pads_enable("lcdgp0");
	pads_enable("lcdgp1");
	pads_enable("lcdgp2");
	pads_enable("lcdgp3");
}

static void
dvf99_rocktech_init(void)
{
}

static const struct dvf99_lcdc_panel dvf99_rocktech =
{
	.name = "rocktech",
	.xres = 800,
	.yres = 480,
	.bits_per_pixel = LCD_COLOR16,
	.clock_rate = 30000000,
	.init_fn = dvf99_rocktech_init,
	.init_seq = {
		/* Display panel */
		{ LCDC_REG_DISPIR,   0x0001 },	/* 24bits TFT-LCD */
		{ LCDC_REG_PANCSR,   0x0006 },	/* RGB order. Data is sampled on falling-edge of clk */

		/* Vertical Timing register */
		{ LCDC_REG_VSTR,          0 },
		{ LCDC_REG_VFTR,         23 },
		{ LCDC_REG_VETR,         22 },

		/* Horizontal Timing register*/
		{ LCDC_REG_HSTR,          1 },
		{ LCDC_REG_HFTR,         45 },
		{ LCDC_REG_HETR,        209 },

		{ LCDC_REG_YCLPCR,   0xff00 },
		{ LCDC_REG_CCLPCR,   0xff00 },

		{ LCDC_REG_ISTAR4,   0x4009 },

		{ LCDC_REG_GP3CNTR,  0x0001 }, /* GP3 set high (RESET) */

		{ LCDC_SEQ_END,      0x0000 }
	},
};

const struct dvf99_lcdc_panel *dvf99_panels[] = {
	&dvf99_rocktech,
	NULL,
};
