/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

#include <syndwc_mmc.h>
#include <mmc.h>
#include <part.h>

#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>

static void
mmc_init_gpio(int gpio)
{
	gpio_disable(gpio);
	gpio_pull_ctrl(gpio, 1, 1);
}

static struct syndwc_pdata syndwc_pdata = {
	.card_num    = 0,
	.mclk        = 25000000,
	.f_max       = 25000000,
	.host_caps   = MMC_MODE_HS,
	.voltages    = MMC_VDD_32_33 | MMC_VDD_33_34,
};

void board_mmc_set_width(int width)
{
	if (width > 1)
#if CONFIG_SDMMC_LOC_VAL == 0
		mmc_init_gpio(BGPIO(23));
#else
		mmc_init_gpio(DGPIO(15));
#endif
}

int board_mmc_init(bd_t *bis)
{
	clk_set_rate("sdmmc", syndwc_pdata.mclk);
	reset_release("sdmmc");
	clk_enable("sdmmc");

	/* configure GPIOs */
	pads_location("sdmmc", CONFIG_SDMMC_LOC_VAL);

	syndwc_pdata.host_caps |= MMC_MODE_4BIT;

	mmc_init_gpio(DGPIO(14)); /* D0 */
	mmc_init_gpio(BGPIO(21)); /* D1 */
	mmc_init_gpio(BGPIO(22)); /* D2 */
	/* set CS high to avoid switching to SPI mode */
	gpio_direction_output(BGPIO(23), 1);
	gpio_enable(BGPIO(23));   /* D3 */
	mmc_init_gpio(DGPIO(18)); /* CKL */
	mmc_init_gpio(DGPIO(19)); /* CMD */

	gpio_direction_input(BGPIO(25));
	gpio_pull_ctrl(BGPIO(25), 1, 1);
	gpio_enable(BGPIO(25));

	pads_enable("sdmmc");

	return syndwc_initialize(&syndwc_pdata);
}
