/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <linux/mtd/nand.h>
#include <dmw96_nfc.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/pads.h>
#include <asm/arch/hardware.h>

static struct dmw96_nfc_dev nfc_dev = {
	.num = 0,
	.cs = 0,
	.readysel = 0,
};

static void
nand_init_gpio(int gpio)
{
	gpio_disable(gpio);
	gpio_pull_ctrl(gpio, 1, 1);
}

int board_nand_init(struct nand_chip *nand)
{
	reset_release("nfc");
	clk_enable("nfc");

	/* NFC data (0-7) */
	nand_init_gpio(AGPIO(7));  /* D0 */
	nand_init_gpio(AGPIO(8));  /* D1 */
	nand_init_gpio(AGPIO(9));  /* D2 */
	nand_init_gpio(AGPIO(10)); /* D3 */
	nand_init_gpio(AGPIO(11)); /* D4 */
	nand_init_gpio(AGPIO(12)); /* D5 */
	nand_init_gpio(AGPIO(13)); /* D6 */
	nand_init_gpio(AGPIO(0));  /* D7 */

	nand_init_gpio(AGPIO(2));  /* write-enable */
	nand_init_gpio(AGPIO(3));  /* read-enable */
	nand_init_gpio(AGPIO(1));  /* ready1 */
	nand_init_gpio(AGPIO(4));  /* cle */
	nand_init_gpio(AGPIO(5));  /* ale */
	nand_init_gpio(AGPIO(6));  /* cs0 */

	pads_disable("mema17");
	pads_disable("mema18");
	pads_disable("mema21");
	pads_disable("mema22");
	pads_disable("mema23");
	pads_disable("mema24");
	pads_disable("mema25");
	pads_disable("qspi");
	pads_enable("fc");

	dmw96_nfc_init(&nfc_dev);

	return 0;
}
