/*
 * (C) Copyright 2016 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/io.h>
#include <asm/arch/hardware.h>
#include <asm/arch/factory.h>
#include <netdev.h>
#include <phy.h>
#ifdef CONFIG_RTL83XX_SWITCH
#include <rtk_api_ext.h>
#include <rtl8367b_asicdrv_port.h>
#endif
#ifdef CONFIG_QCA833X_SWITCH
#include <qca/athrs17_phy.h>
#endif

#if defined(CONFIG_PHY8710) && defined (CONFIG_RTL83XX_SWITCH)
	#error Both switch and PHY are enabled!
#endif

#if defined(CONFIG_MII_PHY)
	#define ETH_SYSCFG_MODE		0
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_MII_MAC /* ? */
	#define ETH_MODE		"MII_PHY"
	#define CLK_MODE		ETH_MII_PHY
	#define CLK_SPEED		ETH_SPEED_100
	#define PHY_MODE		PHY_INTERFACE_MODE_MII
#elif defined(CONFIG_RMII)
	#define ETH_SYSCFG_MODE		4
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_RMII_MAC
	#define ETH_MODE		"RMII"
	#define CLK_MODE		ETH_RMII
	#define CLK_SPEED		ETH_SPEED_100
	#define PHY_MODE		PHY_INTERFACE_MODE_RMII
#elif defined(CONFIG_RMII_EXT)
	#define ETH_SYSCFG_MODE		4
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_RMII_MAC /* ? */
	#define ETH_MODE		"RMII_EXT"
	#define CLK_MODE		ETH_RMII_EXT
	#define CLK_SPEED		ETH_SPEED_100
	#define PHY_MODE		PHY_INTERFACE_MODE_RMII
#elif defined(CONFIG_RGMII)
	#define ETH_SYSCFG_MODE		1
	#define RTK_SPEED		SPD_1000M
	#define RTK_MODE		MODE_EXT_RGMII /* MODE_EXT_RGMII_33V */
	#define ETH_MODE		"RGMII"
	#define CLK_MODE		ETH_RGMII
	#define CLK_SPEED		ETH_SPEED_1000
	#define PHY_MODE		PHY_INTERFACE_MODE_RGMII
#else
	/* Use MII by default (CONFIG_MII is used for other purposes so we don't check it) */
	#define ETH_SYSCFG_MODE		0
	#define RTK_SPEED		SPD_100M
	#define RTK_MODE		MODE_EXT_MII_PHY
	#define ETH_MODE		"MII"
	#define CLK_MODE		ETH_MII
	#define CLK_SPEED		ETH_SPEED_100
	#define PHY_MODE		PHY_INTERFACE_MODE_MII
#endif

/* optional: override default ETH speed */
#ifdef CONFIG_ETH_SPEED
	#undef CLK_SPEED
	#undef RTK_SPEED
	#if (CONFIG_ETH_SPEED == 10)
		#define CLK_SPEED	ETH_SPEED_10
		#define RTK_SPEED	SPD_10M
	#elif (CONFIG_ETH_SPEED == 100)
		#define CLK_SPEED	ETH_SPEED_100
		#define RTK_SPEED	SPD_100M
	#elif (CONFIG_ETH_SPEED == 1000)
		#define CLK_SPEED	ETH_SPEED_1000
		#define RTK_SPEED	SPD_1000M
	#else
		#error Wrong CONFIG_ETH_SPEED value
	#endif
#endif

#ifdef CONFIG_PHY8710
static void reset_eth_phy_8710(void)
{
	/* Reset the PHY */
	io_mux(CONFIG_ETH_RESET_IO);
	gpio_direction_output(CONFIG_ETH_RESET_GPIO, 0);

	/* Enable the strap pins as GPIOs */
	io_mux("egpio3");
	io_mux("egpio4");
	io_mux("egpio6");
	io_mux("egpio13");
	io_mux("egpio14");
	io_mux("egpio15");
	io_mux("egpio17");

	/* Configure strap pins for SMSC 8710A on EVB */
#ifdef CONFIG_RMII
	gpio_direction_output(EGPIO(13), 1); /* MIISEL = 1 (RMII) */
#else
	gpio_direction_output(EGPIO(13), 0); /* MIISEL = 0 (MII) */
#endif

	gpio_direction_output(EGPIO(3),  1); /* MODE0  = 1 (Enable autoneg) */
	gpio_direction_output(EGPIO(4),  1); /* MODE1  = 1 (Enable autoneg) */
	gpio_direction_output(EGPIO(17), 1); /* MODE2  = 1 (Enable autoneg) */

	gpio_direction_output(EGPIO(6),  1); /* PHYAD0 = 1 (PHY address 3) */
	gpio_direction_output(EGPIO(15), 1); /* PHYAD1 = 1 (PHY address 3) */
	gpio_direction_output(EGPIO(14), 0); /* PHYAD2 = 0 (PHY address 3) */

	udelay(10000);

	/* Take the PHY out of reset */
	gpio_set_value(CONFIG_ETH_RESET_GPIO, 1);

	udelay(1000);

	/* Re-configure strap pins as MII signals (and not GPIOs) */
	io_mux("miirxd0"); /* egpio3 */
	io_mux("miirxd1"); /* egpio4 */
	io_mux("miirxd2"); /* egpio13 */
	io_mux("miirxd3"); /* egpio14 */
	io_mux("miirxer"); /* egpio6 */
	io_mux("miirxck"); /* egpio15 */
	io_mux("miicol");  /* egpio17 */
}
#endif

#ifdef CONFIG_QCA833X_SWITCH
static void reset_eth_qca833x(void)
{
   unsigned int reset_gpio = CONFIG_ETH_RESET_GPIO;

   /* Reset the switch (nRESET) */
   /* According to the ProgrammingGiude - we must wait one second after reset */
   io_mux(CONFIG_ETH_RESET_IO);
   gpio_enable(reset_gpio);
   gpio_direction_output(reset_gpio, 0);
   mdelay(15);
   gpio_set_value(reset_gpio, 1);
   mdelay(150);
}
#endif

#ifdef CONFIG_RTL836X_SWITCH
static void reset_eth_rtl836x(void)
{
   unsigned int reset_gpio = CONFIG_ETH_RESET_GPIO;

   io_mux(CONFIG_ETH_RESET_IO);
   gpio_enable(reset_gpio);
   gpio_direction_output(reset_gpio, 0);
   mdelay(100);
   gpio_set_value(reset_gpio, 1);
   mdelay(1200);
}
#endif

int board_eth_init(bd_t *bis)
{
#ifdef CONFIG_RTL83XX_SWITCH
	rtk_api_ret_t rtk_ret;
	rtk_port_mac_ability_t rtk_macability;
	rtk_uint32 data;
#endif
	int rc = 0;
	uint32_t val;
	int eth_chip;
	int fact_eth_clksrc;

	fact_eth_clksrc = hw_get_eth_clock_source();
	eth_chip = hw_get_eth();
	printf("clksrc: %d, eth: %d\n", fact_eth_clksrc, eth_chip);

	/* select mode: MII/RMII/RGMII - must be done before reset_release */
	val = readl(DVF101_SYSCFG_BASE + DVF101_SYSCFG_GCR1);
	val &= ~(0x7 << 5);
	val |= (ETH_SYSCFG_MODE) << 5;
	writel(val, DVF101_SYSCFG_BASE + DVF101_SYSCFG_GCR1);

	io_mux("main_clk_out");

	puts(ETH_MODE);
	puts(", ");

	rc = clk_eth_config(CLK_MODE, CLK_SPEED);

	io_mux("mdc");
	io_mux("mdio");

#if defined(CONFIG_MII) || defined(CONFIG_MII_PHY)
	io_mux("miicrs_dv");
	io_mux("miirefck");
	io_mux("miirxd0");
	io_mux("miirxd1");
	io_mux("miitxd0");
	io_mux("miitxd1");
	io_mux("miitxen");
	io_mux("miirxer"); /* optional */
	io_mux("miirxck");
	io_mux("miirxd2");
	io_mux("miirxd3");
	io_mux("miitxd2");
	io_mux("miitxd3");
	io_mux("miicol");
	io_mux("miirxcrs");
	io_mux("miirxer");
	io_mux("miitxer");
#elif defined(CONFIG_RMII) || defined(CONFIG_RMII_EXT)
	io_mux("rgmiird0");
	io_mux("rgmiird1");
	io_mux("rgmiirx_ctl"); /* crsdv */
	io_mux("rgmiitd0");
	io_mux("rgmiitd1");
	io_mux("rgmiitxc");    /* refclk */
	io_mux("rgmiitx_ctl"); /* txen */

	io_slew_rate("egpio7", 1); /* txc */
#elif defined(CONFIG_RGMII)
	io_mux("rgmiird0");
	io_mux("rgmiird1");
	io_mux("rgmiird2");
	io_mux("rgmiird3");
	io_mux("rgmiirxc");
	io_mux("rgmiirx_ctl");
	io_mux("rgmiitd0");
	io_mux("rgmiitd1");
	io_mux("rgmiitd2");
	io_mux("rgmiitd3");
	io_mux("rgmiitxc");
	io_mux("rgmiitx_ctl");

	io_slew_rate("egpio7", 1); /* txc */
#endif

#ifdef CONFIG_PHY8710
	reset_eth_phy_8710();
#endif

#ifdef CONFIG_RTL83XX_SWITCH
	/* Reset the switch (nRESET) */
	/* According to the ProgrammingGiude - we must wait one second after reset */
	/* Reset the switch */
	io_mux(CONFIG_ETH_RESET_IO);
	gpio_direction_output(CONFIG_ETH_RESET_GPIO, 0);
	mdelay(10);

	/* Take the PHY out of reset */
	gpio_direction_output(CONFIG_ETH_RESET_GPIO, 1);
	mdelay(100);
#endif

#ifdef CONFIG_RTL836X_SWITCH
	if ( ETH_TYPE_RTL8363NB == eth_chip )
	{
		reset_eth_rtl836x();
	}
#endif

#ifdef CONFIG_QCA833X_SWITCH
	if ( ETH_TYPE_QCA8334 == eth_chip )
	{
		reset_eth_qca833x();
	}
#endif

	rc = designware_initialize(DVF101_ETHERMAC_BASE, PHY_MODE);
	if (rc < 0) {
		printf("error: can't register MAC\n");
		return -1;
	}

#ifdef CONFIG_QCA833X_SWITCH
	if ( ETH_TYPE_QCA8334 == eth_chip )
	{
		/* Initialize ethernet switch */
		if (ETH_RGMII == CLK_MODE) {
			athrs17_reg_init(RGMII, 0);
		}
		else {
			athrs17_reg_init(RMII, 0);
		}
	}
#endif

#ifdef CONFIG_RTL83XX_SWITCH
	rtk_ret = rtl8367b_setAsicReg(RTL8367B_REG_CHIP_RESET, 1);
	if (rtk_ret != RT_ERR_OK)
		printf("error: chip reset failed\n");
	mdelay(1000);
	rc = 0;
	do {
		rc++;
		rtk_ret = rtl8367b_getAsicReg(RTL8367B_REG_CHIP_RESET, &data);
		if (rtk_ret != RT_ERR_OK)
			break;
		if (rc > 3000) {
			printf("error: timeout during initialization of RTL switch\n");
			return -1;
		}
	} while (data & 1);

	mdelay(10);

	/* TODO: initialize realtek switch, and configure it to selected mode  */
	rtk_ret = rtk_switch_init();
	if (rtk_ret != RT_ERR_OK) {
		printf("error: rtk_switch_init() returned %d\n", (int)rtk_ret);
		return -1;
	}

	/* On RTL8365MB there is only one external link, and it is Ext1 */
	rtk_macability.forcemode	= MAC_FORCE;
	rtk_macability.speed		= RTK_SPEED;
	rtk_macability.duplex		= FULL_DUPLEX;
	rtk_macability.link		= PORT_LINKUP;
	rtk_macability.nway		= DISABLED;
	rtk_macability.txpause		= ENABLED;
	rtk_macability.rxpause		= ENABLED;

	rtk_ret = rtk_port_macForceLinkExt_set(CONFIG_RTK_EXT_PORT, RTK_MODE, &rtk_macability);
	if (rtk_ret != RT_ERR_OK) {
		printf("error: rtk_port_macForceLinkExt1_set() returned %d\n", (int)rtk_ret);
		return -1;
	}

#if defined(CONFIG_RGMII)
	/* Set RGMII Interface 0 TX delay to 2ns and RX to step 0 */
	rtk_port_rgmiiDelayExt0_set(1, 0);
#endif
#endif

	return 0;
}

#ifdef CONFIG_QCA833X_SWITCH

extern uint16_t phy_reg_read(int dev_id, uint32_t phy_addr, uint8_t reg);
extern void phy_reg_write(int dev_id, uint32_t phy_addr, uint8_t reg, uint16_t data);

int do_read_ath_reg(int argc, char * const argv[])
{
    int reg;

    if (argc != 3)
        return CMD_RET_USAGE;

    reg = simple_strtoul(argv[2], NULL, 16);
    printf("0x%x\n", athrs17_reg_read(reg));

    return 0;
}

int do_write_ath_reg(int argc, char * const argv[])
{
    int reg, val;

    if (argc != 4)
        return CMD_RET_USAGE;

    reg = simple_strtoul(argv[2], NULL, 16);
    val = simple_strtoul(argv[3], NULL, 16);
    athrs17_reg_write(reg, val);

    return 0;
}

int do_read_phy_reg(int argc, char * const argv[])
{
    int phy, reg;

    if (argc != 4)
        return CMD_RET_USAGE;

    phy = simple_strtoul(argv[2], NULL, 16);
    reg = simple_strtoul(argv[3], NULL, 16);
    printf("0x%x\n", phy_reg_read(0, phy, reg));

    return 0;
}

int do_write_phy_reg(int argc, char * const argv[])
{
    int phy, reg, val;

    if (argc != 5)
        return CMD_RET_USAGE;

    phy = simple_strtoul(argv[2], NULL, 16);
    reg = simple_strtoul(argv[3], NULL, 16);
    val = simple_strtoul(argv[4], NULL, 16);
    phy_reg_write(0, phy, reg, val);

    return 0;
}

int do_show_ath_status(int argc, char * const argv[])
{
    if (argc != 2)
        return CMD_RET_USAGE;

    printf("LAN:%d, WAN:%d\n", athrs17_pc_status(),
            athrs17_lan_status());

    return 0;
}

int do_atheros(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
    char *cmd = argv[1];

    if (strcmp(cmd, "phy_read") == 0)
        return do_read_phy_reg(argc, argv);

    if (strcmp(cmd, "phy_write") == 0)
        return do_write_phy_reg(argc, argv);

    if (strcmp(cmd, "read") == 0)
        return do_read_ath_reg(argc, argv);

    if (strcmp(cmd, "write") == 0)
        return do_write_ath_reg(argc, argv);

    if (strcmp(cmd, "stat") == 0)
        return do_show_ath_status(argc, argv);

    return CMD_RET_USAGE;
}

U_BOOT_CMD(
        atheros, 5, 1, do_atheros,
        "Atheros switch sub-system",
        "read      - read register\n"
        "atheros write     - write register\n"
        "atheros phy_read  - read PHY register\n"
        "atheros phy_write - write PHY register\n"
        "atheros stat - show link status of LAN and PC ports \n"
        );
#endif

