/****************************************************************************
 *
 * Copyright (c) 2013 by Grandstream Networks, Inc.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 *
 ***************************************************************************/

#include <common.h>
#include <asm/io.h>
#include <malloc.h>
#include <nand.h>
#include <asm/arch/factory.h>
#include <jffs2/jffs2.h>
#if defined(CONFIG_EMBED_CFG)
#if defined(CONFIG_EMBED_CFG_HEADER)
#include "fact_embeded_cfg.h"
#endif
#endif

#if defined( CONFIG_QSPI )
int mtdparts_init(void);
int find_dev_and_part(const char *id, struct mtd_device **dev,
	u8 *part_num, struct part_info **part);
#endif


static hw_info_t hw_info;
static GS_FACTORYPARA * factory_para = NULL;
static unsigned char fact_para_static[GS_FACTORY_CFG_SIZE];
uint16_t empty_mac[3] = { 0x0b00, 0x0082, 0x0000 };
char *empty_pn = "00000000000";

static int load_factory_para(void)
{
#if defined(CONFIG_EMBED_CFG)
#if defined(CONFIG_EMBED_CFG_HEADER)
	memcpy( factory_para, fact_embeded_cfg, sizeof(fact_embeded_cfg) );
	printf("Notice: Loading embedded factory configuration @ 0x%08X from header\n", factory_para);
#else
	extern char _binary_factory_cfg_start;
	extern char _binary_factory_cfg_end;
	int len = &_binary_factory_cfg_end - &_binary_factory_cfg_start;
	int i =0;

	printf("Notice: Loading embedded factory configuration @ 0x%08X len %d\n", factory_para, len);

	memcpy( factory_para, &_binary_factory_cfg_start, len );
#endif
#else
#if defined(CONFIG_NAND_DMW96)
	int ret = 0;
	size_t size;
	nand_info_t *nand;
	nand = &nand_info[nand_curr_device];
	size = GS_FACTORY_CFG_SIZE;
	ret = nand_read_skip_bad(nand, GS_FACTORY_CFG_OFFSET,
				&size, NULL, nand->size, (u_char *)factory_para);
	if (ret) {
		printf("Warning: Can't read factory parameters from NAND: error %d\n", ret);
		return -1;
	}
#elif defined(CONFIG_QSPI)
	flash_info_t* info;
	int ret = 0;
	ulong addr_first;
	struct mtd_device *dev;
	struct part_info *part;
	u8 pnum, bank;

	mtdparts_init();
	ret = find_dev_and_part("factory", &dev, &pnum, &part);
	if (ret == 0) {
		bank = dev->id->num;
		info = &flash_info[bank];
		addr_first = part->offset + info->start[0];

		memcpy(factory_para, (long)addr_first, GS_FACTORY_CFG_SIZE);
	}
	else {
		printf("Warning: Can't read factory parameters from QSPI\n");
		return -1;
	}
	#else
		#error
	#endif
#endif
	return 0;
}

static unsigned short fact_para_checksum(GS_FACTORYPARA * factPara)
{
	unsigned char  *p = (unsigned char *)factPara;
	unsigned short sum = 0;
	unsigned short checksum = 0;
	unsigned length = 0;
	int i;

	length = ( factPara->length[1] << 16 ) | factPara->length[0];
	if ( length == 0xFFFFFFFF || length > GS_FACTORY_CFG_SIZE ) {
		return -1;
	}

	for ( i = 0; i < length; i+= 2 )
	{
		sum += ((p[i] << 8) | p[i+1]);
	}
	checksum = sum;

	return ((checksum >> 8) | (checksum << 8));
}

GS_FACTORYPARA *fact_para_get(void)
{
	if ( factory_para == NULL )
	{
		factory_para = (GS_FACTORYPARA*)fact_para_static;
		if ( load_factory_para() < 0 ) 
		{
			factory_para = NULL;
		}
		else
		{
			if ( fact_para_checksum( factory_para ) != 0 ) 
			{
				printf("Factory checksum error\n");
				factory_para = NULL;
			}
		}
	}
	return factory_para;
}

GS_FACTORYPARA *fact_para_reload(void)
{
	factory_para = NULL;
	return fact_para_get();
}

char *hw_get_pn_str(void)
{
	factory_param_sector_t *fact_para = (factory_param_sector_t*)fact_para_get();

	if ( fact_para != NULL )
		return (char*)(&fact_para->factory_para.PartNumber[0]);
	else
		return empty_pn;
}

int hw_get_eth_clock_source(void)
{
	hw_info_t* hw_info = hw_get_hw_feature();

	if ( hw_info == NULL )
		return 0;

	return hw_info->eth_clock_source;
}

//===========================================================================
int
hw_get_eth_interface(
    void
)
//===========================================================================
{
    hw_info_t* hw_info = hw_get_hw_feature();

    if ( hw_info == NULL )
        return 0;

    return hw_info->eth_interface;
}

//===========================================================================
int
hw_get_eth(
    void
)
//===========================================================================
{
    hw_info_t* hw_info = hw_get_hw_feature();

    if ( hw_info == NULL )
        return 0;

    return hw_info->eth;
}

uint16_t *hw_get_mac(void)
{
	factory_param_sector_t *fact_para = (factory_param_sector_t*)fact_para_get();

	if ( fact_para != NULL )
		return (uint16_t*)(&fact_para->factory_para.mac[0]);
	else
		return empty_mac;
}

/* Description: Get the hardware logic features from HW feature table in flash factory
*              parameter sector
*/
hw_info_t *hw_get_hw_feature(void)
{
	factory_param_sector_t *fact_para;
	hw_feature_table_header_t *hwTableHdr;
	hw_feature_attrib_t *featurePtr;
	uint16_t tableLength, parsedLength, index, featureLen;
	static int feature_table_loaded = 0;

	if ( feature_table_loaded )
	{
		return &hw_info;
	}

	fact_para = (factory_param_sector_t*)fact_para_get();
	memset( &hw_info, 0, sizeof ( hw_info ) );

	if ( fact_para == NULL )
	{
		printf("Invalid factory data\n");
		return NULL;
	}

	// make sure there is at least one HW feature attribute in the flash
	if( !(fact_para->factory_para.length[0] >= sizeof(factory_param_sector_t)) )
	{
		return &hw_info;
	}

	// make sure the flash sector contains a complete HW feature table
	tableLength = fact_para->hw_feature_table_header.hw_feature_table_len;
	tableLength += sizeof(GS_FACTORYPARA);
	if ( !(fact_para->factory_para.length[0] >= tableLength) )
	{
		printf("Warning: HW feature table invalid\n");
		while(1);
	}

	// locate the HW feature table in the sector, in case there are other tables in it
	hwTableHdr = &fact_para->hw_feature_table_header;
	while (HW_FEATURE_TABLE_VERSION != hwTableHdr->hw_feature_table_version)
	{
		hwTableHdr = (hw_feature_table_header_t*)((uint16_t*)hwTableHdr + hwTableHdr->hw_feature_table_len);
		if ((uint16_t*)hwTableHdr-(uint16_t*)fact_para >= fact_para->factory_para.length[0])
		{  // did not find the HW feature table in the flash sector
			hwTableHdr = NULL;
			break;
		}
	}
	if ( !(NULL!= hwTableHdr) )
	{
		printf("Warning: Can't find feature table\n");
		while(1);
	}

	tableLength = sizeof(uint16_t) * hwTableHdr->hw_feature_table_len;
	featurePtr = &fact_para->hw_feature;
	parsedLength = 6;

	for (index=0; index<hwTableHdr->hw_feature_attrib_count; index++)
	{
		switch (featurePtr->hw_feature_type)
		{
		case HW_FEATURE_VENDOR:
			memcpy( &hw_info.vendor, &featurePtr->hw_feature_value,
				featurePtr->hw_feature_len * sizeof(uint16_t) );
			break;
		case HW_FEATURE_MODELSTR:
			memcpy( &hw_info.modelstr, &featurePtr->hw_feature_value,
				featurePtr->hw_feature_len * sizeof(uint16_t) );
			break;
		case HW_FEATURE_MODEL_BASE:
			memcpy( &hw_info.gs_model, &featurePtr->hw_feature_value,
				featurePtr->hw_feature_len * sizeof(uint16_t) );
			break;
		case HW_FEATURE_MODELALIAS:
			memcpy( &hw_info.modelalias, &featurePtr->hw_feature_value,
				featurePtr->hw_feature_len * sizeof(uint16_t) );
			break;
		case HW_FEATURE_OEM_ID:
			hw_info.oem_id = featurePtr->hw_feature_value;
			break;
		case HW_FEATURE_VENDOR_FULLNAME:
			memcpy( &hw_info.vendor_fullname, &featurePtr->hw_feature_value,
				featurePtr->hw_feature_len * sizeof(uint16_t) );
			break;
		case HW_FEATURE_FWKEY:
			memcpy( &hw_info.fwkey, &featurePtr->hw_feature_value,
				featurePtr->hw_feature_len * sizeof(uint16_t) );
			break;
		case HW_FEATURE_ETH:
			hw_info.eth = featurePtr->hw_feature_value;
			break;
		case HW_FEATURE_ETH_INTERFACE:
			hw_info.eth_interface = featurePtr->hw_feature_value;
			break;
		case HW_FEATURE_ETH_CLOCK_SOURCE:
			hw_info.eth_clock_source = featurePtr->hw_feature_value;
			break;
		default:
			break;
		}

		// move to the next attribute
		featureLen = sizeof(hw_feature_attrib_t)
			+ sizeof(uint16_t)*( featurePtr->hw_feature_len - 1 );
		parsedLength += featureLen;
		featurePtr = (hw_feature_attrib_t*)((char*)featurePtr + featureLen);
		if (parsedLength >= tableLength)
		{
			break;
		}
	}

	feature_table_loaded = 1;
	return &hw_info;
}

int save_fact_para(GS_FACTORYPARA * buf_ptr)
{
	int ret = 0;
#if defined( CONFIG_NAND_DMW96 )
	size_t size;
	nand_info_t *nand;
	nand_erase_options_t nand_erase_options;
	unsigned char* buf;

	nand = &nand_info[nand_curr_device];

	buf = (unsigned char *)malloc(nand->erasesize);

	puts("Reading block zero...\n");
	nand_erase_options.length = nand->erasesize;
	size = nand->erasesize;
	ret = nand_read(nand, 0, &size, buf);
	if (ret)
	{
		puts("Error reading block zero\n");
		return ret;
	}

	memcpy( buf + GS_FACTORY_CFG_OFFSET, buf_ptr, GS_FACTORY_CFG_SIZE );

	puts("Erasing NAND...\n");
	nand_erase_options.length = nand->erasesize;
	nand_erase_options.quiet = 0;
	nand_erase_options.jffs2 = 0;
	nand_erase_options.scrub = 0;
	nand_erase_options.offset = 0;
	ret = nand_erase_opts(nand, &nand_erase_options);
	if (ret) {
		puts("Error erasing factory sector\n");
		return ret;
	}

	puts ("Writing to NAND...\n");
	size = nand->erasesize;
	ret = nand_write(nand, 0, &size, buf);
	if (ret) {
		puts("Error writing factory sector\n");
		return ret;
	}

	puts("done\n");
#elif defined (CONFIG_QSPI)
	flash_info_t *info = NULL;
	ulong addr_first, addr_last;
	struct mtd_device *dev;
	struct part_info *part;
	u8 pnum, bank;
	int rc;

	mtdparts_init();
	if (find_dev_and_part("factory", &dev, &pnum, &part) == 0) {
		bank = dev->id->num;
		info = &flash_info[bank];
		addr_first = part->offset + info->start[0];
		addr_last = addr_first + GS_FACTORY_CFG_SIZE - 1;

		if (flash_sect_protect(0, (long)addr_first, addr_last))
		{
			flash_sect_protect(1, (long)addr_first, addr_last);
			return 0;
		}

		printf("Erasing Flash...\n");
		if (flash_sect_erase((long)addr_first, addr_last))
		{
			flash_sect_protect(1, (long)addr_first, addr_last);
			return 0;
		}

		printf("Writing to Flash... ");
		rc = flash_write(buf_ptr, (long)addr_first, GS_FACTORY_CFG_SIZE);
		if (rc != 0)
		{
			flash_perror(rc);
			flash_sect_protect(1, (long)addr_first, addr_last);
			return 0;
		}
		flash_sect_protect(1, (long)addr_first, addr_last);
		printf("done\n");
	}
#else
	#error
#endif
	return ret;
}



/* EOF */
