/*
 * (C) Copyright 2012 DSP Group
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>

#include <syndwc_mmc.h>
#include <mmc.h>
#include <part.h>

#include <asm/io.h>
#include <asm/arch/gpio.h>
#include <asm/arch/clock.h>
#include <asm/arch/io.h>
#include <asm/arch/hardware.h>

#ifdef CONFIG_SDMMC_DETECT_GPIO
static int
mmc_get_card_detect(void)
{
	return !gpio_get_value(CONFIG_SDMMC_DETECT_GPIO);
}
#endif

static struct syndwc_pdata syndwc_pdata = {
	.card_num    = 0,
	.mclk        = 50000000,
	.f_max       = 50000000,
	.host_caps   = MMC_MODE_HS | MMC_MODE_HS_52MHz, //MMC_MODE_DDR_52MHz
	.voltages    = MMC_VDD_32_33 | MMC_VDD_33_34,
#ifdef CONFIG_SDMMC_DETECT_GPIO
	.card_detect = &mmc_get_card_detect,
#endif
};

#ifdef CONFIG_EMMC
int
mmc_get_env_addr(struct mmc *mmc, int copy, u32 *env_addr)
{
	int ret, i = 1;
	unsigned boot_part;
	disk_partition_t info;

	/* search bootable partition first */
	boot_part = (mmc->part_config >> 3) & 0x7;
	if (boot_part == 1 || boot_part == 2) {
		printf("switching to boot partition %u\n", boot_part);
		mmc_select_hwpart(mmc->block_dev.dev, boot_part);
	}

	/* try to find the u-boot environment partition */
	do {
		ret = get_partition_info(&mmc->block_dev, i++, &info);
		if (!ret && !strcmp((const char *)info.type, BOOT_PART_ENV)) {
			if (info.size * 512 < CONFIG_ENV_SIZE) {
				printf("ERROR: Environment partition too small!\n");
				return 1;
			}
			*env_addr = info.start * 512;
			printf("found evironment partition at %8.8x\n", *env_addr);
			return 0;
		}
	} while (ret == 0);

	printf("ERROR: No environment partition found! "
	       "Use gptpart to create one...\n");

	return 1;
}
#endif /* CONFIG_EMMC */

int board_mmc_init(bd_t *bis)
{
	writel(0x0841800 | readl(0x053020ec), 0x053020ec);
	writel(0x70000, 0x053020f0);
	clk_set_rate("sdio_clk", syndwc_pdata.mclk);
	reset_release("sdio_clk");
	clk_enable("sdio_clk");

	/* configure pins */
	io_pull("dgpio19", 1, 1); /* cmd */
	io_pull("dgpio14", 1, 1); /* d0 */
	io_mux("sd_clk");
	io_mux("sd_cmd");
	io_mux("sd_d0");

#ifdef CONFIG_SDMMC_4BIT
	syndwc_pdata.host_caps |= MMC_MODE_4BIT;
	io_pull("dgpio12", 1, 1); /* d1 */
	io_pull("dgpio13", 1, 1); /* d2 */
	io_pull("dgpio15", 1, 1); /* d3 */
	io_mux("sd_d1");
	io_mux("sd_d2");
	io_mux("sd_d3");
#endif

#ifdef CONFIG_SDMMC_WP
	io_mux("sd_wr_prot");
#endif

#if defined(CONFIG_SDMMC_DETECT_MUX) && defined(CONFIG_SDMMC_DETECT_GPIO)
	gpio_direction_input(CONFIG_SDMMC_DETECT_GPIO);
	io_pull(CONFIG_SDMMC_DETECT_MUX, 1, 1);
	io_mux(CONFIG_SDMMC_DETECT_MUX);
#else
	io_pull("dgpio17", 1, 1); /* detect */
	io_mux("sd_detect");
#endif

	/* Configure SD lines to 3.3V */
	gpio_direction_output(CGPIO(31), 1);
	/* wait for voltage to be stable */
	mdelay(1);
	io_pull("cgpio31", 1, 1);
	io_mux("cgpio31");

	return syndwc_initialize(&syndwc_pdata);
}
