#!/bin/bash

. /opt/dspg/v2.3.0-rc2/environment-setup-cortexa9-neon-dspg-linux-gnueabi

DST_DIR=../../../../ht/image_build/image_source

CURDIR="$(pwd)"
CFG="no"
EMBED_CFG="n"
EMBED_CFG_HEADER="y"

# Valid PRODUCT_MODEL settings:
# ht818
: ${PRODUCT_MODEL:="${1}"}
: ${CONSOLE:="no"}
SERIAL="no"

usage() {
	echo "build uboot with console both locked and unlocked"
    echo "Usage: ./build.sh [Models]"
    echo "Product Models: ht818"
}

build_uboot () {
	SERIAL=$1
	if [ x${SERIAL} = xyes ];then
		echo "===================== build uboot with unlocked console ====================="
		SERIAL_UNLOCKED=y
	else
		echo "===================== build uboot with  locked  console ====================="
		SERIAL_UNLOCKED=n
	fi

	make CONFIG_SERIAL_UNLOCKED="${SERIAL_UNLOCKED}" FACTORY_CFG="$CFG" CONFIG_EMBED_CFG="$EMBED_CFG" CONFIG_EMBED_CFG_HEADER="${EMBED_CFG_HEADER}" clean
	make CONFIG_SERIAL_UNLOCKED="${SERIAL_UNLOCKED}" FACTORY_CFG="$CFG" CONFIG_EMBED_CFG="$EMBED_CFG" CONFIG_EMBED_CFG_HEADER="${EMBED_CFG_HEADER}" $TARGET
	make CONFIG_SERIAL_UNLOCKED="${SERIAL_UNLOCKED}" FACTORY_CFG="$CFG" CONFIG_EMBED_CFG="$EMBED_CFG" CONFIG_EMBED_CFG_HEADER="${EMBED_CFG_HEADER}" V=99 CROSS_COMPILE=$CROSS_COMPILE

	if [ "$EMBED_CFG" = "y" ]; then
		../bootastic/mkbootimg.${PRODUCT_MODEL} \
			--loader=u-boot.bin \
			--loader-addr=${LOADADDR} \
			--crc32 \
			--output=u-boot.xbin
		cp -v u-boot.xbin u-boot.xbin.$CFG
	else
		if [ x$SERIAL = xyes ];then
		    cp -v u-boot.bin u-boot.bin.${PRODUCT_MODEL}.unlocked
		else
		    cp -v u-boot.bin u-boot.bin.${PRODUCT_MODEL}
		fi
	fi


	echo "======================================================="
	echo "Release u-boot for ${PRODUCT_MODEL}  CONSOLE=${SERIAL} "
	echo "======================================================="

	case "$PRODUCT_MODEL" in
		ht818)
			release_ht ht818
			;;
		*)
			usage
			exit
			;;
	esac
}

release_ht () {
	PRODUCT=$1

	if [ "$EMBED_CFG" = "y" ]; then
		uboot=./u-boot.xbin.$CFG
		cp -v $uboot ${DST_DIR}/bin/fact/
	else
		if [ x$CONSOLE = xyes ];then
			uboot=./u-boot.bin.${PRODUCT_MODEL}.unlocked
			cp -v $uboot ${DST_DIR}/bin/u-boot.${PRODUCT_MODEL}.dev.unlocked
		else
			uboot=./u-boot.bin.${PRODUCT_MODEL}
			cp -v $uboot ${DST_DIR}/bin/u-boot.${PRODUCT_MODEL}.dev
		fi
	fi
}

if [ x"$1" = x"help" ]; then
    usage
    exit
fi

if [ x"${PRODUCT_MODEL}" == x ]; then
    usage
    exit
fi

if [ x"$1" == xcfg ]; then
    CFG_PATH="board/grandstream/ht818"
    CFG="`echo $2 | sed -e s/\.cfg//`"
    EMBED_CFG="y"
    if [ "${EMBED_CFG_HEADER}x" = "yx" ]; then
        CFG_HEADER="fact_embeded_cfg"
        [ -e "${CFG_PATH}/${CFG_HEADER}" ] && rm ${CFG_PATH}/${CFG_HEADER}
        [ -e "${CFG_PATH}/${CFG_HEADER}.h" ] && rm ${CFG_PATH}/${CFG_HEADER}.h
        DD_BLOCK_SIZE=1024
        DD_FACT_CFG_OFFSET_SKIP=64 # GS_FACTORY_CFG_OFFSET_FLASH
        DD_FACT_CFG_SIZE_COUNT=64 # GS_FACTORY_CFG_SIZE_FLASH
        [ -e "${CFG_PATH}/${CFG}.cfg" ] && dd if=${CFG_PATH}/${CFG}.cfg of=${CFG_PATH}/${CFG_HEADER} bs=${DD_BLOCK_SIZE} skip=${DD_FACT_CFG_OFFSET_SKIP} count=${DD_FACT_CFG_SIZE_COUNT}
        [ -e "${CFG_PATH}/${CFG_HEADER}" ] && {
            cd ${CFG_PATH}
            ${CURDIR}/bin2header ${CFG_HEADER}
            cd -
        }
    else
        [ -e "${CFG_PATH}/${CFG}.o" ] && rm ${CFG_PATH}/${CFG}.o
    fi
fi

echo "============================================="
echo "Building U-boot for ${PRODUCT_MODEL}"
echo "============================================="

case "$PRODUCT_MODEL" in
    ht818)
        TARGET=ht818_nand_defconfig
	LOADADDR=0x41000000
        ;;
    *)
        usage
        exit
        ;;
esac

if [ "$EMBED_CFG" = "y" ]; then
	#build uboot with embeded cfg
	build_uboot  $CONSOLE
else
	#build uboot with console locked
	CONSOLE="no"
	build_uboot  $CONSOLE

	#build uboot with console unlocked
	CONSOLE="yes"
	build_uboot  $CONSOLE
fi
