/*
 * (C) Copyright 2013 Grandstream Networks, Inc.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <nand.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <ubi_uboot.h>
#include <jffs2/load_kernel.h>
#define CONFIG_MMU 
#include <asm/arch/factory.h>
#include <jffs2/jffs2.h>
#include <asm/arch/serial.h>

enum {
	GS_PARTS_SYSTEM = 0,
	GS_PARTS_SYSTEM2,
	GS_PARTS_MAX
};

const char *gs_system_parts[GS_PARTS_MAX] = {
	"system",
	"system2"
};

enum {
	GS_VOLUEM_CORE=0,
	GS_VOLUEM_BASE,
	GS_VOLUEM_PROG,
	GS_VOLUEM_MAX
};

const char *gs_ubi_volumes[GS_VOLUEM_MAX] = {
	"core",
	"base",
	"prog"
};

typedef struct {
	char image_ok;
	unsigned int version;
	unsigned int provision_counter;
	unsigned int compatible_version[2];
} gs_image_info_t;

gs_image_info_t gs_image_info[GS_PARTS_MAX][GS_VOLUEM_MAX] = {
	{{0,},{0,}},
	{{0,},{0,}}
};

#if 0
static int calculate_checksum( unsigned char *buf,
	unsigned int size, unsigned short header_checksum)
{
	int i;
	unsigned short check_sum = 0;
	unsigned short *ptr = ( unsigned short *)buf;
	for (i = 0; i < size/sizeof( unsigned short); i++)
	{
		check_sum += gs_endian_conv16( ptr[i] );
	}

	/* verify the checksum */
	if (((header_checksum + check_sum) & 0xffff) != 0)
		return 1;
	return 0;
}
#endif

int compare_versions( unsigned int Version1, unsigned int Version2 )
{
	unsigned short v[2] ;

	v[0] = ( Version1 >> 24) & 0xff;
	v[1] = ( Version2 >> 24) & 0xff;
	if ( v[0] > v[1] )
		return -1;
	else if ( v[1] > v[0] )
		return 1;

	v[0] = ( Version1 >> 16) & 0xff;
	v[1] = ( Version2 >> 16) & 0xff;
	if ( v[0] > v[1] )
		return -1;
	else if ( v[1] > v[0] )
		return 1;

	v[0] = ( Version1 >> 8) & 0xff;
	v[1] = ( Version2 >> 8) & 0xff;
	if ( v[0] > v[1] )
		return -1;
	else if ( v[1] > v[0] )
		return 1;

	v[0] = Version1 & 0xff;
	v[1] = Version2 & 0xff;
	if ( v[0] > v[1] )
		return -1;
	else if ( v[1] > v[0] )
		return 1;

	return 0;
}

static int check_part_volumes(char* mtd_partname, int idx)
{
	int vindex;
	int ret = 0;
	ImageHeader image_header;
	const char *vol_name;
	gs_image_info_t *img_info;

	if (ubi_part(mtd_partname, NULL)) {
		printf("\n** Cannot find mtd partition \"%s\"\n",
			mtd_partname);
		return -1;
	}

	for (vindex = 0; vindex < GS_VOLUEM_MAX; vindex++) {
		memset((void *)&image_header, 0, sizeof(ImageHeader));
		switch(vindex) {
		case GS_VOLUEM_CORE:
		case GS_VOLUEM_BASE:
		case GS_VOLUEM_PROG:
			vol_name = gs_ubi_volumes[vindex];
			img_info = &gs_image_info[idx][vindex];
			ubi_volume_read((char *)vol_name, (char *)&image_header,
				sizeof(ImageHeader));
			if (image_header.Signature != 0x23c97af9) {
				printf("%s: invalid signature\n", vol_name);
				img_info->image_ok = 0;
				continue;
			}
			img_info->image_ok = 1;
			img_info->version = gs_endian_conv32(image_header.Version);
			img_info->compatible_version[0] =
				gs_endian_conv32(image_header.CompatibleVersion[0]);
			img_info->compatible_version[1] =
				gs_endian_conv32(image_header.CompatibleVersion[1]);
			img_info->provision_counter =
				gs_endian_conv32(image_header.ProvisionCounter);
			break;
		default:
			printf("Error! unhandled volume\n");
			break;
		}
	}

	return ret;
}

void dump_images_version(void) 
{
	gs_image_info_t *core_info;
	gs_image_info_t *base_info;
	gs_image_info_t *prog_info;
	int ii;

	for (ii = 0; ii< GS_PARTS_MAX; ii++) {
		core_info = &gs_image_info[ii][GS_VOLUEM_CORE];
		base_info = &gs_image_info[ii][GS_VOLUEM_BASE];
		prog_info = &gs_image_info[ii][GS_VOLUEM_PROG];
		printf("===============%s===============\n", gs_system_parts[ii]);
		printf("Core version=%d.%d.%d.%d Base version=%d.%d.%d.%d Prog version=%d.%d.%d.%d\n"
			"Compatible with core %d.%d.%d.%d and base %d.%d.%d.%d\n"
			"Provision Counter=core:%u base:%u prog:%u\n",
			(core_info->version >> 24) & 0xff, (core_info->version >> 16) & 0xff,
			(core_info->version >> 8) & 0xff, (core_info->version) & 0xff,
			(base_info->version >> 24) & 0xff,(base_info->version >> 16) & 0xff,
			(base_info->version >> 8) & 0xff, (base_info->version) & 0xff, 
			(prog_info->version >> 24) & 0xff, (prog_info->version >> 16) & 0xff,
			(prog_info->version >> 8) & 0xff, (prog_info->version) & 0xff,

			(prog_info->compatible_version[0] >> 24 ) & 0xff,
			(prog_info->compatible_version[0] >> 16 ) & 0xff,
			(prog_info->compatible_version[0] >> 8 ) & 0xff,
			(prog_info->compatible_version[0]) & 0xff,
			(prog_info->compatible_version[1] >> 24 ) & 0xff,
			(prog_info->compatible_version[1] >> 16 ) & 0xff,
			(prog_info->compatible_version[1] >> 8 ) & 0xff,
			(prog_info->compatible_version[1]) & 0xff,

			core_info->provision_counter,
			base_info->provision_counter,
			prog_info->provision_counter);
		};
}

int get_max_provision_counter(int coreCounter, int baseCounter, int progCounter)
{
	int max_counter = 0;

	max_counter = (progCounter >= baseCounter) ? (progCounter) : (baseCounter);
	max_counter = (max_counter >= coreCounter) ? (max_counter) : (coreCounter);

	return max_counter;
}

int check_all_parts(void)
{
	gs_image_info_t *core_info, *base_info, *prog_info;
	gs_image_info_t *core2_info, *base2_info, *prog2_info;
	char compatible[GS_PARTS_SYSTEM];
	int retval = 0;
	int max_counter_system = 0;
	int max_counter_system2 = 0;

	check_part_volumes((char *)gs_system_parts[GS_PARTS_SYSTEM], GS_PARTS_SYSTEM);
	check_part_volumes((char *)gs_system_parts[GS_PARTS_SYSTEM2], GS_PARTS_SYSTEM2);

	core_info = &gs_image_info[GS_PARTS_SYSTEM][GS_VOLUEM_CORE];
	base_info = &gs_image_info[GS_PARTS_SYSTEM][GS_VOLUEM_BASE];
	prog_info = &gs_image_info[GS_PARTS_SYSTEM][GS_VOLUEM_PROG];

	core2_info = &gs_image_info[GS_PARTS_SYSTEM2][GS_VOLUEM_CORE];
	base2_info = &gs_image_info[GS_PARTS_SYSTEM2][GS_VOLUEM_BASE];
	prog2_info = &gs_image_info[GS_PARTS_SYSTEM2][GS_VOLUEM_PROG];

	dump_images_version();

	/* if all parts are OK */
	if (core_info->image_ok && base_info->image_ok && prog_info->image_ok &&
		core2_info->image_ok && base2_info->image_ok && prog2_info->image_ok) {
		printf("All images look ok\n");
		/* check prog compatibility */
		if (!compare_versions(prog_info->compatible_version[0], core_info->version) &&
			!compare_versions(prog_info->compatible_version[1], core_info->version )) {
			printf("core base prog are compatible\n");
			compatible[GS_PARTS_SYSTEM] = 1;
		}

		/* check prog2 compatibility */
		if (!compare_versions(prog2_info->compatible_version[0], core2_info->version) &&
			!compare_versions(prog2_info->compatible_version[1], core2_info->version))
		{
			printf("core2 base2 prog2 are compatible\n");
				compatible[GS_PARTS_SYSTEM2] = 1;
		}

		if ( compatible[GS_PARTS_SYSTEM] && !compatible[GS_PARTS_SYSTEM2] )
			return 0;
		if ( !compatible[GS_PARTS_SYSTEM] && compatible[GS_PARTS_SYSTEM2] )
			return 1;

		/* check provision counter */
		max_counter_system = get_max_provision_counter(core_info->provision_counter,base_info->provision_counter,prog_info->provision_counter);
		max_counter_system2 = get_max_provision_counter(core2_info->provision_counter,base2_info->provision_counter,prog2_info->provision_counter);
		if (max_counter_system > max_counter_system2)
			return 0;
		else if (max_counter_system < max_counter_system2)
			return 1;

		/* compare versions */
		/* prog first */
		retval = compare_versions(prog_info->version, prog2_info->version);
		printf( "Compare progs, %s\n", (retval == 0) ? "both are equal" :
			((retval == 1) ? "prog2 is selected" : "prog is selected"));
		if (retval == 1)
			return 1;
		else if (retval == -1)
			return 0;

		/* base second */
		retval = compare_versions(base_info->version, base2_info->version);
		printf( "Compare bases, %s\n", ( retval == 0 ) ? "both are equal" :
			((retval == 1) ? "base2 is selected" : "base is selected"));
			if (retval == 1)
				return 1;
			else if (retval == -1)
				return 0;

		/* core third */
		retval = compare_versions(core_info->version, core2_info->version);
		printf( "Compare cores, %s\n", ( retval == 0 ) ? "both are equal" :
			((retval == 1) ? "core2 is selected" : "core is selected"));
		if (retval == 1)
			return 1;
		else if (retval == -1)
			return 0;

		return 0;
	}

	/* if core, base and prog are ok */
	if (core_info->image_ok && base_info->image_ok &&
		prog_info->image_ok && (!core2_info->image_ok ||
		!base2_info->image_ok || !prog2_info->image_ok)) {
		printf("Only core, base and prog are OK.\n");
		return 0;
	}
	
	/* if core2, base2 and prog2 are ok */
	if ((!core_info->image_ok || !base_info->image_ok ||
		!prog_info->image_ok) && core2_info->image_ok &&
		base2_info->image_ok && prog2_info->image_ok) {
		printf ("Only core2, base2 and prog2 are OK.\n");
		return 1;
	}
	printf("core(%s) base(%s) prog(%s),core2(%s) base2(%s) prog2(%s)\n",
		core_info->image_ok ? "O" : "N",
		base_info->image_ok ? "O" : "N",
		prog_info->image_ok ? "O" : "N",
		core2_info->image_ok ? "O" : "N",
		base2_info->image_ok ? "O" : "N",
		prog2_info->image_ok ? "O" : "N");

	printf ( "Settling for default. Selecting core, base and prog.\n" );
	return 0;
}

int do_gsboot(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char boot_cmd[512];
	char *console, *uart_locked;

	console = getenv("console");
	if ( console )
	{
		if ( strcmp(console, "yes") == 0)
		{
			setenv("uart_locked", "uart_locked=0");
		}
		if ( strncmp(console, "ttyS", 4) )
		{
			dvf_serial_late_init();
		}
	}
	uart_locked = getenv("uart_locked");
	if ( !uart_locked )
	{
		setenv("uart_locked", "uart_locked=1");
		uart_locked = "uart_locked=1";
	}

	dcache_enable();

    /* set mtdparts to default before checking */
	snprintf(boot_cmd, sizeof(boot_cmd), "setenv mtdparts %s", MTDPARTS_DEFAULT);
	run_command(boot_cmd, 0);

	if (check_all_parts())
	{
		printf("Selecting core2, base2 and prog2.\n");
		snprintf(boot_cmd, sizeof(boot_cmd),
			"setenv mtdparts %s && setenv linux_bootargs %s %s"
			" && run linux_boot",
			MTDPARTS_DEFAULT2, KERNEL_BOOTARGS2,
			uart_locked);

		run_command(boot_cmd, 0);
	}
	else
	{
		printf("Selecting core, base and prog.\n");
		snprintf(boot_cmd, sizeof(boot_cmd),
			"setenv mtdparts %s && setenv linux_bootargs %s %s"
			" && run linux_boot",
			MTDPARTS_DEFAULT, KERNEL_BOOTARGS,
			uart_locked);

		run_command(boot_cmd, 0);
	}

	return 0;
}

/**************************************************/
U_BOOT_CMD(
	gsboot, 1, 1, do_gsboot, "gs boot", "" 
);

