/*
 * Copyright 2012 Grandstream Networks, Inc.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <nand.h>
#include <malloc.h>
#include <asm/arch/factory.h>
#include <jffs2/jffs2.h>
#include <cli.h>

extern char console_buffer[CONFIG_SYS_CBSIZE];

unsigned short mac_fact_para_checksum(GS_FACTORYPARA *factPara)//dean
{
	unsigned char factParaBuf[GS_FACTORY_CFG_SIZE];
	GS_FACTORYPARA* pfactParaBuf;
	unsigned char *p = (unsigned char *)&factParaBuf[0];
	unsigned short sum = 0;
	unsigned short checksum = 0;
	unsigned int length = 0;
	int i;

	length = ( factPara->length[1] << 16 ) | factPara->length[0];
	if ( length == 0xFFFFFFFF || length > GS_FACTORY_CFG_SIZE ) {
		return 0;
	}

	memcpy(&factParaBuf, factPara, length);
	pfactParaBuf = (GS_FACTORYPARA*)&factParaBuf[0];
	pfactParaBuf->checksum = 0;

	for (i = 0; i < length; i+=2)
	{
		sum += (p[i] << 8 | p[i+1]);
	}

	checksum = 0xffff - sum + 0x1;

	return (checksum >> 8 | checksum << 8);
}

int update_ethaddr (void)
{
	GS_FACTORYPARA *factPara = fact_para_get();
	char buf[32];

	if ( factPara == NULL )
	{
		setenv("ethaddr", "00:0B:82:00:00:00");
		return 1;
	}

	memset(buf, 0, sizeof(buf));
	sprintf(buf, "%02X:%02X:%02X:%02X:%02X:%02X",
		factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
		factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
		factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);

	printf ("MAC:   %s\n", buf);
	setenv("ethaddr", buf);

	return 0;
}

int do_setmac (cmd_tbl_t *cmdtp, int flag, int argc, char *const argv[])
{
	int rcode = 0;
	GS_FACTORYPARA *factPara;
	ulong newMacAddr = 0;
	ulong newMacAddrHi = 0;
	int inputLen;

	if (argc > 1) {
		printf ("Usage:\n%s\n", cmdtp->usage);
		return 1;
	}

	factPara = fact_para_reload();
	if ( factPara == NULL )
		return 1;

	do {
		inputLen = cli_readline("Input MAC addr: ");
		if (inputLen == 12) 
			break;
		printf ("Invalid MAC address format.\n");
	} while (1);

	if (inputLen == 12)
	{
		/* simple_strtoul will truncate to 32-bits due to no 64-bit multiply */
		newMacAddr = simple_strtoul(console_buffer, NULL, 16);
		console_buffer[4] = 0;
		newMacAddrHi = simple_strtoul( console_buffer, NULL, 16 );
	}
	else
	{
		printf ("Invalid MAC address format.\n");
		return 1;
	}

	factPara->mac[0] = ((newMacAddrHi >> 8) & 0xFF) | (((newMacAddrHi) & 0xFF) << 8);
	factPara->mac[1] = ((newMacAddr >> 24) & 0xFF) | (((newMacAddr >> 16 ) & 0xFF) << 8);
	factPara->mac[2] = ((newMacAddr >> 8) & 0xFF) | (((newMacAddr) & 0xFF) << 8);
	printf("MAC addr set to: %02X:%02X:%02X:%02X:%02X:%02X\n",
		factPara->mac[0] & 0xFF, (factPara->mac[0] >> 8) & 0xFF,
		factPara->mac[1] & 0xFF, (factPara->mac[1] >> 8) & 0xFF,
		factPara->mac[2] & 0xFF, (factPara->mac[2] >> 8) & 0xFF);

	factPara->checksum = mac_fact_para_checksum(factPara);
	rcode = save_fact_para(factPara);
	if (rcode)
	{
		printf("Write factory data failed\n");
	}

	return rcode;
}

U_BOOT_CMD(
	setmac, 1, 1, do_setmac,
	"setmac - program MAC addr for devices\n",
	"input xxxxxx or xxxxxxxxxxxx - last 6 digit or whole 12 digit of MAC, 00:0b:82:xx:xx:xx\n"
);

