/*
 * NVRAM variable manipulation (Linux kernel half)
 *
 * Copyright 2004, Broadcom Corporation
 * All Rights Reserved.
 *
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: nvram_linux.c,v 1.1 2013/02/05 23:28:00 sfrick Exp $
 *
 * SXIE - added ioctl to erase all tuples
 * SCF - rewrote _nvram_read and nvram_commit from scratch to support zlib compression
 *     - modified _nvram_free to correctly track nvram_size
 *     - modified _nvram_realloc to allow hash table to grow dynamically to any size up
       -          to max avail kmalloc'able memory
 */
#include <common.h>
#include <malloc.h>
#include <errno.h>
#include <flash.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/concat.h>
#include <jffs2/jffs2.h>
#include <nvram.h>

/* Defines */

/* Globals */
static unsigned int nvram_initialized = 0;

static void
puts_trim_cr(char *str)
{
	int len= strlen(str);
	if (len && (str[len-1] == '\r')) len--;
	printf("%.*s\n", len, str);
}

int do_nvram(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char * value;
	char * name;

	if (argc < 2)
		return CMD_RET_USAGE;

	if (!strcmp(argv[1], "init")) {
		if( !dev_nvram_init() )
			nvram_initialized = 1;
		return 0;
	}

	if (nvram_initialized == 0) {
		printf("please run 'nvram init' first\n");
		return 0;
	}

	if (!strcmp(argv[1], "get")) {
		if (argc < 3)
			return CMD_RET_USAGE;

		if ((value = nvram_get(argv[2])))
			puts_trim_cr(value);
	}
	else if (!strcmp(argv[1], "set")) {
		if (argc < 3)
			return CMD_RET_USAGE;

		value = argv[2];
		name = strsep(&value, "=");
		nvram_set(name, value);
	}
	else if (!strcmp(argv[1], "unset")) {
		if (argc < 3)
			return CMD_RET_USAGE;
		nvram_unset(argv[2]);        
	}
	else if (!strcmp(argv[1], "commit")) {
		nvram_commit();
	}
	else if (!strcmp(argv[1], "show")) {
		char *nvram_buf = malloc(NVRAM_BUFFER_SPACE);
		if (nvram_buf)
		{
			nvram_getall(nvram_buf, NVRAM_BUFFER_SPACE);
			for (name = nvram_buf; *name; name += strlen(name) + 1)
				puts_trim_cr(name);
			free(nvram_buf);
		}
	}
	else {
		return CMD_RET_USAGE;
	}

	return 0;
}

U_BOOT_CMD(nvram, 3, 0, do_nvram,
	"NVRAM support",
	"init - nvram initialize\n"
	"nvram show\n"
	"nvram get name\n"
	"nvram set name=value\n"
	"nvram unset name\n"
	"nvram commit\n"
);


