
/*
 * (C) Copyright 2011
 * DSPG Technologies GmbH
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <command.h>
#include <part.h>

int do_write(cmd_tbl_t *cmdtp, int flag, int argc, char * const argv[])
{
	char *ep;
	int dev;
	int part = 0;
	struct block_dev_desc *dev_desc = NULL;
	void *addr;
	disk_partition_t part_info;
	ulong offset = 0u;
	ulong limit = 0u;
	uint blksz;

	if (argc != 4 && argc != 5) {
		cmd_usage(cmdtp);
		return 1;
	}

	dev = (int)simple_strtoul(argv[2], &ep, 16);
	if (*ep) {
		if (*ep != ':') {
			printf("Invalid block device %s\n", argv[2]);
			return 1;
		}
		part = (int)simple_strtoul(++ep, NULL, 16);
	}

	dev_desc = get_dev(argv[1], dev);
	if (dev_desc == NULL) {
		printf("Block device %s %d not supported\n", argv[1], dev);
		return 1;
	}

	addr = (void *)simple_strtoul(argv[3], NULL, 16);

	if (part != 0) {
		if (get_partition_info(dev_desc, part, &part_info)) {
			printf("Cannot find partition %d\n", part);
			return 1;
		}
		offset = part_info.start;
		limit = part_info.size;
		blksz = part_info.blksz;
	} else {
		offset = 0;
		limit = dev_desc->lba;
		blksz = dev_desc->blksz;
	}

	if (argc >= 5) {
		unsigned long s = simple_strtoul(argv[4], NULL, 16);

		s = (s + blksz - 1) / blksz;
		if (s > limit) {
			printf("Access beyond partition size!\n");
			return 1;
		}
		limit = s;
	}

	if (!dev_desc->block_write) {
		printf("Read only device!");
		return 1;
	}

	return dev_desc->block_write(dev, offset, limit, (void *)addr) != limit;
}

U_BOOT_CMD(
	write, 5, 0, do_write,
	"Write binary data to a partition",
	"<interface> <dev[:part]> <addr> [size]\n"
	"    - write from 'addr' on 'interface' device 'dev', partition 'part',\n"
	"      e.g. write mmc 0:1 ${loadaddr} 200"
);

