/*
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * Designware QoS Ethernet IP driver for u-boot
 */

#include <common.h>
#include <miiphy.h>
#include <malloc.h>
#include <linux/err.h>
#include <asm/io.h>
#include <linux/ccu.h>
#include <asm/arch/factory.h>
#include "designware_qos.h"

#if defined( CONFIG_RTL836X_SWITCH )
#ifdef RTL_SWITCH_API
#include <rtk_switch.h>
#include <port.h>
#include <cpu.h>
#include <rtl8367c_asicdrv_port.h>
#define RTL_PHY_MASK 0xFFFFFFF5 /* RTL8363NB */
struct mii_dev *rtl_bus;
#endif
#endif

#define MAC_MDIO_ADDRESS     0x0200
#define MAC_MDIO_DATA        0x0204

#define BASE_ADDR_MAC 0x06E00000

#define DWCEQOS_MDIO_PHYADDR(x)     (((x) & 0x1f) << 21)
#define DWCEQOS_MDIO_PHYREG(x)      (((x) & 0x1f) << 16)

#define DWCEQOS_MAC_MDIO_ADDR_CR(x)      (((x & 15)) << 8)
#define DWCEQOS_MAC_MDIO_ADDR_CR_20      2
#define DWCEQOS_MAC_MDIO_ADDR_CR_35      3
#define DWCEQOS_MAC_MDIO_ADDR_CR_60      0
#define DWCEQOS_MAC_MDIO_ADDR_CR_100     1
#define DWCEQOS_MAC_MDIO_ADDR_CR_150     4
#define DWCEQOS_MAC_MDIO_ADDR_CR_250     5
#define DWCEQOS_MAC_MDIO_ADDR_GOC_READ   (3<<2)
#define DWCEQOS_MAC_MDIO_ADDR_GOC_WRITE  (1<<2)
#define DWCEQOS_MAC_MDIO_ADDR_GB         (1<<0)

/* Simple MDIO functions implementing mii_bus */
int dw_mdio_read(struct mii_dev *bus, int addr, int devad, int reg)
{
	struct eth_mac_regs *mac_p = bus->priv;
	ulong start;
	u32 miiaddr;
	int timeout = CONFIG_MDIO_TIMEOUT;

	miiaddr = DWCEQOS_MDIO_PHYADDR(addr) |
		  DWCEQOS_MDIO_PHYREG(reg) |
		  DWCEQOS_MAC_MDIO_ADDR_CR(DWCEQOS_MAC_MDIO_ADDR_CR_250) |
		  DWCEQOS_MAC_MDIO_ADDR_GOC_READ;
	writel(miiaddr | MII_BUSY, &mac_p->miiaddr);

	start = get_timer(0);
	while (get_timer(start) < timeout) {
		if (!(readl(&mac_p->miiaddr) & MII_BUSY))
			return readl(&mac_p->miidata);
		udelay(10);
	};

	return -1;
}

int dw_mdio_write(struct mii_dev *bus, int addr, int devad, int reg,
			 u16 val)
{
	struct eth_mac_regs *mac_p = bus->priv;
	ulong start;
	u32 miiaddr;
	int ret = -1, timeout = CONFIG_MDIO_TIMEOUT;

	writel(val, &mac_p->miidata);

	miiaddr = DWCEQOS_MDIO_PHYADDR(addr) |
		  DWCEQOS_MDIO_PHYREG(reg) |
		  DWCEQOS_MAC_MDIO_ADDR_CR(DWCEQOS_MAC_MDIO_ADDR_CR_250) |
		  DWCEQOS_MAC_MDIO_ADDR_GOC_WRITE;
	writel(miiaddr | MII_BUSY, &mac_p->miiaddr);

	start = get_timer(0);
	while (get_timer(start) < timeout) {
		if (!(readl(&mac_p->miiaddr) & MII_BUSY)) {
			ret = 0;
			break;
		}
		udelay(10);
	};

	return ret;
}

#if defined( CONFIG_RTL836X_SWITCH )
#ifdef RTL_SWITCH_API
extern int rtl_init_mii_dev(struct mii_dev *bus);

static int switch_mdio_read(struct mii_dev *bus, int phyaddr, int devad, int phyreg)
{
	unsigned int RegData;
	int ret;

	if (RTL_PHY_MASK & (1 << phyaddr))
		return 0;

	ret = rtl8367c_getAsicPHYReg(phyaddr, phyreg, &RegData);
	if (ret != 0)
		printf("%s %d return err\n", __func__, __LINE__);

	return RegData;
}

static int switch_mdio_write(struct mii_dev *bus, int phyaddr, int devad, int phyreg,
	u16 phydata)
{
	int ret;

	if (RTL_PHY_MASK & (1 << phyaddr))
		return 0;

	ret = rtl8367c_setAsicPHYReg(phyaddr, phyreg, phydata);
	if (ret != 0)
		printf("%s %d return err\n", __func__, __LINE__);

	return 0;
}

int switch_mdio_reset(struct mii_dev *bus)
{
	return 0;
}

int rtl8363nb_switch_init(struct mii_dev *bus)
{
	rtk_port_mac_ability_t mac_cfg;
	rtk_mode_ext_t mode;
	rtk_data_t txDelay, rxDelay;
	rtk_api_ret_t ret;;


	rtl_init_mii_dev(bus);
	ret = rtk_switch_init();

	/* Set extension port 0 to RGMII with Force mode, 1000M, Full-duplex, enable TX&RX pause*/
	mode = MODE_EXT_RGMII;
	mac_cfg.forcemode = MAC_FORCE;
	mac_cfg.speed = PORT_SPEED_1000M;
	mac_cfg.duplex = PORT_FULL_DUPLEX;
	mac_cfg.link = PORT_LINKUP;
	mac_cfg.nway = DISABLED;
	mac_cfg.txpause = ENABLED;
	mac_cfg.rxpause = ENABLED;
	ret = rtk_port_macForceLinkExt_set(EXT_PORT0, mode, &mac_cfg);

#ifdef CONFIG_RTL836X_SWITCH_EXT
	txDelay = 1;
	rxDelay = 4;
	ret = rtk_port_rgmiiDelayExt_set( EXT_PORT0, txDelay, rxDelay );
	ret = rtk_port_rgmiiDelayExt_get(EXT_PORT0, &txDelay, &rxDelay);
	printf("EXT_PORT0 get RGMII TX delay %d RX delay %d, ret: %d\n", txDelay, rxDelay, ret);
#endif
	rtl8363_port_isolation(1);
	ret = rtk_port_phyEnableAll_set(1);

	return 0;
}
#endif /* RTL_SWITCH_API */
#endif /* CONFIG_RTL836X_SWITCH */

static int dw_mdio_init(char *name, struct eth_mac_regs *mac_regs_p)
{
	int eth_chip = hw_get_eth();
	int clk_src = hw_get_eth_clock_source();
	struct mii_dev *bus = mdio_alloc();

	if (!bus) {
		printf("Failed to allocate MDIO bus\n");
		return -1;
	}

	sprintf(bus->name, name);
	bus->priv = (void *)mac_regs_p;

#if defined( CONFIG_RTL836X_SWITCH )
	if ( eth_chip == ETH_TYPE_RTL8363NB )
	{
		bus->read = switch_mdio_read;
		bus->write = switch_mdio_write;
		bus->phy_mask = RTL_PHY_MASK;
		bus->reset = switch_mdio_reset;
		rtl8363nb_switch_init(bus);
	}
	else
#endif
	{
		bus->read = dw_mdio_read;
		bus->write = dw_mdio_write;
	}

	return mdio_register(bus);
}

static void tx_descs_init(struct eth_device *dev)
{
	struct dw_eth_dev *priv = dev->priv;
	struct eth_dma_regs *dma_p = priv->dma_regs_p;
	struct dmamacdescr *desc_table_p = &priv->tx_mac_descrtable[0];
	char *txbuffs = &priv->txbuffs[0];
	struct dmamacdescr *desc_p;
	u32 idx;

	for (idx = 0; idx < CONFIG_TX_DESCR_NUM; idx++) {
		desc_p = &desc_table_p[idx];
		desc_p->buf1_addr= &txbuffs[idx * CONFIG_ETH_BUFSIZE];
		desc_p->buf2_addr = NULL;
		desc_p->buf_length = 0;
		desc_p->dmamac_cntl = 0;
	}

	writel((ulong)&desc_table_p[0],
	       &dma_p->ch0_txdesc_list_addr);
	writel((ulong)(desc_table_p +
	       (sizeof(struct dmamacdescr) * (CONFIG_TX_DESCR_NUM-1))),
	       &dma_p->ch0_txdesc_tail_pointer);
	writel(CONFIG_TX_DESCR_NUM - 1,
	       &dma_p->ch0_txdesc_ring_length);
}

static void rx_descs_init(struct eth_device *dev)
{
	struct dw_eth_dev *priv = dev->priv;
	struct eth_dma_regs *dma_p = priv->dma_regs_p;
	struct dmamacdescr *desc_table_p = &priv->rx_mac_descrtable[0];
	char *rxbuffs = &priv->rxbuffs[0];
	struct dmamacdescr *desc_p;
	u32 idx;

	for (idx = 0; idx < CONFIG_RX_DESCR_NUM; idx++) {
		desc_p = &desc_table_p[idx];
		desc_p->buf1_addr = &rxbuffs[idx * CONFIG_ETH_BUFSIZE];
		desc_p->buf2_addr = NULL;
		desc_p->buf_length = (MAC_MAX_FRAME_SZ & DESC_RXCTRL_SIZE1MASK);
		desc_p->dmamac_cntl = DWCEQOS_DMA_RDES3_INTE |
				      DWCEQOS_DMA_RDES3_OWN  |
				      DWCEQOS_DMA_RDES3_BUF1V;
	}

	writel((ulong)&desc_table_p[0],
	       &dma_p->ch0_rxdesc_list_addr);
	writel((ulong)(desc_table_p +
	       (sizeof(struct dmamacdescr) * (CONFIG_RX_DESCR_NUM-1))),
	       &dma_p->ch0_rxdesc_tail_pointer);
	writel(CONFIG_RX_DESCR_NUM - 1,
	       &dma_p->ch0_rxdesc_ring_length);
	writel(DWCEQOS_DMA_CH_CTRL_PBL(4) |
	       DWCEQOS_DMA_CH_RX_CTRL_BUFSIZE(DWCEQOS_DWCEQOS_RX_BUF_SIZE) |
	       DWCEQOS_DMA_CH_CTRL_START,
	       &dma_p->ch0_rx_control);
}

static void descs_init(struct eth_device *dev)
{
	tx_descs_init(dev);
	rx_descs_init(dev);
}

static int mac_reset(struct eth_device *dev)
{
	struct dw_eth_dev *priv = dev->priv;
	struct eth_mac_regs *mac_p = priv->mac_regs_p;
	struct eth_dma_regs *dma_p = priv->dma_regs_p;
	ulong start;
	int timeout = CONFIG_MACRESET_TIMEOUT;

	if (priv->initialized)
		return 0;

	writel(FULLDPLXMODE, &mac_p->conf);
	writel(DMAMAC_SWR, &dma_p->mode);

	if (priv->interface == PHY_INTERFACE_MODE_RGMII) {
		priv->initialized = 1;
		return 0;
	}

	start = get_timer(0);
	while (get_timer(start) < timeout) {
		if (!(readl(&dma_p->mode) & DMAMAC_SWR)) {
			priv->initialized = 1;
			return 0;
		}

		/* Try again after 10usec */
		udelay(10);
	};

	printf("MAC reset failed\n");
	return 0;
}

static int dw_write_hwaddr(struct eth_device *dev)
{
	struct dw_eth_dev *priv = dev->priv;
	struct eth_mac_regs *mac_p = priv->mac_regs_p;
	u32 macid_lo, macid_hi;
	u8 *mac_id = &dev->enetaddr[0];

	macid_lo = mac_id[0] + (mac_id[1] << 8) + \
		   (mac_id[2] << 16) + (mac_id[3] << 24);
	macid_hi = mac_id[4] + (mac_id[5] << 8);

	writel(macid_hi, &mac_p->macaddr0hi);
	writel(macid_lo, &mac_p->macaddr0lo);

	return 0;
}

static void dw_adjust_link(struct dw_eth_dev *priv,
			   struct eth_mac_regs *mac_p,
			   struct phy_device *phydev)
{
	u32 conf = readl(&mac_p->conf);

	conf &= ~(MII_PORTSELECT | FES_100 | FULLDPLXMODE);
#ifdef CONFIG_PHY_FIXED
	conf |= FULLDPLXMODE;
	if (priv->interface != PHY_INTERFACE_MODE_RGMII)
		conf |= MII_PORTSELECT | FES_100;
#else
	if (!phydev->link) {
		printf("%s: No link.\n", phydev->dev->name);
		return;
	}

	if (phydev->speed != 1000)
		conf |= MII_PORTSELECT;

	if (phydev->speed == 100)
		conf |= FES_100;

	if (phydev->duplex)
		conf |= FULLDPLXMODE;
#endif

	writel(conf, &mac_p->conf);

#ifdef CONFIG_PHY_FIXED
	if (priv->interface == PHY_INTERFACE_MODE_RGMII)
		printf("Speed: 1000, full duplex\n");
	else
		printf("Speed: 100, full duplex\n");
#else
	printf("Speed: %d, %s duplex%s\n", phydev->speed,
	       (phydev->duplex) ? "full" : "half",
	       (phydev->port == PORT_FIBRE) ? ", fiber mode" : "");
#endif
}

static void dw_eth_halt(struct eth_device *dev)
{
	struct dw_eth_dev *priv = dev->priv;
	struct eth_mac_regs *mac_p = priv->mac_regs_p;

	writel(readl(&mac_p->conf) & ~(RXENABLE | TXENABLE), &mac_p->conf); /* FIXME */
	//writel(readl(&dma_p->opmode) & ~(RXSTART | TXSTART), &dma_p->opmode); /* FIXME */

#ifndef CONFIG_PHY_FIXED
	phy_shutdown(priv->phydev);
#endif
}

static int dw_eth_init(struct eth_device *dev, bd_t *bis)
{
	struct dw_eth_dev *priv = dev->priv;
	struct eth_mac_regs *mac_p = priv->mac_regs_p;
	struct eth_mtl_regs *mtl_p = priv->mtl_regs_p;
	struct eth_dma_regs *dma_p = priv->dma_regs_p;

	descs_init(dev);

	writel(DWCEQOS_DMA_SYSBUS_MODE_BURST(1) | DWCEQOS_DMA_SYSBUS_MODE_FB |
	       DWCEQOS_DMA_SYSBUS_MODE_AAL |
	       DWCEQOS_DMA_SYSBUS_MODE_RD_OSR_LIMIT(1) |
	       DWCEQOS_DMA_SYSBUS_MODE_WR_OSR_LIMIT(1),
	       &dma_p->sysbusmode);

	writel(DWCEQOS_DMA_CH_CTRL_PBL(4) | DWCEQOS_DMA_CH_CTRL_START,
	       &dma_p->ch0_tx_control);

	/* MTL */
	writel(DWCEQOS_MTL_TXQ_SIZE(0x800) | DWCEQOS_MTL_TXQ_TXQEN |
	       DWCEQOS_MTL_TXQ_TSF | DWCEQOS_MTL_TXQ_TTC512,
	       &mtl_p->txq0_oper_mode);
	writel(DWCEQOS_MTL_RXQ_SIZE(0x1000),
	       &mtl_p->rxq0_oper_mode);

	/* Enable for AV */
	writel(0x00000001, &mac_p->macrxqctrl0);
	/* Enable multicast/broadcast */
	writel(0x00100000, &mac_p->macrxqctrl1);
	/* Enable all priorities for Q0 */
	writel(0x000000ff, &mac_p->macrxqctrl2);

	/* MAC regs */
	writel(DWCEQOS_MAC_PKT_FILT_RA | DWCEQOS_MAC_PKT_FILT_PM |
	       DWCEQOS_MAC_PKT_FILT_PR,
	       &mac_p->packetfilter);
	/* Disable CRC checks: */
	writel(0x00010000, BASE_ADDR_MAC + MAC_EXT_CONFIGURATION);
	writel(DWCEQOS_MAC_CFG_DM | DWCEQOS_MAC_CFG_TE | DWCEQOS_MAC_CFG_RE,
	       &mac_p->conf);

#ifndef CONFIG_PHY_FIXED
	/* Start up the PHY */
	if (phy_startup(priv->phydev)) {
		printf("Could not initialize PHY %s\n",
		       priv->phydev->dev->name);
		return -1;
	}
#endif

	dw_adjust_link(priv, mac_p, priv->phydev);

#ifndef CONFIG_PHY_FIXED
	if (!priv->phydev->link) {
		printf("No link\n");
		return -1;
	}
#endif

	return 0;
}

static int dw_eth_send(struct eth_device *dev, void *packet, int length)
{
	struct dw_eth_dev *priv = dev->priv;
	struct eth_dma_regs *dma_p = priv->dma_regs_p;
	u32 desc_num = priv->tx_currdescnum;
	volatile struct dmamacdescr *desc_p = &priv->tx_mac_descrtable[desc_num];
	struct dmamacdescr *desc_p_last =
			&priv->tx_mac_descrtable[CONFIG_TX_DESCR_NUM - 1];

	/* Check if the descriptor is owned by CPU */
	if (desc_p->dmamac_cntl & DESC_TXSTS_OWNBYDMA) {
		printf("CPU not owner of tx frame\n");
		return -1;
	}
	memcpy((void *)desc_p->buf1_addr, (void *)packet, length);

	desc_p->buf_length |= (length << DESC_TXCTRL_SIZE1SHFT) & \
			      DESC_TXCTRL_SIZE1MASK;

	desc_p->dmamac_cntl = DESC_TXSTS_OWNBYDMA | DESC_TXCTRL_TXLAST |
			      DESC_TXCTRL_TXFIRST | length;

	/* Test the wrap-around condition. */
	if (++desc_num >= CONFIG_TX_DESCR_NUM)
		desc_num = 0;

	priv->tx_currdescnum = desc_num;

	writel((ulong)desc_p_last,
	       &dma_p->ch0_txdesc_tail_pointer);

	while (desc_p->dmamac_cntl & DESC_TXSTS_OWNBYDMA)
		;

	return 0;
}

static int dw_eth_recv(struct eth_device *dev)
{
	struct dw_eth_dev *priv = dev->priv;
	u32 status, desc_num = priv->rx_currdescnum;
	struct dmamacdescr *desc_p = &priv->rx_mac_descrtable[desc_num];
	int length = 0;

	status = desc_p->dmamac_cntl;

	/* Check if the owner is the CPU */
	if (!(status & DESC_RXSTS_OWNBYDMA)) {
		length = status & 0x7fff; /* desc_p->buf_length; */

		NetReceive(desc_p->buf1_addr, length);

		/*
		 * Make the current descriptor valid again and go to
		 * the next one
		 */
		desc_p->dmamac_cntl = DWCEQOS_DMA_RDES3_INTE |
				      DWCEQOS_DMA_RDES3_OWN |
				      DWCEQOS_DMA_RDES3_BUF1V;

		/* Test the wrap-around condition. */
		if (++desc_num >= CONFIG_RX_DESCR_NUM)
			desc_num = 0;
	}

	priv->rx_currdescnum = desc_num;

	return length;
}

static int dw_phy_init(struct eth_device *dev)
{
#ifndef CONFIG_PHY_FIXED
	struct dw_eth_dev *priv = dev->priv;
	struct phy_device *phydev;
	int mask = 0xffffffff;

#ifdef CONFIG_PHY_ADDR
	mask = 1 << CONFIG_PHY_ADDR;
#endif

	phydev = phy_find_by_mask(priv->bus, mask, priv->interface);
	if (!phydev) {
		printf("could not find PHY\n");
		return -1;
	}

	phy_connect_dev(phydev, dev);

	phydev->supported &= PHY_GBIT_FEATURES;
	phydev->advertising = phydev->supported;

	priv->phydev = phydev;
	phy_config(phydev);
#endif

	return 1;
}

int designware_initialize(ulong base_addr, u32 interface)
{
	struct eth_device *dev;
	struct dw_eth_dev *priv;

	dev = (struct eth_device *) malloc(sizeof(struct eth_device));
	if (!dev)
		return -ENOMEM;

	/*
	 * Since the priv structure contains the descriptors which need a strict
	 * buswidth alignment, memalign is used to allocate memory
	 */
	priv = (struct dw_eth_dev *) memalign(ARCH_DMA_MINALIGN,
					      sizeof(struct dw_eth_dev));
	if (!priv) {
		free(dev);
		return -ENOMEM;
	}

	memset(dev, 0, sizeof(struct eth_device));
	memset(priv, 0, sizeof(struct dw_eth_dev));

	sprintf(dev->name, "dwqosmac.%lx", base_addr);
	dev->iobase = (int)base_addr;
	dev->priv = priv;

	priv->initialized = 0;
	priv->dev = dev;
	priv->mac_regs_p = (struct eth_mac_regs *)base_addr;
	priv->mtl_regs_p = (struct eth_mtl_regs *)(base_addr +
			DW_MTL_BASE_OFFSET);
	priv->dma_regs_p = (struct eth_dma_regs *)(base_addr +
			DW_DMA_BASE_OFFSET);

	dev->init = dw_eth_init;
	dev->send = dw_eth_send;
	dev->recv = dw_eth_recv;
	dev->halt = dw_eth_halt;
	dev->write_hwaddr = dw_write_hwaddr;

	priv->interface = interface;

	mac_reset(dev);

	eth_register(dev);

	dw_mdio_init(dev->name, priv->mac_regs_p);
	priv->bus = miiphy_get_dev_by_name(dev->name);

	return dw_phy_init(dev);
}
