/*
 * (C) Copyright 2010
 * Vipin Kumar, ST Micoelectronics, vipin.kumar@st.com.
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#ifndef _DW_ETH_H
#define _DW_ETH_H

#define CONFIG_TX_DESCR_NUM	16
#define CONFIG_RX_DESCR_NUM	100
#define CONFIG_ETH_BUFSIZE	2048
#define TX_TOTAL_BUFSIZE	(CONFIG_ETH_BUFSIZE * CONFIG_TX_DESCR_NUM)
#define RX_TOTAL_BUFSIZE	(CONFIG_ETH_BUFSIZE * CONFIG_RX_DESCR_NUM)

#define CONFIG_MACRESET_TIMEOUT	(3 * CONFIG_SYS_HZ)
#define CONFIG_MDIO_TIMEOUT	(3 * CONFIG_SYS_HZ)
#define CONFIG_PHYRESET_TIMEOUT	(3 * CONFIG_SYS_HZ)
#define CONFIG_AUTONEG_TIMEOUT	(5 * CONFIG_SYS_HZ)

struct eth_mac_regs {
	u32 conf;		/* 0x00 */
	u32 extconf;		/* 0x04 */
	u32 packetfilter;	/* 0x08 */
	u32 watchdogtimeout;	/* 0x0c */
	u32 machashtablereg0;	/* 0x10 */
	u32 machashtablereg1;	/* 0x14 */
	u32 machashtablereg2;	/* 0x18 */
	u32 machashtablereg3;	/* 0x1C */
	u32 machashtablereg4;	/* 0x20 */
	u32 machashtablereg5;	/* 0x24 */
	u32 machashtablereg6;	/* 0x28 */
	u32 machashtablereg7;	/* 0x2c */
	u8 reserved[32];
	u32 vlantag;		/* 0x50 */
	u8 reserved_1[4];
	u32 vlanhashtable;	/* 0x58 */
	u8 reserved_2[4];
	u32 macvlanincl;	/* 0x60 */
	u32 macinnervlanincl;	/* 0x64 */
	u8 reserved_3[8];
	u32 macq0txflowctrl;	/* 0x70 */
	u32 macq1txflowctrl;	/* 0x74 */
	u32 macq2txflowctrl;	/* 0x78 */
	u32 macq3txflowctrl;	/* 0x7C */
	u32 macq4txflowctrl;	/* 0x80 */
	u32 macq5txflowctrl;	/* 0x84 */
	u32 macq6txflowctrl;	/* 0x88 */
	u32 macq7txflowctrl;	/* 0x8C */
	u32 macqrxflowctrl;	/* 0x90 */
	u8 reserved_4[4];
	u32 mactxqprtymap0;	/* 0x98 */
	u32 mactxqprtymap1;	/* 0x9C */
	u32 macrxqctrl0;	/* 0xa0 */
	u32 macrxqctrl1;	/* 0xa4 */
	u32 macrxqctrl2;	/* 0xa8 */
	u8 reserved_5[100];
	u32 macversion;		/* 0x110 */
	u32 macdebug;		/* 0x114 */
	u8 reserved_6[4];
	u32 machwfeature0;	/* 0x11c */
	u32 machwfeature1;	/* 0x120 */
	u32 machwfeature2;	/* 0x124 */
	u8 reserved_7[216];
	u32 miiaddr;		/* 0x200 */
	u32 miidata;		/* 0x204 */
	u8 reserved_8[248];
	u32 macaddr0hi;		/* 0x300 */
	u32 macaddr0lo;		/* 0x304 */
};

struct eth_mtl_regs { /* 0xD00 */
	u32 txq0_oper_mode;	/* 0x00 */
	u8 reserved[44];
	u32 rxq0_oper_mode;	/* 0x30 */
};

#define BIT(nr) (1UL << (nr))
#define GENMASK(h, l) \
	(((~0UL) << (l)) & (~0UL >> (BITS_PER_LONG - 1 - (h))))

/* MAC configuration register definitions */
#define FRAMEBURSTENABLE	(1 << 18)
#define MII_PORTSELECT		(1 << 15)
#define FES_100			(1 << 14)
#define DISABLERXOWN		(1 << 10)
#define FULLDPLXMODE		(1 << 13)
#define TXENABLE		(1 << 1)
#define RXENABLE		(1 << 0)

/* MII address register definitions */
#define MII_BUSY		(1 << 0)
#define MII_WRITE		(1 << 1)
#define MII_CLKRANGE_60_100M	(0)
#define MII_CLKRANGE_100_150M	(0x4)
#define MII_CLKRANGE_20_35M	(0x8)
#define MII_CLKRANGE_35_60M	(0xC)
#define MII_CLKRANGE_150_250M	(0x10)
#define MII_CLKRANGE_250_300M	(0x14)

#define MIIADDRSHIFT		(11)
#define MIIREGSHIFT		(6)
#define MII_REGMSK		(0x1F << 6)
#define MII_ADDRMSK		(0x1F << 11)


struct eth_dma_regs {
	u32 mode;			/* 0x00 */
	u32 sysbusmode;			/* 0x04 */
	u32 interruptstatus;		/* 0x08 */
	u32 debugstatus0;		/* 0x0c */
	u32 debugstatus1;		/* 0x10 */
	u32 debugstatus2;		/* 0x14 */
	u8 reserved[8];
	u32 axi4txaracecontrol;		/* 0x20 */
	u32 axi4rxawacecontrol;		/* 0x24 */
	u32 axi4txrxawaracecontrol;	/* 0x28 */
	u8 reserved_1[212];
	u32 ch0_control;		/* 0x100 */
	u32 ch0_tx_control;		/* 0x104 */
	u32 ch0_rx_control;		/* 0x108 */
	u8 reserved_2[4];
	u32 ch0_txdesc_list_haddr;	/* 0x110 */
	u32 ch0_txdesc_list_addr;	/* 0x114 */
	u32 ch0_rxdesc_list_haddr;	/* 0x118 */
	u32 ch0_rxdesc_list_addr;	/* 0x11c */
	u32 ch0_txdesc_tail_pointer;/* 0x120 */
	u8 reserved_3[4];
	u32 ch0_rxdesc_tail_pointer;/* 0x128 */
	u32 ch0_txdesc_ring_length;	/* 0x12c */
	u32 ch0_rxdesc_ring_length;	/* 0x130 */
	u32 ch0_interrupt_enable;	/* 0x134 */
	u32 ch0_rx_int_watchdog_tmr;/* 0x138 */
	u32 ch0_slot_func_ctrl_stat;/* 0x13c */
	u8 reserved_4[4];
	u32 ch0_current_app_txdesc;	/* 0x144 */
	u8 reserved_5[4];
	u32 ch0_current_app_rxdesc;	/* 0x14c */
	u32 ch0_current_app_txbuf_h;/* 0x150 */
	u32 ch0_current_app_txbuf;	/* 0x154 */
	u32 ch0_current_app_rxbuf_h;/* 0x158 */
	u32 ch0_current_app_rxbuf;	/* 0x15c */
	u32 ch0_status;		/* 0x160 */
	u8 reserved_6[8];
	u32 ch0_miss_frame_cnt;	/* 0x16c */
};


#define DW_MTL_BASE_OFFSET	(0xd00)
#define DW_DMA_BASE_OFFSET	(0x1000)

/*DMA mode register definitions */
#define DMA_TXRX_ARBITRATION	(1 << 1)
#define DMAMAC_SWR		(1 << 0)

/* Bus mode register definitions */
#define FIXEDBURST		(1 << 0)
#define PRIORXTX_41		(3 << 14)
#define PRIORXTX_31		(2 << 14)
#define PRIORXTX_21		(1 << 14)
#define PRIORXTX_11		(0 << 14)
#define BURST_4			(1 << 1)
#define BURST_8			(1 << 2)
#define BURST_16		(1 << 3)
#define BURST_32		(1 << 4)
#define BURST_64		(1 << 5)
#define RXHIGHPRIO		(1 << 1)

/* Poll demand definitions */
#define POLL_DATA		(0xFFFFFFFF)

/* Operation mode definitions */
#define STOREFORWARD		(1 << 21)
#define FLUSHTXFIFO		(1 << 20)
#define TXSTART			(1 << 13)
#define TXSECONDFRAME		(1 << 2)
#define RXSTART			(1 << 1)

/* Descriptior related definitions */
#define MAC_MAX_FRAME_SZ	(1600)

struct dmamacdescr {
	void *buf1_addr;
	void *buf2_addr;
	u32 buf_length;
	u32 dmamac_cntl;
};

/*
 * txrx_status definitions
 */

/* tx status bits definitions */
#if defined(CONFIG_DW_ALTDESCRIPTOR)

#define DESC_TXSTS_OWNBYDMA		(1 << 31)
#define DESC_TXSTS_TXINT		(1 << 30)
#define DESC_TXSTS_TXLAST		(1 << 29)
#define DESC_TXSTS_TXFIRST		(1 << 28)
#define DESC_TXSTS_TXCRCDIS		(1 << 27)

#define DESC_TXSTS_TXPADDIS		(1 << 26)
#define DESC_TXSTS_TXCHECKINSCTRL	(3 << 22)
#define DESC_TXSTS_TXRINGEND		(1 << 21)
#define DESC_TXSTS_TXCHAIN		(1 << 20)
#define DESC_TXSTS_MSK			(0x1FFFF << 0)

#else

#define DESC_TXSTS_OWNBYDMA		(1 << 31)
#define DESC_TXSTS_MSK			(0x1FFFF << 0)

#endif

/* rx status bits definitions */
#define DESC_RXSTS_OWNBYDMA		(1 << 31)
#define DESC_RXSTS_DAFILTERFAIL		(1 << 30)
#define DESC_RXSTS_FRMLENMSK		(0x3FFF << 16)
#define DESC_RXSTS_FRMLENSHFT		(16)

#define DESC_RXSTS_ERROR		(1 << 15)
#define DESC_RXSTS_RXTRUNCATED		(1 << 14)
#define DESC_RXSTS_SAFILTERFAIL		(1 << 13)
#define DESC_RXSTS_RXIPC_GIANTFRAME	(1 << 12)
#define DESC_RXSTS_RXDAMAGED		(1 << 11)
#define DESC_RXSTS_RXVLANTAG		(1 << 10)
#define DESC_RXSTS_RXFIRST		(1 << 9)
#define DESC_RXSTS_RXLAST		(1 << 8)
#define DESC_RXSTS_RXIPC_GIANT		(1 << 7)
#define DESC_RXSTS_RXCOLLISION		(1 << 6)
#define DESC_RXSTS_RXFRAMEETHER		(1 << 5)
#define DESC_RXSTS_RXWATCHDOG		(1 << 4)
#define DESC_RXSTS_RXMIIERROR		(1 << 3)
#define DESC_RXSTS_RXDRIBBLING		(1 << 2)
#define DESC_RXSTS_RXCRC		(1 << 1)

/*
 * dmamac_cntl definitions
 */

/* tx control bits definitions */

#define DESC_TXCTRL_SIZE1MASK		(0x3FFF << 0)
#define DESC_TXCTRL_SIZE1SHFT		(0)
#define DESC_TXCTRL_SIZE2MASK		(0x3FFF << 16)
#define DESC_TXCTRL_SIZE2SHFT		(16)


#define DESC_TXCTRL_TXINT		(1 << 31)
#define DESC_TXCTRL_TXFIRST		(1 << 29)
#define DESC_TXCTRL_TXLAST		(1 << 28)
#define DESC_TXCTRL_TXCHECKINSCTRL	(3 << 27)
#define DESC_TXCTRL_TXCRCDIS		(1 << 26)
#define DESC_TXCTRL_TXRINGEND		(1 << 25)


/* rx control bits definitions */

#define DESC_RXCTRL_SIZE1MASK		(0x3FFF << 0)
#define DESC_RXCTRL_SIZE1SHFT		(0)
#define DESC_RXCTRL_SIZE2MASK		(0x3FFF << 16)
#define DESC_RXCTRL_SIZE2SHFT		(16)


struct dw_eth_dev {
	struct dmamacdescr tx_mac_descrtable[CONFIG_TX_DESCR_NUM];
	struct dmamacdescr rx_mac_descrtable[CONFIG_RX_DESCR_NUM];
	char txbuffs[TX_TOTAL_BUFSIZE];
	char rxbuffs[RX_TOTAL_BUFSIZE];

	u32 initialized;
	u32 interface;
	u32 tx_currdescnum;
	u32 rx_currdescnum;

	struct eth_mac_regs *mac_regs_p;
	struct eth_mtl_regs *mtl_regs_p;
	struct eth_dma_regs *dma_regs_p;

	struct eth_device *dev;
	struct phy_device *phydev;
	struct mii_dev *bus;
} __attribute__ ((aligned(8)));

/* Speed specific definitions */
#define SPEED_10M		1
#define SPEED_100M		2
#define SPEED_1000M		3

/* Duplex mode specific definitions */
#define DWCEQOS_HALF_DUPLEX		1
#define DWCEQOS_FULL_DUPLEX		2

#define DMA_SYSBUS_MODE                  0x1004

/* DMA channel registers */
#define DMA_CH0_TX_CONTROL               0x1104
#define DMA_CH0_RX_CONTROL               0x1108
#define DMA_CH0_TXDESC_LIST_ADDRESS      0x1114
#define DMA_CH0_RXDESC_LIST_ADDRESS      0x111c
#define DMA_CH0_TXDESC_TAIL_POINTER      0x1120
#define DMA_CH0_RXDESC_TAIL_POINTER      0x1128
#define DMA_CH0_TXDESC_RING_LENGTH       0x112c
#define DMA_CH0_RXDESC_RING_LENGTH       0x1130
#define MAC_RXQ_CTRL0                    0x00a0
#define MAC_RXQ_CTRL1                    0x00a4
#define MAC_RXQ_CTRL2                    0x00a8

/* MTL */
#define MTL_TXQ0_OPERATION_MODE          0x0d00
#define MTL_RXQ0_OPERATION_MODE          0x0d30
/* MTL */
#define REG_DWCEQOS_MTL_OPER             0x0c00
#define REG_DWCEQOS_MTL_DEBUG_ST         0x0c0c
#define REG_DWCEQOS_MTL_TXQ0_DEBUG_ST    0x0d08
#define REG_DWCEQOS_MTL_RXQ0_DEBUG_ST    0x0d38

#define DWCEQOS_MTL_TXQ_TXQEN            BIT(3)
#define DWCEQOS_MTL_TXQ_TSF              BIT(1)
#define DWCEQOS_MTL_TXQ_FTQ              BIT(0)
#define DWCEQOS_MTL_TXQ_TTC512           0x00000070

#define DWCEQOS_MTL_TXQ_SIZE(x)          ((((x) - 256) & 0xff00) << 8)
#define DWCEQOS_MTL_RXQ_SIZE(x)          ((((x) - 256) & 0xff00) << 12)

/* MAC */
#define MAC_CONFIGURATION                0x0000
#define MAC_EXT_CONFIGURATION            0x0004
#define MAC_PACKET_FILTER                0x0008
#define MAC_MDIO_ADDRESS                 0x0200
#define MAC_MDIO_DATA                    0x0204
#define MAC_RXQ_CTRL0                    0x00a0
#define MAC_RXQ_CTRL1                    0x00a4
#define MAC_RXQ_CTRL2                    0x00a8

#define DWCEQOS_MAC_CFG_ACS              BIT(20)
#define DWCEQOS_MAC_CFG_JD               BIT(17)
#define DWCEQOS_MAC_CFG_JE               BIT(16)
#define DWCEQOS_MAC_CFG_PS               BIT(15)
#define DWCEQOS_MAC_CFG_FES              BIT(14)
#define DWCEQOS_MAC_CFG_DM               BIT(13)
#define DWCEQOS_MAC_CFG_DO               BIT(10)
#define DWCEQOS_MAC_CFG_TE               BIT(1)
#define DWCEQOS_MAC_CFG_IPC              BIT(27)
#define DWCEQOS_MAC_CFG_RE               BIT(0)

#define DWCEQOS_MAC_PKT_FILT_RA          BIT(31)
#define DWCEQOS_MAC_PKT_FILT_HPF         BIT(10)
#define DWCEQOS_MAC_PKT_FILT_SAF         BIT(9)
#define DWCEQOS_MAC_PKT_FILT_SAIF        BIT(8)
#define DWCEQOS_MAC_PKT_FILT_DBF         BIT(5)
#define DWCEQOS_MAC_PKT_FILT_PM          BIT(4)
#define DWCEQOS_MAC_PKT_FILT_DAIF        BIT(3)
#define DWCEQOS_MAC_PKT_FILT_HMC         BIT(2)
#define DWCEQOS_MAC_PKT_FILT_HUC         BIT(1)
#define DWCEQOS_MAC_PKT_FILT_PR          BIT(0)

/* DMA */
#define REG_DWCEQOS_DMA_MODE             0x1000
#define REG_DWCEQOS_DMA_SYSBUS_MODE      0x1004
#define REG_DWCEQOS_DMA_IS               0x1008
#define REG_DWCEQOS_DMA_DEBUG_ST0        0x100c

/* DMA channel registers */
#define REG_DWCEQOS_DMA_CH0_CTRL         0x1100
#define REG_DWCEQOS_DMA_CH0_TX_CTRL      0x1104
#define REG_DWCEQOS_DMA_CH0_RX_CTRL      0x1108
#define REG_DWCEQOS_DMA_CH0_TXDESC_LIST  0x1114
#define REG_DWCEQOS_DMA_CH0_RXDESC_LIST  0x111c
#define REG_DWCEQOS_DMA_CH0_TXDESC_TAIL  0x1120
#define REG_DWCEQOS_DMA_CH0_RXDESC_TAIL  0x1128
#define REG_DWCEQOS_DMA_CH0_TXDESC_LEN   0x112c
#define REG_DWCEQOS_DMA_CH0_RXDESC_LEN   0x1130
#define REG_DWCEQOS_DMA_CH0_IE           0x1134
#define REG_DWCEQOS_DMA_CH0_CUR_TXDESC   0x1144
#define REG_DWCEQOS_DMA_CH0_CUR_RXDESC   0x114c
#define REG_DWCEQOS_DMA_CH0_CUR_TXBUF    0x1154
#define REG_DWCEQOS_DMA_CH0_CUR_RXBUG    0x115c
#define REG_DWCEQOS_DMA_CH0_STA          0x1160

#define DWCEQOS_DMA_MODE_TXPR            BIT(11)
#define DWCEQOS_DMA_MODE_DA              BIT(1)

#define DWCEQOS_DMA_SYSBUS_MODE_EN_LPI   BIT(31)
#define DWCEQOS_DMA_SYSBUS_MODE_FB       BIT(0)
#define DWCEQOS_DMA_SYSBUS_MODE_AAL      BIT(12)

#define DWCEQOS_DMA_SYSBUS_MODE_RD_OSR_LIMIT(x) \
	(((x) << 16) & 0x000F0000)
#define DWCEQOS_DMA_SYSBUS_MODE_RD_OSR_LIMIT_DEFAULT    3
#define DWCEQOS_DMA_SYSBUS_MODE_RD_OSR_LIMIT_MASK       GENMASK(19, 16)

#define DWCEQOS_DMA_SYSBUS_MODE_WR_OSR_LIMIT(x) \
	(((x) << 24) & 0x0F000000)
#define DWCEQOS_DMA_SYSBUS_MODE_WR_OSR_LIMIT_DEFAULT    3
#define DWCEQOS_DMA_SYSBUS_MODE_WR_OSR_LIMIT_MASK       GENMASK(27, 24)

#define DWCEQOS_DMA_SYSBUS_MODE_BURST_MASK GENMASK(7, 1)
#define DWCEQOS_DMA_SYSBUS_MODE_BURST(x) \
	(((x) << 1) & DWCEQOS_DMA_SYSBUS_MODE_BURST_MASK)
#define DWCEQOS_DMA_SYSBUS_MODE_BURST_DEFAULT   GENMASK(3, 1)

#define DWCEQOS_DMA_CH_CTRL_PBLX8       BIT(16)
#define DWCEQOS_DMA_CH_CTRL_DSL(x)      ((x) << 18)

#define DWCEQOS_DWCEQOS_RX_BUF_SIZE 1536
/* 2048 */

#define DWCEQOS_DMA_CH_CTRL_PBL(x)       ((x) << 16)
#define DWCEQOS_DMA_CH_CTRL_START        BIT(0)
#define DWCEQOS_DMA_CH_RX_CTRL_BUFSIZE(x) ((x) << 1)
#define DWCEQOS_DMA_CH_TX_OSP            BIT(4)
#define DWCEQOS_DMA_CH_TX_TSE            BIT(12)

#define DWCEQOS_DMA_CH0_IE_NIE           BIT(15)
#define DWCEQOS_DMA_CH0_IE_AIE           BIT(14)
#define DWCEQOS_DMA_CH0_IE_RIE           BIT(6)
#define DWCEQOS_DMA_CH0_IE_TIE           BIT(0)
#define DWCEQOS_DMA_CH0_IE_FBEE          BIT(12)
#define DWCEQOS_DMA_CH0_IE_RBUE          BIT(7)

#define DWCEQOS_DMA_IS_DC0IS             BIT(0)
#define DWCEQOS_DMA_IS_MTLIS             BIT(16)
#define DWCEQOS_DMA_IS_MACIS             BIT(17)

#define DWCEQOS_DMA_CH0_IS_TI            BIT(0)
#define DWCEQOS_DMA_CH0_IS_RI            BIT(6)
#define DWCEQOS_DMA_CH0_IS_RBU           BIT(7)
#define DWCEQOS_DMA_CH0_IS_FBE           BIT(12)
#define DWCEQOS_DMA_CH0_IS_CDE           BIT(13)
#define DWCEQOS_DMA_CH0_IS_AIS           BIT(14)

#define DWCEQOS_DMA_CH0_IS_TEB           GENMASK(18, 16)
#define DWCEQOS_DMA_CH0_IS_TX_ERR_READ   BIT(16)
#define DWCEQOS_DMA_CH0_IS_TX_ERR_DESCR  BIT(17)

#define DWCEQOS_DMA_CH0_IS_REB           GENMASK(21, 19)
#define DWCEQOS_DMA_CH0_IS_RX_ERR_READ   BIT(19)
#define DWCEQOS_DMA_CH0_IS_RX_ERR_DESCR  BIT(20)

/* DMA descriptor bits for RX normal descriptor (read format) */
#define DWCEQOS_DMA_RDES3_OWN     BIT(31)
#define DWCEQOS_DMA_RDES3_INTE    BIT(30)
#define DWCEQOS_DMA_RDES3_BUF2V   BIT(25)
#define DWCEQOS_DMA_RDES3_BUF1V   BIT(24)

/* DMA descriptor bits for RX normal descriptor (write back format) */
#define DWCEQOS_DMA_RDES1_IPCE    BIT(7)
#define DWCEQOS_DMA_RDES3_ES      BIT(15)
#define DWCEQOS_DMA_RDES3_E_JT    BIT(14)
#define DWCEQOS_DMA_RDES3_PL(x)   ((x) & 0x7fff)
#define DWCEQOS_DMA_RDES1_PT      0x00000007
#define DWCEQOS_DMA_RDES1_PT_UDP  BIT(0)
#define DWCEQOS_DMA_RDES1_PT_TCP  BIT(1)
#define DWCEQOS_DMA_RDES1_PT_ICMP 0x00000003

/* DMA descriptor bits for TX normal descriptor (read format) */
#define DWCEQOS_DMA_TDES2_IOC     BIT(31)
#define DWCEQOS_DMA_TDES3_OWN     BIT(31)
#define DWCEQOS_DMA_TDES3_CTXT    BIT(30)
#define DWCEQOS_DMA_TDES3_FD      BIT(29)
#define DWCEQOS_DMA_TDES3_LD      BIT(28)
#define DWCEQOS_DMA_TDES3_CIPH    BIT(16)
#define DWCEQOS_DMA_TDES3_CIPP    BIT(17)
#define DWCEQOS_DMA_TDES3_CA      0x00030000
#define DWCEQOS_DMA_TDES3_TSE     BIT(18)
#define DWCEQOS_DMA_DES3_THL(x)   ((x) << 19)
#define DWCEQOS_DMA_DES2_B2L(x)   ((x) << 16)

#define DWCEQOS_DMA_TDES3_TCMSSV    BIT(26)

/* DMA channel states */
#define DMA_TX_CH_STOPPED   0
#define DMA_TX_CH_SUSPENDED 6

#define DMA_GET_TX_STATE_CH0(status0) ((status0 & 0xF000) >> 12)

#endif
