/*
* Copyright c                  Realtek Semiconductor Corporation, 2006
* All rights reserved.
*
* Program : Control smi connected RTL8366
* Abstract :
* Author : Yu-Mei Pan (ympan@realtek.com.cn)
*  $Id: smi.c,v 1.2 2008-04-10 03:04:19 shiehyy Exp $
*/
#include "rtk_types.h"
#include "rtk_error.h"
//#include <stdio.h>
#include <linux/types.h>

#define MDC_MDIO_CTRL0_REG          31
#define MDC_MDIO_START_REG          29
#define MDC_MDIO_CTRL1_REG          21
#define MDC_MDIO_ADDRESS_REG        23
#define MDC_MDIO_DATA_WRITE_REG     24
#define MDC_MDIO_DATA_READ_REG      25
#define MDC_MDIO_PREAMBLE_LEN       32

#define MDC_MDIO_START_OP          0xFFFF
#define MDC_MDIO_ADDR_OP           0x000E
#define MDC_MDIO_READ_OP           0x0001
#define MDC_MDIO_WRITE_OP          0x0003

struct mii_dev *g_mbus = NULL;

extern int dw_mdio_read(struct mii_dev *bus, int phyaddr, int devad, int phyreg);
extern int dw_mdio_write(struct mii_dev *bus, int phyaddr, int devad, int phyreg, u16 phydata);

rtk_int32 smi_read(rtk_uint32 mAddrs, rtk_uint32 *rData);
rtk_int32 smi_write(rtk_uint32 mAddrs, rtk_uint32 rData);

/*******************************************************************************/
/*  MDC/MDIO porting                                                           */
/*******************************************************************************/
/* define the PHY ID currently used */
#define MDC_MDIO_PHY_ID     29  /* PHY ID 0 or 29 */

/* MDC/MDIO, redefine/implement the following Macro */
#define MDC_MDIO_WRITE(preamableLength, phyID, regID, data)
#define MDC_MDIO_READ(preamableLength, phyID, regID, pData)

static void rtlglue_drvMutexLock(void)
{
	/* It is empty currently. Implement this function if Lock/Unlock function is needed */
	return;
}

static void rtlglue_drvMutexUnlock(void)
{
	/* It is empty currently. Implement this function if Lock/Unlock function is needed */
	return;
}

int rtl_init_mii_dev(struct mii_dev *bus)
{
	g_mbus = bus;
	return 0;
}

rtk_int32 smi_read(rtk_uint32 mAddrs, rtk_uint32 *rData)
{
	if (mAddrs > 0xFFFF)
		return RT_ERR_INPUT;

	if (rData == NULL)
		return RT_ERR_NULL_POINTER;

	if (!g_mbus) {
		printf("smi_read RT_ERR_NULL_POINTER\n");
		return RT_ERR_NULL_POINTER;
	}

	/* Lock */
	rtlglue_drvMutexLock();

	/* Write address control code to register 31 */
	dw_mdio_write(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_CTRL0_REG, MDC_MDIO_ADDR_OP);

	/* Write address to register 23 */
	dw_mdio_write(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_ADDRESS_REG, mAddrs);

	/* Write read control code to register 21 */
	dw_mdio_write(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_CTRL1_REG, MDC_MDIO_READ_OP);

	/* Read data from register 25 */
	*rData = dw_mdio_read(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_DATA_READ_REG);

	/* Unlock */
	rtlglue_drvMutexUnlock();

	return RT_ERR_OK;
}

rtk_int32 smi_write(rtk_uint32 mAddrs, rtk_uint32 rData)
{
	if (mAddrs > 0xFFFF)
		return RT_ERR_INPUT;

	if (rData > 0xFFFF)
		return RT_ERR_INPUT;

	if (!g_mbus) {
		printf("smi_write RT_ERR_NULL_POINTER\n");
		return RT_ERR_NULL_POINTER;
	}

	/* Lock */
	rtlglue_drvMutexLock();

	/* Write address control code to register 31 */
	dw_mdio_write(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_CTRL0_REG, MDC_MDIO_ADDR_OP);

	/* Write address to register 23 */
	dw_mdio_write(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_ADDRESS_REG, mAddrs);

	/* Write data to register 24 */
	dw_mdio_write(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_DATA_WRITE_REG, rData);

	/* Write data control code to register 21 */
	dw_mdio_write(g_mbus, MDC_MDIO_PHY_ID, 0, MDC_MDIO_CTRL1_REG, MDC_MDIO_WRITE_OP);

	/* Unlock */
	rtlglue_drvMutexUnlock();

	return RT_ERR_OK;
}
