#ifndef __CONFIG_DVF101_H
#define __CONFIG_DVF101_H

#include <asm/arch/gpio.h>
#include <linux/sizes.h>

#define CONFIG_OF_LIBFDT
#define CONFIG_OF_BOARD_SETUP

#ifdef CONFIG_NAND
    #define MTDIDS_DEFAULT		"nand0=dmw_nand"

    #define NAND_SLICED_UPDATE                                  \
        "nandfile=nand.image.slice\0"                           \
        "last_nand_offset=0\0"                                  \
        "nandfileloadcmd=tftp\0"                                \
        "nand_sliced_update="                                   \
          "if test \"0\" -ne $last_nand_offset;"                \
          " then "                                              \
             "$erasecmd $last_nand_offset;"                     \
          " else "                                              \
             "nand erase.chip;"                                 \
          " fi; "                                               \
          "setenv slice 0;"                                     \
          "while $nandfileloadcmd $nandfile$slice;"             \
          " do "                                                \
             "$writecmd $loadaddr $last_nand_offset $filesize;" \
             "setexpr slice $slice + 1;"                        \
          " done\0"

    #ifdef CONFIG_SOFTWARE_UPGRADE_UBOOT
        #define MTDPARTS_DEFAULT_UBOOT \
            "2m(u-boot-1),"  \
            "2m(u-boot-2),"
    #else
        #define MTDPARTS_DEFAULT_UBOOT ""
    #endif
    #ifdef CONFIG_SOFTWARE_UPGRADE_ROOTFS
        #define MTDPARTS_DEFAULT_ROOTFS \
            "60m(rootfs),"   \
            "60m(rootfs-2)"
    #else
        #define MTDPARTS_DEFAULT_ROOTFS \
            "-(rootfs)"
    #endif
    #define MTDPARTS_DEFAULT   \
        "mtdparts=dmw_nand:"   \
        "2m(u-boot),"          \
        "1m(u-boot-env),"      \
        MTDPARTS_DEFAULT_UBOOT \
        MTDPARTS_DEFAULT_ROOTFS

    #ifdef CONFIG_JFFS2_NAND
        #define KERNEL_BOOTARGS \
           "root=mtd:rootfs rootfstype=jffs2\0"
    #else
        #define KERNEL_BOOTARGS \
           "ubi.mtd=rootfs root=ubi0:rootfs rootfstype=ubifs rw\0"
    #endif

    #define FLASH_ENV \
        "erasecmd=nand erase.part\0" \
        "readcmd=nand read\0"        \
        "writecmd=nand write\0"

    #ifdef CONFIG_JFFS2_NAND
        #define KERNEL_BOOTCMD \
            "run chk_parts; chpart ${rootfspart}; "                           \
            "fsload ${loadaddr} /boot/uImage && "                             \
            "fsload ${devtree_addr} boot/${devicetree} && run bootm\0"
    #else
        #define KERNEL_BOOTCMD \
            "run chk_parts; ubi part ${rootfspart} && ubifsmount ubi0:rootfs && "  \
            "ubifsload ${loadaddr} boot/uImage && "                           \
            "ubifsload ${devtree_addr} boot/${devicetree} && run bootm\0"
    #endif

    #ifdef CONFIG_JFFS2_NAND
        #define MANIFEST_SYSTEM \
            "dfu_manifest_system=${erasecmd} ${rootfspart} "                  \
                    "&& ${writecmd} ${loadaddr} ${rootfspart} ${filesize} "   \
                    "; reset\0"                                               \
            "mtdparts="MTDPARTS_DEFAULT"\0"
    #else
        #define MANIFEST_SYSTEM \
            "dfu_manifest_system=${erasecmd} ${rootfspart} "                  \
                    "&& ubi part ${rootfspart} "                              \
                    "&& ubi create rootfs "                                   \
                    "&& ubi write ${loadaddr} rootfs ${filesize} "            \
                    "; reset\0"                                               \
            "mtdparts="MTDPARTS_DEFAULT"\0"
    #endif
#elif defined(CONFIG_EMMC)
	#define CONFIG_CMD_DISK

	#ifndef CONFIG_EMMC_CARDNUM
		#define CONFIG_EMMC_CARDNUM	"0"
	#endif

	#define KERNEL_BOOTARGS						\
		"rootfstype=ext4 root=/dev/mmcblk0p3 rootwait\0"

	#define FLASH_ENV						\
		"readcmd=disk read\0"					\
		"writecmd=disk write\0"					\
		"switchcmd=mmc dev "CONFIG_EMMC_CARDNUM"\0"

	#define KERNEL_BOOTCMD						\
		"mmc set-phases "CONFIG_EMMC_CARDNUM" && "		\
		"${switchcmd} 0 && "					\
		"${readcmd} ${loadaddr} "				\
		"mmc "CONFIG_EMMC_CARDNUM":1 && "			\
		"${readcmd} ${devtree_addr} "				\
		"mmc "CONFIG_EMMC_CARDNUM":2 && run bootm\0"

	#define MANIFEST_SYSTEM						\
		"defparts=gptpart dev mmc 0 1 && gptpart del all && "	\
			"gptpart add data uboot 2M && "			\
			"gptpart add env uboot-env 2M && "		\
			"gptpart save && "				\
			"gptpart dev mmc 0 0 && gptpart del all && "	\
			"gptpart add data kernel 4M && "		\
			"gptpart add data devtree 1M && "		\
			"gptpart add data rootfs && "			\
			"gptpart save\0"				\
		"update_kernel=${switchcmd} 0 && "			\
			"${writecmd} ${loadaddr} "			\
			"mmc "CONFIG_EMMC_CARDNUM" 0:1 ${filesize}\0"	\
		"update_devtree=${switchcmd} 0 && "			\
			"${writecmd} ${loadaddr} "			\
			"mmc "CONFIG_EMMC_CARDNUM" 0:2 ${filesize}\0"	\
		"update_rootfs=${switchcmd} 0 && "			\
			"${writecmd} ${loadaddr} "			\
			"mmc "CONFIG_EMMC_CARDNUM" 0:3 ${filesize}\0"	\
		"update_uboot=${switchcmd} 1 && "			\
			"${writecmd} ${loadaddr} "			\
			"mmc "CONFIG_EMMC_CARDNUM" 1:1 ${filesize}\0"	\
		"update_ubootenv=${switchcmd} 1 && "			\
			"${writecmd} ${loadaddr} "			\
			"mmc "CONFIG_EMMC_CARDNUM" 1:2 ${filesize}\0"

	#define NAND_SLICED_UPDATE ""
	#define CONFIG_SYS_NO_FLASH
	#define CONFIG_SYS_MAX_FLASH_SECT		1
#else
    #define CONFIG_SYS_NO_FLASH
    #define CONFIG_SYS_MAX_FLASH_SECT		1
    #define CONFIG_ENV_IS_NOWHERE
    #define CONFIG_ENV_SIZE			(256 << 10) /* 256k */
    #define FLASH_ENV
    #define KERNEL_BOOTCMD "run bootm\0"
    #define KERNEL_BOOTARGS
    #define MANIFEST_SYSTEM
    #define NAND_SLICED_UPDATE
#endif

#define UPGRADE_ENV "ubootpart=u-boot\0"

#define BOOTMEDIUM_ENV_SETTINGS \
	FLASH_ENV                                                         \
	UPGRADE_ENV                                                       \
	"rootfspart=rootfs\0"                                             \
	"linux_boot=" KERNEL_BOOTCMD                                      \
	"linux_boot_flash=" KERNEL_BOOTCMD                                \
	"linux_bootargs=" KERNEL_BOOTARGS                                 \
	"linux_bootargs_flash=" KERNEL_BOOTARGS                           \
	"setubimode=setenv linux_boot ${linux_boot_flash} "               \
		"&& setenv linux_bootargs ${linux_bootargs_flash} "       \
		"&& save\0"                                               \
	"chk_parts=test -n ${mtdparts} || mtdparts default\0"             \
	"dfu_manifest_2=run chk_parts; run dfu_manifest_u-boot-env\0"     \
	"dfu_manifest_u-boot=run u-boot-upgrade\0"                        \
	"dfu_manifest_u-boot-env=${erasecmd} u-boot-env "                 \
		"&& ${writecmd} ${loadaddr} u-boot-env\0"                 \
	MANIFEST_SYSTEM                                                   \
	"u-boot-upgrade=run chk_parts; "                                  \
		"${erasecmd} ${ubootpart} "                               \
		"&& ${writecmd} ${loadaddr} ${ubootpart} "                \
		"&& save "                                                \
		"&& echo Done. Please do a power cycle to reset board\0"  \
	"bgvalue=100\0"                                                   \
	NAND_SLICED_UPDATE

#define CONFIG_ENV_OVERWRITE
#ifdef CONFIG_NAND
    #define CONFIG_NAND_DMW96
    #define CONFIG_LZO    /* needed for UBI */
    #define CONFIG_RBTREE /* needed for UBI */

    #define CONFIG_SYS_NO_FLASH
    #define CONFIG_SYS_NAND_BASE		0x05400000
    #define CONFIG_SYS_MAX_NAND_DEVICE		1
    #define CONFIG_SYS_NAND_ONFI_DETECTION
    #define CONFIG_SYS_NAND_SELF_INIT
    #define NAND_MAX_CHIPS			1 /* needed for u-boot bbt */

    #define CONFIG_NAND_DMW96_REV1
    #define CONFIG_CMD_NAND
    #define CONFIG_CMD_JFFS2
    #define CONFIG_JFFS2_NAND
    #define CONFIG_CMD_UBI
    #define CONFIG_CMD_UBIFS
    #define CONFIG_CMD_MTDPROTECT

    /* Environment settings */
    #define CONFIG_ENV_IS_IN_NAND
    #define CONFIG_ENV_OFFSET			(  2 << 20) /* 2M */
    #define CONFIG_ENV_SIZE			(256 << 10) /* 256k */
    #define CONFIG_ENV_RANGE			(  1 << 20) /* 1M */
#endif

/*
 * Physical Memory Map
 */
#ifndef CONFIG_NR_DRAM_BANKS
#define CONFIG_NR_DRAM_BANKS		1
#endif
#ifndef PHYS_SDRAM_1
#define PHYS_SDRAM_1			0x40000000
#define CONFIG_SYS_TEXT_BASE		0x41000000
#define CONFIG_SYS_LOAD_ADDR		0x42000000
#endif
#define CONFIG_SYS_SDRAM_BASE		PHYS_SDRAM_1

#ifndef CONFIG_SYS_DEVTREE_OFFSET
#define CONFIG_SYS_DEVTREE_OFFSET	"0x43000000"
#endif
#define SYS_DEVTREE_OFFSET		"devtree_addr=" CONFIG_SYS_DEVTREE_OFFSET "\0"
#ifndef CONFIG_SYS_DEVICETREE
#define CONFIG_SYS_DEVICETREE		"dvf101-evb.dtb"
#endif
#define SYS_DEVICETREE			"devicetree=" CONFIG_SYS_DEVICETREE "\0"

#ifndef CONFIG_SYS_INITRD_OFFSET
#define CONFIG_SYS_INITRD_OFFSET	"0x44000000"
#endif
#define SYS_INITRD_OFFSET		"initrd_addr=" CONFIG_SYS_INITRD_OFFSET "\0"
#ifndef CONFIG_SYS_INITRD
#define CONFIG_SYS_INITRD		"rootfs.cpio.uboot"
#endif
#define SYS_INITRD			"initrd=" CONFIG_SYS_INITRD "\0"

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN		(2 * SZ_1M)

/*
 * Stack sizes - set up in start.S using sizes below
 */
#define CONFIG_SYS_INIT_SP_ADDR		(CONFIG_SYS_TEXT_BASE - SZ_4K - \
                                         GENERATED_GBL_DATA_SIZE)

#define CONFIG_SYS_DCACHE_OFF
#define CONFIG_SYS_DCACHE_INVALIDATE

#define CONFIG_ARCH_CPU_INIT
#define CONFIG_BOARD_EARLY_INIT_F
#define CONFIG_BOARD_LATE_INIT
#define CFG_PBSIZE			200

#define CONFIG_ARMCORTEXA9
#define CONFIG_HAVE_CCU

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO
#define CONFIG_VERSION_VARIABLE

#define CONFIG_SYS_ALT_MEMTEST
#define CONFIG_SYS_MEMTEST_START	0x41000000
#define CONFIG_SYS_MEMTEST_END		0x410fffff
#define CONFIG_SYS_MEMTEST_SCRATCH	0x40000000

#define CONFIG_INITRD_TAG		1
#define CONFIG_CMDLINE_TAG /* pass cmdline  */
#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_OF_LIBFDT

#define CONFIG_DVF101_RAM_SIZE_OVERWRITE (512 * 1024 * 1024)

#define CONFIG_SYS_HZ			1000

/*
 * Hardware drivers
 */
#ifdef CONFIG_NETWORKING
    #define CONFIG_PHYLIB
    #define CONFIG_CMD_MII
    #define CONFIG_DESIGNWARE_ETH_QOS
    #define CONFIG_RANDOM_MACADDR
#endif

/*
 * NS16550 Configuration
 */
#define CFG_DSPG_DMW_SERIAL

#define CONFIG_SERIAL_MULTI

#define CONFIG_CMDLINE_EDITING		1
#define CONFIG_AUTO_COMPLETE		1
#define CONFIG_MX_CYCLIC		1

#define CONFIG_CONS_INDEX		0

#define CONFIG_BAUDRATE			115200
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define CONFIG_BOOTP_MASK		CONFIG_BOOTP_DEFAULT

#define CONFIG_SYS_HUSH_PARSER		1
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "

#include <config_cmd_default.h>
#define CONFIG_CMD_IMI
#define CONFIG_CMD_BDI
#define CONFIG_CMD_MEMORY
#define CONFIG_CMD_ENV
#define CONFIG_CMD_RUN
#define CONFIG_CMD_ECHO
#define CONFIG_CMD_ITEST
#define CONFIG_CMD_REV
#define CONFIG_CMD_GREPENV
#define CONFIG_CMD_DEFENV
#define CONFIG_CMD_MD5SUM
#define CONFIG_MD5
#define CONFIG_CMD_SETEXPR
#ifdef CONFIG_NETWORKING
    #define CONFIG_CMD_NET
    #define CONFIG_CMD_DHCP
    #define CONFIG_CMD_PING
#endif
#if defined(CONFIG_MMC) && defined(CONFIG_EMMC)
    #error "Do not define MMC and EMMC at the same time"
#endif
#ifdef CONFIG_MMC
    #define CONFIG_DOS_PARTITION
    #define CONFIG_GENERIC_MMC
    #define CONFIG_SYNDWC_MMC
    #define CONFIG_CMD_MMC
    #define CONFIG_CMD_FAT
    #define CONFIG_SYS_SYNDWC_BASE CONFIG_SD_MMC2_BASE
#endif
#ifdef CONFIG_EMMC
    #define CONFIG_SUPPORT_EMMC_BOOT
    #define CONFIG_DOS_PARTITION
    #define CONFIG_GENERIC_MMC
    #define CONFIG_SYNDWC_MMC
    #define HAVE_BLOCK_DEVICE
    #define CONFIG_CMD_MMC
    #define CONFIG_SYS_SYNDWC_BASE CONFIG_SD_MMC1_BASE
    #define CONFIG_ENV_IS_IN_MMC		1
    #define CONFIG_SYS_MMC_ENV_DEV		0
    #define CONFIG_ENV_SIZE			0x40000 /* 256k */
#endif

#define CONFIG_BOOTDELAY		4
#define CONFIG_BOOTABORTKEY		0x1b /* ascii for escape */
#define CONFIG_TFTP_PORT		1

#define CONFIG_NET_RETRY_COUNT		7

#define CONFIG_BOOTCOMMAND \
	"if mmc rescan; then " \
		"if run loadbootscript; then " \
			"echo \"Boot script found on SD card. Do 'run bootscript' to execute\"; " \
			/* "run bootscript; " */ \
		"else " \
			"run linux_boot; " \
		"fi; " \
	"else run linux_boot; fi"

#ifdef DW_USBTTY_AS_DEFAULT
#define USBTTY_ENV_SETTINGS \
	"stdin=usbtty\0"  \
	"stdout=usbtty\0" \
	"stderr=usbtty\0"
#elif defined(CONFIG_ARM_DCC)
#define USBTTY_ENV_SETTINGS \
	"stdin=dcc\0"  \
	"stdout=dcc\0" \
	"stderr=dcc\0"
#else
#define USBTTY_ENV_SETTINGS
#endif

#define CONFIG_EXTRA_ENV_SETTINGS \
	SYS_DEVTREE_OFFSET                                                \
	SYS_DEVICETREE                                                    \
	SYS_INITRD_OFFSET                                                 \
	SYS_INITRD                                                        \
	"ethaddr=01:02:03:04:05:06\0"                                     \
	"ipaddr=172.19.35.74\0"                                           \
	"serverip=172.19.35.75\0"                                         \
	"loadbootscript=fatload mmc 0 ${loadaddr} boot.scr\0"             \
	"bootscript=echo Running bootscript from mmc ...; "               \
		"source ${loadaddr}\0"                                    \
	"dfu_manifest=run bootm\0"                                        \
	"console=console=ttyS0\0"                                         \
	"bootm=setenv bootargs ${console} ${linux_bootargs} ${mtdparts} " \
		"&& bootm ${loadaddr} - ${devtree_addr}\0"                \
	"linux_boot_nfs=tftp ${loadaddr} uImage "                         \
		"&& tftp ${devtree_addr} ${devicetree} "                  \
		"&& run bootm\0"                                          \
	"linux_bootargs_nfs=root=/dev/nfs ip=dhcp "                       \
		"nfsroot=192.168.0.1:/\0"                                 \
	"setnfsmode=setenv linux_boot ${linux_boot_nfs} "                 \
		"&& setenv linux_bootargs ${linux_bootargs_nfs} "         \
		"&& save\0"                                               \
	"bootram=setenv bootargs ${console} $linux_bootargs} ${mtdparts} "\
		"&& bootm ${loadaddr} ${initrd_addr} ${devtree_addr}\0"   \
	BOOTMEDIUM_ENV_SETTINGS                                           \
	USBTTY_ENV_SETTINGS

/*
 * Static configuration when assigning fixed address
 */
#define CONFIG_BOOTFILE			"uImage"
#define CONFIG_LOADADDR			CONFIG_SYS_LOAD_ADDR

#ifdef CONFIG_NAND
	/*
	 * Flash configuration
	 */
	#define CONFIG_CMD_MTDPARTS
	#define CONFIG_MTD_DEVICE
	#define CONFIG_MTD_PARTITIONS
#elif defined(CONFIG_EMMC)
	#define CONFIG_DOS_PARTITION
	#define CONFIG_EFI_PARTITION
	#define CONFIG_CMD_DISK
	#define CONFIG_CMD_GPTPART
	#define CONFIG_CMD_EXT4
	#define CONFIG_FS_EXT4
#endif

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP		/* undef to save memory */
#define CONFIG_SYS_PROMPT		"DVF101 # "
#define CONFIG_SYS_CBSIZE		1024

/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE + \
					 sizeof(CONFIG_SYS_PROMPT) + 16)
/* max number of command args */
#define CONFIG_SYS_MAXARGS		64
/* boot argument buffer size */
#define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE

#define CONFIG_MTD_NAND_VERIFY_WRITE
#define CONFIG_LIB_RAND
#define CONFIG_CMD_MEMTEST

/* Secure boot */
/* #define CONFIG_DVF_SECURE_BOOT */
/* #define CONFIG_DVF_SECURE_BOOT_DEBUG */

#define CONFIG_SYS_GENERIC_BOARD

#endif
