#ifndef __CONFIG_DVF97_H
#define __CONFIG_DVF97_H

#include <asm/arch/gpio.h>
#include <linux/sizes.h>

#ifdef CONFIG_QSPI
    #define MTDIDS_DEFAULT		"nor0=qspi0"

    #define NAND_SLICED_UPDATE

    #ifdef CONFIG_SOFTWARE_UPGRADE_UBOOT
        #define MTDPARTS_DEFAULT_UBOOT \
            "640k(u-boot-1),"  \
            "640k(u-boot-2),"
    #else
        #define MTDPARTS_DEFAULT_UBOOT ""
    #endif
    #ifdef CONFIG_SOFTWARE_UPGRADE_ROOTFS
        #define MTDPARTS_DEFAULT_ROOTFS \
            "15m(rootfs),"   \
            "15m(rootfs-2)"
    #else
        #define MTDPARTS_DEFAULT_ROOTFS \
            "-(rootfs)"
    #endif
    #define MTDPARTS_DEFAULT   \
        "mtdparts=qspi0:"      \
        "640k(u-boot),"        \
        "128k(u-boot-env),"    \
        MTDPARTS_DEFAULT_UBOOT \
        "1856k(kernel),"       \
        "128k(devtree),"       \
        MTDPARTS_DEFAULT_ROOTFS

    #define KERNEL_BOOTARGS \
        "root=mtd:rootfs rootfstype=jffs2\0"

    #define FLASH_ENV \
        "erasecmd=erase\0"           \
        "readcmd=true\0"             \
        "writecmd=cp.b\0"

    #define KERNEL_BOOTCMD \
        "run chk_parts; cp.b kernel ${loadaddr}; cp.b devtree ${devtree_addr}; run bootm\0"
    #define MANIFEST_SYSTEM \
        "dfu_manifest_system=${erasecmd} ${rootfspart} "                  \
                "&& ${writecmd} ${loadaddr} ${rootfspart} ${filesize} "   \
                "; reset\0"
#else
    #error "Unsupported boot medium"
#endif

#ifdef CONFIG_SOFTWARE_UPGRADE_UBOOT
    #define UPGRADE_ENV                                                   \
	"ubootpart=u-boot-1\0"                                            \
	"uboot_update=0\0"                                                \
	"rootfs_update=0\0"                                               \
	"uboot_partition_safe=u-boot-1\0"
#else
    #define UPGRADE_ENV "ubootpart=u-boot\0"
#endif /* CONFIG_SOFTWARE_UPGRADE_UBOOT */

#define BOOTMEDIUM_ENV_SETTINGS \
	FLASH_ENV                                                         \
	UPGRADE_ENV                                                       \
	"rootfspart=rootfs\0"                                             \
	"linux_boot=" KERNEL_BOOTCMD                                      \
	"linux_boot_flash=" KERNEL_BOOTCMD                                \
	"linux_bootargs=" KERNEL_BOOTARGS                                 \
	"linux_bootargs_flash=" KERNEL_BOOTARGS                           \
	"setubimode=setenv linux_boot ${linux_boot_flash} "               \
		"&& setenv linux_bootargs ${linux_bootargs_flash} "       \
		"&& save\0"                                               \
	"chk_parts=test -n ${mtdparts} || mtdparts default\0"             \
	"dfu_manifest_2=run chk_parts; run dfu_manifest_u-boot-env\0"     \
	"dfu_manifest_u-boot=run u-boot-upgrade\0"                        \
	"dfu_manifest_u-boot-env=${erasecmd} u-boot-env "                 \
		"&& ${writecmd} ${loadaddr} u-boot-env\0"                 \
	"dfu_manifest_devtree=${erasecmd} devtree "                       \
		"&& ${writecmd} ${loadaddr} devtree ${filesize}\0"        \
	"dfu_manifest_kernel=${erasecmd} kernel "                         \
		"&& ${writecmd} ${loadaddr} kernel ${filesize}\0"         \
	MANIFEST_SYSTEM                                                   \
	"u-boot-upgrade=run chk_parts; "                                  \
		"${erasecmd} ${ubootpart} "                               \
		"&& ${writecmd} ${loadaddr} ${ubootpart} "                \
		"&& save "                                                \
		"&& echo Done. Please do a power cycle to reset board\0"  \
	"bgvalue=100\0"                                                   \
	NAND_SLICED_UPDATE

#ifdef CONFIG_QSPI
    #define CONFIG_CMD_FLASH
    #define CONFIG_CMD_JFFS2
    #define CONFIG_CMD_CRAMFS

    #define CONFIG_DSPG_QSPI
    #define CONFIG_SYS_FLASH_EXIT
    #define CONFIG_SYS_FLASH_BASE		0x80000000
    #define CONFIG_SYS_MAX_FLASH_SECT		16384	/* 512Mbits / 4096Bytes = 16384 sectors */
    #define CONFIG_SYS_MAX_FLASH_BANKS		1

    /* The following is required for crc32_le needed for Secureboot */
    #define CONFIG_RBTREE
    #define CONFIG_CMD_UBI

    /* Environment settings */
    #define CONFIG_ENV_OVERWRITE
    #define CONFIG_ENV_IS_IN_FLASH
    #define CONFIG_ENV_OFFSET			(640 << 10) /* 640k */
    #define CONFIG_ENV_SIZE			(128 << 10) /* 128k */
#endif

/*
 * Physical Memory Map
 */
#ifndef CONFIG_NR_DRAM_BANKS
#define CONFIG_NR_DRAM_BANKS		1
#endif
#ifndef PHYS_SDRAM_1
#define PHYS_SDRAM_1			0xC8000000
#define CONFIG_SYS_TEXT_BASE		0xC8200000
#define CONFIG_SYS_LOAD_ADDR		0xC8010000
#endif
#define CONFIG_SYS_SDRAM_BASE		PHYS_SDRAM_1

#ifndef CONFIG_SYS_DEVTREE_OFFSET
#define CONFIG_SYS_DEVTREE_OFFSET	"0xC8300000"
#endif
#define SYS_DEVTREE_OFFSET		"devtree_addr=" CONFIG_SYS_DEVTREE_OFFSET "\0"
#ifndef CONFIG_SYS_DEVICETREE
#define CONFIG_SYS_DEVICETREE		"dvf97-evb.dtb"
#endif
#define SYS_DEVICETREE			"devicetree=" CONFIG_SYS_DEVICETREE "\0"

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN		(2 * SZ_1M)

/*
 * Stack sizes - set up in start.S using sizes below
 */
#define CONFIG_SYS_INIT_SP_ADDR		(CONFIG_SYS_TEXT_BASE - SZ_4K - \
                                         GENERATED_GBL_DATA_SIZE)

#define CONFIG_SYS_DCACHE_OFF

#define CONFIG_ARCH_CPU_INIT
#define CONFIG_BOARD_EARLY_INIT_F
#define CONFIG_BOARD_LATE_INIT
#define CFG_PBSIZE			200

#define CONFIG_ARM926EJS

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO
#define CONFIG_VERSION_VARIABLE

#define CONFIG_SYS_ALT_MEMTEST
#define CONFIG_SYS_MEMTEST_START	0x100000
#define CONFIG_SYS_MEMTEST_END		0x10000000

#define CONFIG_INITRD_TAG		1
#define CONFIG_CMDLINE_TAG /* pass cmdline  */
#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_OF_LIBFDT

#define CONFIG_SYS_HZ			1000

/*
 * Hardware drivers
 */
#ifdef CONFIG_NETWORKING
    /* #define CONFIG_MACG */
    #define CONFIG_DESIGNWARE_ETH
    #define CONFIG_DW_ALTDESCRIPTOR
    #define CONFIG_DW_SEARCH_PHY
    /* #define CONFIG_DW_AUTONEG */
    #define CONFIG_NET_MULTI
    #define CONFIG_MII
    #define CONFIG_CMD_MII
    #define CONFIG_PHYLIB
#endif

/* SPI */
#define CONFIG_DSPG_SPI
#define CONFIG_CMD_SPI

/*
 * NS16550 Configuration
 */
#define CFG_DSPG_DMW_SERIAL

#define CONFIG_SERIAL_MULTI

#define CONFIG_CMDLINE_EDITING		1
#define CONFIG_AUTO_COMPLETE		1
#define CONFIG_MX_CYCLIC		1

#define CONFIG_CONS_INDEX		0

#define CONFIG_BAUDRATE			115200
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define CONFIG_BOOTP_MASK		CONFIG_BOOTP_DEFAULT

#define CONFIG_SYS_HUSH_PARSER		1
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "

#include <config_cmd_default.h>
#define CONFIG_CMD_IMI
#define CONFIG_CMD_BDI
#define CONFIG_CMD_MEMORY
#define CONFIG_CMD_ENV
#define CONFIG_CMD_RUN
#define CONFIG_CMD_ECHO
#define CONFIG_CMD_ITEST
#define CONFIG_CMD_REV
#define CONFIG_CMD_GREPENV
#define CONFIG_CMD_DEFENV
#define CONFIG_CMD_MD5SUM
#define CONFIG_MD5
#define CONFIG_CMD_SETEXPR
#ifdef CONFIG_NETWORKING
    #define CONFIG_CMD_NET
    #define CONFIG_CMD_DHCP
    #define CONFIG_CMD_PING
#endif
#define CONFIG_CMD_MII

/* I2C settings */
#ifdef CONFIG_I2C
    #define CONFIG_HARD_I2C
    #define CONFIG_PNX8181_I2C
    #define CONFIG_PNX8181_I2C_ADDRESS_1	0x06200000
    #define CONFIG_PNX8181_I2C_ADDRESS_2	0x06200000
    #define CONFIG_PNX8181_I2C_BUS_MAX		1
    #define CONFIG_SYS_I2C_SPEED		100000
    #define CONFIG_SYS_I2C_SLAVE		0xfe
    #define CONFIG_CMD_I2C
    #define CONFIG_I2C_MULTI_BUS
#endif

#define CONFIG_BOOTDELAY		4
#define CONFIG_BOOTABORTKEY		0x1b /* ascii for escape */
#define CONFIG_TFTP_PORT		1

#define CONFIG_NET_RETRY_COUNT		7

#define CONFIG_BOOTCOMMAND \
	"run linux_boot"

#ifdef DW_USBTTY_AS_DEFAULT
#define USBTTY_ENV_SETTINGS \
	"stdin=usbtty\0"  \
	"stdout=usbtty\0" \
	"stderr=usbtty\0"
#elif defined(CONFIG_ARM_DCC)
#define USBTTY_ENV_SETTINGS \
	"stdin=dcc\0"  \
	"stdout=dcc\0" \
	"stderr=dcc\0"
#else
#define USBTTY_ENV_SETTINGS
#endif

#define CONFIG_EXTRA_ENV_SETTINGS \
	SYS_DEVTREE_OFFSET                                                \
	SYS_DEVICETREE                                                    \
	"dfu_manifest=run bootm\0"                                        \
	"console=console=ttyS0\0"                                         \
	"bootm=setenv bootargs ${console} ${linux_bootargs} ${mtdparts} " \
		"&& bootm ${loadaddr} - ${devtree_addr}\0"                \
	"linux_boot_nfs=tftp ${loadaddr} uImage "                         \
		"&& tftp ${devtree_addr} ${devicetree} "                  \
		"&& run bootm\0"                                          \
	"linux_bootargs_nfs=root=/dev/nfs ip=dhcp "                       \
		"nfsroot=192.168.0.1:/\0"                                 \
	"setnfsmode=setenv linux_boot ${linux_boot_nfs} "                 \
		"&& setenv linux_bootargs ${linux_bootargs_nfs} "         \
		"&& save\0"                                               \
	BOOTMEDIUM_ENV_SETTINGS                                           \
	USBTTY_ENV_SETTINGS

/*
 * Static configuration when assigning fixed address
 */
#ifdef CONFIG_NETWORKING
    #define CONFIG_ETHADDR		54:89:07:9B:8A:0D
    #define CONFIG_NETMASK		255.255.255.0
    #define CONFIG_IPADDR		192.168.0.240
    #define CONFIG_SERVERIP		192.168.0.1
#endif
#define CONFIG_BOOTFILE			"uImage"
#define CONFIG_LOADADDR			CONFIG_SYS_LOAD_ADDR

/*
 * Flash configuration
 */
#define CONFIG_CMD_MTDPARTS
#define CONFIG_MTD_DEVICE
#define CONFIG_MTD_PARTITIONS

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP		/* undef to save memory */
#define CONFIG_SYS_PROMPT		"DVF97 # "
#define CONFIG_SYS_CBSIZE		1024

/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE + \
					 sizeof(CONFIG_SYS_PROMPT) + 16)
/* max number of command args */
#define CONFIG_SYS_MAXARGS		64
/* boot argument buffer size */
#define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE

/* Secure boot */
#define CONFIG_DVF97_SECURE_BOOT
/* #define CONFIG_DVF97_SECURE_BOOT_DEBUG */

/* #define CONFIG_CSS */
#ifdef CONFIG_CSS
    #define CONFIG_CSS_AHBRAM		1

    #define CONFIG_CSS_DEBUG		1

    #define CONFIG_CSS_ITCM_BASE	0x08000000
    #define CONFIG_CSS_ITCM_SIZE	(272 * SZ_1K)

    #define CONFIG_CSS_AHBRAM_BASE	0x08200000
    #define CONFIG_CSS_AHBRAM_SIZE	(82 * SZ_1K)

    #define CONFIG_CSS_DTCM_BASE	0x08100000
    #define CONFIG_CSS_DTCM_SIZE	SZ_128K

     /* this memory location is free after u-boot relocation and should never be used */
    #define CONFIG_CSS_DRAM_ADDR	PHYS_SDRAM_1
    #define CONFIG_CSS_DRAM_SIZE	SZ_1M
#endif

#define CONFIG_SYS_GENERIC_BOARD

#endif
