#ifndef __CONFIG_DVF99_H
#define __CONFIG_DVF99_H

#include <asm/arch/gpio.h>
#include <linux/sizes.h>

#ifdef CONFIG_NAND
    #define MTDIDS_DEFAULT		"nand0=dmw_nand"

    #define NAND_SLICED_UPDATE                                  \
        "nandfile=nand.image.slice\0"                           \
        "last_nand_offset=0\0"                                  \
        "nandfileloadcmd=tftp\0"                                \
        "nand_sliced_update="                                   \
          "if test \"0\" -ne $last_nand_offset;"                \
          " then "                                              \
             "$erasecmd $last_nand_offset;"                     \
          " else "                                              \
             "nand erase.chip;"                                 \
          " fi; "                                               \
          "setenv slice 0;"                                     \
          "while $nandfileloadcmd $nandfile$slice;"             \
          " do "                                                \
             "$writecmd $loadaddr $last_nand_offset $filesize;" \
             "setexpr slice $slice + 1;"                        \
          " done\0"

    #ifdef CONFIG_SOFTWARE_UPGRADE_UBOOT
        #define MTDPARTS_DEFAULT_UBOOT \
            "2m(u-boot-1),"  \
            "2m(u-boot-2),"
    #else
        #define MTDPARTS_DEFAULT_UBOOT ""
    #endif
    #ifdef CONFIG_SOFTWARE_UPGRADE_ROOTFS
        #define MTDPARTS_DEFAULT_ROOTFS \
            "60m(rootfs),"   \
            "60m(rootfs-2)"
    #else
        #define MTDPARTS_DEFAULT_ROOTFS \
            "-(rootfs)"
    #endif
    #define MTDPARTS_DEFAULT   \
        "mtdparts=dmw_nand:"   \
        "2m(u-boot),"          \
        "1m(u-boot-env),"      \
        MTDPARTS_DEFAULT_UBOOT \
        MTDPARTS_DEFAULT_ROOTFS

    #ifdef CONFIG_JFFS2_NAND
        #define KERNEL_BOOTARGS \
           "root=mtd:rootfs rootfstype=jffs2\0"
    #else
        #define KERNEL_BOOTARGS \
           "ubi.mtd=rootfs root=ubi0:rootfs rootfstype=ubifs rw\0"
    #endif

    #define FLASH_ENV \
        "erasecmd=nand erase.part\0" \
        "readcmd=nand read\0"        \
        "writecmd=nand write\0"

    #ifdef CONFIG_JFFS2_NAND
        #define KERNEL_BOOTCMD \
            "run chk_parts; chpart ${rootfspart}; "                           \
            "fsload ${loadaddr} /boot/uImage && "                             \
            "fsload ${devtree_addr} boot/${devicetree} && run bootm\0"
    #else
        #define KERNEL_BOOTCMD \
            "run chk_parts; ubi part ${rootfspart} && ubifsmount ubi0:rootfs && "  \
            "ubifsload ${loadaddr} boot/uImage && "                           \
            "ubifsload ${devtree_addr} boot/${devicetree} && run bootm\0"
    #endif

    #ifdef CONFIG_JFFS2_NAND
        #define MANIFEST_SYSTEM \
            "dfu_manifest_system=${erasecmd} ${rootfspart} "                  \
                    "&& ${writecmd} ${loadaddr} ${rootfspart} ${filesize} "   \
                    "; reset\0"
    #else
        #define MANIFEST_SYSTEM \
            "dfu_manifest_system=${erasecmd} ${rootfspart} "                  \
                    "&& ubi part ${rootfspart} "                              \
                    "&& ubi create rootfs "                                   \
                    "&& ubi write ${loadaddr} rootfs ${filesize} "            \
                    "; reset\0"
    #endif
#else
#ifdef CONFIG_QSPI
    #define MTDIDS_DEFAULT		"nor0=qspi0"

    #define NAND_SLICED_UPDATE

    #ifdef CONFIG_SOFTWARE_UPGRADE_UBOOT
        #define MTDPARTS_DEFAULT_UBOOT \
            "640k(u-boot-1),"  \
            "640k(u-boot-2),"
    #else
        #define MTDPARTS_DEFAULT_UBOOT ""
    #endif
    #ifdef CONFIG_SOFTWARE_UPGRADE_ROOTFS
        #define MTDPARTS_DEFAULT_ROOTFS \
            "15m(rootfs),"   \
            "15m(rootfs-2)"
    #else
        #define MTDPARTS_DEFAULT_ROOTFS \
            "-(rootfs)"
    #endif
    #define MTDPARTS_DEFAULT   \
        "mtdparts=qspi0:"       \
        "640k(u-boot),"        \
        "128k(u-boot-env),"    \
        MTDPARTS_DEFAULT_UBOOT \
        "1856k(kernel),"       \
        "128k(devtree),"       \
        MTDPARTS_DEFAULT_ROOTFS

    #define KERNEL_BOOTARGS \
        "root=mtd:rootfs rootfstype=jffs2\0"

    #define FLASH_ENV \
        "erasecmd=erase\0"           \
        "readcmd=true\0"             \
        "writecmd=cp.b\0"

    #define KERNEL_BOOTCMD \
        "run chk_parts; cp.b kernel ${loadaddr}; cp.b devtree ${devtree_addr}; run bootm\0"
    #define MANIFEST_SYSTEM \
        "dfu_manifest_system=${erasecmd} ${rootfspart} "                  \
                "&& ${writecmd} ${loadaddr} ${rootfspart} ${filesize} "   \
                "; reset\0"
#else
#ifdef CONFIG_EMMC
    #ifndef CONFIG_EMMC_CARDNUM
        #define CONFIG_EMMC_CARDNUM	"0"
    #endif

    #define NAND_SLICED_UPDATE ""

    #define KERNEL_BOOTARGS                                               \
        "rootfstype=ext4 root=/dev/mmcblk0p3 rootwait\0"

    #define FLASH_ENV                                                     \
        "readcmd=read\0"                                                  \
        "writecmd=write\0"                                                \
        "switchcmd=mmc dev "CONFIG_EMMC_CARDNUM"\0"

    #define KERNEL_BOOTCMD                                                \
        "${switchcmd} 0 && "                                              \
        "${readcmd} mmc "CONFIG_EMMC_CARDNUM":1 ${loadaddr} 0 && "        \
        "${readcmd} mmc "CONFIG_EMMC_CARDNUM":2 ${devtree_addr} 0 && "    \
        "run bootm\0"

    #define MTDPARTS_DEFAULT                                              \
        "mtdparts=\0"

    #define MANIFEST_SYSTEM                                               \
        "update_kernel=${switchcmd} 0 && ${writecmd} "                    \
            "mmc "CONFIG_EMMC_CARDNUM":1 ${loadaddr} ${filesize}\0"       \
        "update_devtree=${switchcmd} 0 && ${writecmd} "                   \
            "mmc "CONFIG_EMMC_CARDNUM":2 ${loadaddr} ${filesize}\0"       \
        "update_rootfs=${switchcmd} 0 && ${writecmd} "                    \
            "mmc "CONFIG_EMMC_CARDNUM":3 ${loadaddr} ${filesize}\0"       \
        "update_uboot=${switchcmd} 1 && &{writecmd} "                     \
            "mmc "CONFIG_EMMC_CARDNUM":1 ${loadaddr} ${filesize}\0"       \
        "update_ubootenv=${switchcmd} 1 && ${writecmd} "                  \
            "mmc "CONFIG_EMMC_CARDNUM":2 ${loadaddr} ${filesize}\0"
#else
    #error "Unsupported boot medium"
#endif
#endif
#endif

#ifdef CONFIG_SOFTWARE_UPGRADE_UBOOT
    #define UPGRADE_ENV                                                   \
	"ubootpart=u-boot-1\0"                                            \
	"uboot_update=0\0"                                                \
	"rootfs_update=0\0"                                               \
	"uboot_partition_safe=u-boot-1\0"
#else
    #define UPGRADE_ENV "ubootpart=u-boot\0"
#endif

#define BOOTMEDIUM_ENV_SETTINGS \
	FLASH_ENV                                                         \
	UPGRADE_ENV                                                       \
	"rootfspart=rootfs\0"                                             \
	"linux_boot=" KERNEL_BOOTCMD                                      \
	"linux_boot_flash=" KERNEL_BOOTCMD                                \
	"linux_bootargs=" KERNEL_BOOTARGS                                 \
	"linux_bootargs_flash=" KERNEL_BOOTARGS                           \
	"setubimode=setenv linux_boot ${linux_boot_flash} "               \
		"&& setenv linux_bootargs ${linux_bootargs_flash} "       \
		"&& save\0"                                               \
	"chk_parts=test -n ${mtdparts} || mtdparts default\0"             \
	"dfu_manifest_2=run chk_parts; run dfu_manifest_u-boot-env\0"     \
	"dfu_manifest_u-boot=run u-boot-upgrade\0"                        \
	"dfu_manifest_u-boot-env=${erasecmd} u-boot-env "                 \
		"&& ${writecmd} ${loadaddr} u-boot-env\0"                 \
	"dfu_manifest_devtree=${erasecmd} devtree "                       \
		"&& ${writecmd} ${loadaddr} devtree ${filesize}\0"        \
	"dfu_manifest_kernel=${erasecmd} kernel "                         \
		"&& ${writecmd} ${loadaddr} kernel ${filesize}\0"         \
	MANIFEST_SYSTEM                                                   \
	"u-boot-upgrade=run chk_parts; "                                  \
		"${erasecmd} ${ubootpart} "                               \
		"&& ${writecmd} ${loadaddr} ${ubootpart} "                \
		"&& save "                                                \
		"&& echo Done. Please do a power cycle to reset board\0"  \
	"bgvalue=100\0"                                                   \
	NAND_SLICED_UPDATE

#ifdef CONFIG_NAND
    #define CONFIG_NAND_DMW96
    #define CONFIG_LZO    /* needed for UBI */
    #define CONFIG_RBTREE /* needed for UBI */

    #define CONFIG_SYS_NO_FLASH
    #define CONFIG_SYS_NAND_BASE		0x05400000
    #define CONFIG_SYS_MAX_NAND_DEVICE		1
    #define CONFIG_SYS_NAND_ONFI_DETECTION
    #define CONFIG_SYS_NAND_SELF_INIT
    #define NAND_MAX_CHIPS			1 /* needed for u-boot bbt */

    #define CONFIG_NAND_DMW96_REV1
    #define CONFIG_CMD_NAND
    #define CONFIG_CMD_JFFS2
    #define CONFIG_JFFS2_NAND
    #define CONFIG_CMD_UBI
    #define CONFIG_CMD_UBIFS
    #define CONFIG_CMD_MTDPROTECT

    /* Environment settings */
    #define CONFIG_ENV_OVERWRITE
    #define CONFIG_ENV_IS_IN_NAND
    #define CONFIG_ENV_OFFSET			(  2 << 20) /* 2M */
    #define CONFIG_ENV_SIZE			(256 << 10) /* 256k */
    #define CONFIG_ENV_RANGE			(  1 << 20) /* 1M */
#else
#ifdef CONFIG_QSPI
    #define CONFIG_CMD_FLASH
    #define CONFIG_CMD_JFFS2
    #define CONFIG_CMD_CRAMFS

    #define CONFIG_DSPG_QSPI
    #define CONFIG_SYS_FLASH_EXIT
    #define CONFIG_SYS_FLASH_BASE		0x80000000
    #define CONFIG_SYS_MAX_FLASH_SECT		16384	/* 512Mbits / 4096Bytes = 16384 sectors */
    #define CONFIG_SYS_MAX_FLASH_BANKS		2

#if 0
    /* The following is required for crc32_le needed for Secureboot */
    #define CONFIG_RBTREE
    #define CONFIG_CMD_UBI
#endif

    /* Environment settings */
    #define CONFIG_ENV_OVERWRITE
    #define CONFIG_ENV_IS_IN_FLASH
    #define CONFIG_ENV_OFFSET			(640 << 10) /* 640k */
    #define CONFIG_ENV_SIZE			(128 << 10) /* 128k */
#else
#ifdef CONFIG_EMMC
    #define CONFIG_ENV_IS_IN_MMC		1
    #define CONFIG_SYS_MMC_ENV_DEV		0
    #define CONFIG_ENV_SIZE			0x40000 /* 256k */
#endif
#endif
#endif

#include <configs/dvf99.h>
/*
 * Physical Memory Map
 */
#ifndef CONFIG_NR_DRAM_BANKS
#define CONFIG_NR_DRAM_BANKS		1
#endif
#ifndef PHYS_SDRAM_1
#define PHYS_SDRAM_1			0x40000000
#define CONFIG_SYS_TEXT_BASE		0x41000000
#define CONFIG_SYS_LOAD_ADDR		0x42000000
#endif
#define CONFIG_SYS_SDRAM_BASE		PHYS_SDRAM_1

#ifndef CONFIG_SYS_DEVTREE_OFFSET
#define CONFIG_SYS_DEVTREE_OFFSET	"0x43000000"
#endif
#define SYS_DEVTREE_OFFSET		"devtree_addr=" CONFIG_SYS_DEVTREE_OFFSET "\0"
#ifndef CONFIG_SYS_DEVICETREE
#define CONFIG_SYS_DEVICETREE		"dvf99-evb.dtb"
#endif
#define SYS_DEVICETREE			"devicetree=" CONFIG_SYS_DEVICETREE "\0"

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN		(2 * SZ_1M)

/*
 * Stack sizes - set up in start.S using sizes below
 */
#define CONFIG_SYS_INIT_SP_ADDR		(CONFIG_SYS_TEXT_BASE - SZ_4K - \
                                         GENERATED_GBL_DATA_SIZE)

#define CONFIG_SYS_DCACHE_OFF

#define CONFIG_ARCH_CPU_INIT
#define CONFIG_BOARD_EARLY_INIT_F
#define CONFIG_BOARD_LATE_INIT
#define CFG_PBSIZE			200

#define CONFIG_ARM926EJS
#define CONFIG_HAVE_CCU

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO
#define CONFIG_VERSION_VARIABLE

#define CONFIG_SYS_ALT_MEMTEST
#define CONFIG_SYS_MEMTEST_START	0x100000
#define CONFIG_SYS_MEMTEST_END		0x10000000

#define CONFIG_OF_LIBFDT

#ifdef CONFIG_LCD
    #define CONFIG_VIDEO
    #define LCD_BPP			LCD_COLOR16
    #define CONFIG_CMD_BMP
    #define CONFIG_BMP_16BPP
    #define CONFIG_CFB_CONSOLE
    #define CONFIG_VGA_AS_SINGLE_DEVICE

    #define VIDEO_KBD_INIT_FCT		0
    #define VIDEO_TSTC_FCT		serial_tstc
    #define VIDEO_GETC_FCT		serial_getc

    /* #define VIDEO_FB_LITTLE_ENDIAN */
    /* #define VIDEO_FB_16BPP_PIXEL_SWAP */
    #define VIDEO_FB_16BPP_WORD_SWAP
#endif

#define CONFIG_SYS_HZ			1000

/*
 * Hardware drivers
 */
#ifdef CONFIG_NETWORKING
    #define CONFIG_MACG
    #define CONFIG_NET_MULTI
    #define CONFIG_MII
#endif

#ifdef CONFIG_MMC
    #define CONFIG_DOS_PARTITION
    #define CONFIG_GENERIC_MMC
    #define CONFIG_SYNDWC_MMC
#endif

/* SPI */
#define CONFIG_DSPG_SPI
#define CONFIG_CMD_SPI

/*
 * NS16550 Configuration
 */
#define CFG_DSPG_DMW_SERIAL

#define CONFIG_SERIAL_MULTI

#define CONFIG_CMDLINE_EDITING		1
#define CONFIG_AUTO_COMPLETE		1
#define CONFIG_MX_CYCLIC		1

#define CONFIG_CONS_INDEX		0

#define CONFIG_BAUDRATE			115200
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define CONFIG_BOOTP_MASK		CONFIG_BOOTP_DEFAULT

#define CONFIG_SYS_HUSH_PARSER		1
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "

/*
 * USB
 */
#ifdef CONFIG_USB
    #define CONFIG_USB_DEVICE			1
    #define CONFIG_USB_TTY			1
    #define CONFIG_SYS_CONSOLE_IS_IN_ENV	1
    #define CONFIG_USBD_DFU			1

    #define CONFIG_USB_HIGH_SPEED		1
    #define CONFIG_DWC_OTG_UDC			1

    /* DWC driver settings */
    #define CONFIG_DWC_U_BOOT			1
    #define CONFIG_USB_DWC_DEBUG		1
    #define CONFIG_USB_GADGET_DWC_HDRC		1
    #define CONFIG_USB_DWC_DEVICE_ONLY		1

    #define CONFIG_USB_DWC_PORT			1

    #define CONFIG_SYS_DWC_OTG_BASE		0x00300000

    #define CONFIG_USBD_VENDORID		0x1d50    /* Linux/NetChip */
    #define CONFIG_USBD_PRODUCTID_GSERIAL	0x5120    /* gserial */
    #define CONFIG_USBD_PRODUCTID_CDCACM	0x5119
    #define CONFIG_USBD_MANUFACTURER		"DSPG"
    #define CONFIG_USBD_PRODUCT_NAME		"DFU/USB"
    #define CONFIG_USBD_DFU_XFER_SIZE		4096
    #define CONFIG_USBD_DFU_INTERFACE		2

    #define CONFIG_USBD_DFUF			1
    #define CONFIG_USBD_DFUF_SI			1
#endif

#include <config_cmd_default.h>
#define CONFIG_CMD_IMI
#define CONFIG_CMD_BDI
#define CONFIG_CMD_MEMORY
#define CONFIG_CMD_ENV
#define CONFIG_CMD_RUN
#define CONFIG_CMD_ECHO
#define CONFIG_CMD_ITEST
#define CONFIG_CMD_REV
#define CONFIG_CMD_GREPENV
#define CONFIG_CMD_DEFENV
#define CONFIG_CMD_MD5SUM
#define CONFIG_MD5
#define CONFIG_CMD_SETEXPR
#ifdef CONFIG_NETWORKING
    #define CONFIG_CMD_NET
    #define CONFIG_CMD_DHCP
    #define CONFIG_CMD_PING
#endif
#ifdef CONFIG_MMC
    #define CONFIG_CMD_MMC
    #define CONFIG_CMD_FAT
#endif

/* I2C settings */
#ifdef CONFIG_I2C
    #define CONFIG_HARD_I2C
    #define CONFIG_PNX8181_I2C
    #define CONFIG_PNX8181_I2C_ADDRESS_1	0x06200000
    #define CONFIG_PNX8181_I2C_ADDRESS_2	0x06100000
    #define CONFIG_PNX8181_I2C_BUS_MAX		2
    #define CONFIG_SYS_I2C_SPEED		100000
    #define CONFIG_SYS_I2C_SLAVE		0xfe
    #define CONFIG_CMD_I2C
    #define CONFIG_I2C_MULTI_BUS
#endif

#define CONFIG_BOOTDELAY		4
#define CONFIG_BOOTABORTKEY		0x1b /* ascii for escape */
#define CONFIG_TFTP_PORT		1

#define CONFIG_NET_RETRY_COUNT		7

#define CONFIG_BOOTCOMMAND \
	"if mmc rescan; then " \
		"if run loadbootscript; then " \
			"echo \"Boot script found on SD card. Do 'run bootscript' to execute\"; " \
			/* "run bootscript; " */ \
		"else " \
			"run linux_boot; " \
		"fi; " \
	"else run linux_boot; fi"

#ifdef DW_USBTTY_AS_DEFAULT
#define USBTTY_ENV_SETTINGS \
	"stdin=usbtty\0"  \
	"stdout=usbtty\0" \
	"stderr=usbtty\0"
#elif defined(CONFIG_ARM_DCC)
#define USBTTY_ENV_SETTINGS \
	"stdin=dcc\0"  \
	"stdout=dcc\0" \
	"stderr=dcc\0"
#else
#define USBTTY_ENV_SETTINGS
#endif

#define CONFIG_EXTRA_ENV_SETTINGS \
	SYS_DEVTREE_OFFSET                                                \
	SYS_DEVICETREE                                                    \
	"loadbootscript=fatload mmc 0 ${loadaddr} boot.scr\0"             \
	"bootscript=echo Running bootscript from mmc ...; "               \
		"source ${loadaddr}\0"                                    \
	"dfu_manifest=run bootm\0"                                        \
	"console=console=ttyS0\0"                                         \
	"bootm=setenv bootargs ${console} ${linux_bootargs} ${mtdparts} " \
		"&& bootm ${loadaddr} - ${devtree_addr}\0"                \
	"linux_boot_nfs=tftp ${loadaddr} uImage "                         \
		"&& tftp ${devtree_addr} ${devicetree} "                  \
		"&& run bootm\0"                                          \
	"linux_bootargs_nfs=root=/dev/nfs ip=dhcp "                       \
		"nfsroot=192.168.0.1:/\0"                                 \
	"setnfsmode=setenv linux_boot ${linux_boot_nfs} "                 \
		"&& setenv linux_bootargs ${linux_bootargs_nfs} "         \
		"&& save\0"                                               \
	BOOTMEDIUM_ENV_SETTINGS                                           \
	USBTTY_ENV_SETTINGS

/*
 * Static configuration when assigning fixed address
 */
#ifdef CONFIG_NETWORKING
    #define CONFIG_ETHADDR		54:89:07:9B:8A:0D
    #define CONFIG_NETMASK		255.255.255.0
    #define CONFIG_IPADDR		192.168.0.240
    #define CONFIG_SERVERIP		192.168.0.1
#endif
#define CONFIG_BOOTFILE			"uImage"
#define CONFIG_LOADADDR			CONFIG_SYS_LOAD_ADDR

#ifndef CONFIG_EMMC
    /*
     * Flash configuration
     */
    #define CONFIG_CMD_MTDPARTS
    #define CONFIG_MTD_DEVICE
    #define CONFIG_MTD_PARTITIONS
#endif

#ifdef CONFIG_EMMC
    #define CONFIG_DOS_PARTITION
    #define CONFIG_EFI_PARTITION
    #define CONFIG_CMD_READ
    #define CONFIG_CMD_WRITE
    #define CONFIG_CMD_GPTPART
    #define CONFIG_CMD_EXT4
    #define CONFIG_FS_EXT4
#endif

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP		/* undef to save memory */
#define CONFIG_SYS_PROMPT		"DVF99 # "
#define CONFIG_SYS_CBSIZE		1024

/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE + \
					 sizeof(CONFIG_SYS_PROMPT) + 16)
/* max number of command args */
#define CONFIG_SYS_MAXARGS		64
/* boot argument buffer size */
#define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE

#define CONFIG_MTD_NAND_VERIFY_WRITE
#define CONFIG_LIB_RAND
#define CONFIG_CMD_MEMTEST

/* Secure boot */
/* #define CONFIG_DVF99_SECURE_BOOT */
/* #define CONFIG_DVF99_SECURE_BOOT_DEBUG */

/* #define CONFIG_CSS */
#ifdef CONFIG_CSS
    #define CONFIG_CSS_AHBRAM		1

    #define CONFIG_CSS_DEBUG		1

    #define CONFIG_CSS_ITCM_BASE	0x08000000
    #define CONFIG_CSS_ITCM_SIZE	(272 * SZ_1K)

    #define CONFIG_CSS_AHBRAM_BASE	0x08200000
    #define CONFIG_CSS_AHBRAM_SIZE	(82 * SZ_1K)

    #define CONFIG_CSS_DTCM_BASE	0x08100000
    #define CONFIG_CSS_DTCM_SIZE	SZ_128K

     /* this memory location is free after u-boot relocation and should never be used */
    #define CONFIG_CSS_DRAM_ADDR	PHYS_SDRAM_1
    #define CONFIG_CSS_DRAM_SIZE	SZ_1M
#endif

#define CONFIG_SYS_GENERIC_BOARD

#endif
