#ifndef __CONFIG_HT818_NAND_H
#define __CONFIG_HT818_NAND_H

#include <asm/arch/gpio.h>
#include <linux/sizes.h>

#define CONFIG_IDENT_STRING " HT818-nand"
#define CONFIG_HT818
/*
 * Device drivers & peripherals:
 */
#define CONFIG_NAND
#define CONFIG_MMC
#define CONFIG_NETWORKING
#define CONFIG_LCD_DSPG
#define CONFIG_DSPG_FB_ADDR	0x43800000
#define CONFIG_NVRAM
#define CONFIG_GZIP_COMPRESSED
#define CONFIG_GSFACTORY

/*
 * Board-specific configuration:
 */
#define CONFIG_SDMMC_4BIT

/*
 * Configure which UART to use for stdio
 * 0: choose UART according to straps (UART1 or UART2)
 * 1: UART1
 * 2: UART2
 * 3: UART3
 * 4: UART4
 */
#define CONFIG_USE_UART		0

#define CONFIG_SERIAL_MULTI

#ifdef CONFIG_NETWORKING
	#define CONFIG_RGMII
	#define CONFIG_QCA833X_SWITCH
	#define CONFIG_RTL836X_SWITCH_EXT
	/* comment out below MACRO to enable full realtek switch features. */
	/* #undef CONFIG_RTL836X_SWITCH_EXT */
	#define CONFIG_RTL836X_SWITCH
	#define CONFIG_MDC_MDIO_OPERATION
	#define CONFIG_PHY_FIXED

	#define CONFIG_ETH_RESET_GPIO CGPIO(14)
	#define CONFIG_ETH_RESET_IO "cgpio14"

	/* #define CONFIG_RTL83XX_SWITCH */
	/* #define CONFIG_CHIP_RTL8365MB */
	/* #define CONFIG_PHY8710 */

	/* #define CONFIG_ETH_RESET_GPIO CGPIO(28) */
	/* #define CONFIG_ETH_RESET_IO "cgpio28" */
	/* #define CONFIG_RTK_EXT_PORT EXT_PORT_0 */
#endif

#ifdef CONFIG_NAND
	#define MTDIDS_DEFAULT		"nand0=dmw_nand"
	#define MTDPARTS_DEFAULT	"mtdparts="           \
								"dmw_nand:"           \
								"256k(bastic_fact),"  \
								"1280k(uboot),"       \
								"1280k(uboot2),"      \
								"512k(uboot_env),"    \
								"1536k(nvram),"       \
								"53248k(system),"     \
								"53248k(system2),"    \
								"-(data)\0"

	#define MTDPARTS_DEFAULT2	"mtdparts="           \
								"dmw_nand:"           \
								"256k(bastic_fact),"  \
								"1280k(uboot2),"      \
								"1280k(uboot),"       \
								"512k(uboot_env),"    \
								"1536k(nvram),"       \
								"53248k(system2),"    \
								"53248k(system),"     \
								"-(data)\0"

	#define NAND_DEVTREE_SIZE 		"0x10000"
	#define NAND_DEVTREE_OFFSET		"0x20000"

	#define FLASH_ENV \
		"erasecmd=nand erase.part\0" \
		"readcmd=nand read\0"        \
		"writecmd=nand write\0"

	#define KERNEL_BOOTARGS \
		"ubi.mtd=system ubi.mtd=system2 root=/dev/mtdblock9 rootfstype=squashfs bootpartition=0\0"
	#define KERNEL_BOOTARGS2 \
		"ubi.mtd=system ubi.mtd=system2 root=/dev/mtdblock9 rootfstype=squashfs bootpartition=1\0"

	#define KERNEL_BOOTCMD \
		"run chk_parts; ubi part ${rootfspart} && "  \
		"ubi read ${loadaddr} core && "  \
		"nand read ${devtree_addr} " NAND_DEVTREE_OFFSET " " NAND_DEVTREE_SIZE " && run bootl\0"

#else
	#define CONFIG_SYS_NO_FLASH
	#define CONFIG_SYS_MAX_FLASH_SECT		1
	#define CONFIG_ENV_IS_NOWHERE
	#define CONFIG_ENV_SIZE			(128 << 10) /* 128k */
	#define FLASH_ENV
	#define KERNEL_BOOTCMD "run bootm\0"
	#define KERNEL_BOOTARGS
	#define MANIFEST_SYSTEM
	#define NAND_SLICED_UPDATE
#endif

#define UPGRADE_ENV "ubootpart=u-boot\0"

#define CONFIG_ENV_OVERWRITE
#ifdef CONFIG_NAND
	#define CONFIG_NAND_DMW96
	#define CONFIG_LZO    /* needed for UBI */
	#define CONFIG_RBTREE /* needed for UBI */

	#define CONFIG_SYS_NO_FLASH
	#define CONFIG_SYS_NAND_BASE		0x05400000
	#define CONFIG_SYS_MAX_NAND_DEVICE		1
	#define CONFIG_SYS_NAND_ONFI_DETECTION
	#define CONFIG_SYS_NAND_SELF_INIT
	#define NAND_MAX_CHIPS			1 /* needed for u-boot bbt */

	#define CONFIG_NAND_DMW96_REV1
	#define CONFIG_CMD_NAND
	#define CONFIG_CMD_UBI
	#define CONFIG_CMD_UBIFS
	#define CONFIG_CMD_MTDPROTECT

	#define CONFIG_CMD_NVRAM
	#define CONFIG_CMD_GSMAC
	#define CONFIG_CMD_GSBOOT
	#define CONFIG_CMD_GSLOADCFG

	/* Environment settings */
	#define CONFIG_ENV_IS_IN_NAND

	#define CONFIG_ENV_OFFSET		(2816 << 10) /* 2816k */
	#define CONFIG_ENV_SIZE			(128 << 10) /* 128k */
	#define CONFIG_ENV_RANGE		(128 << 12) /* 128k * 4 = 512k */
#endif

/*
 * Physical Memory Map
 */
#ifndef CONFIG_NR_DRAM_BANKS
#define CONFIG_NR_DRAM_BANKS		1
#endif
#ifndef PHYS_SDRAM_1
#define PHYS_SDRAM_1			0x40000000
#define CONFIG_SYS_TEXT_BASE		0x41000000
#define CONFIG_SYS_LOAD_ADDR		0x42000000
#define CONFIG_SYS_LOAD_ADDR_SKIP_HEADER 0x42004000
#endif
#define CONFIG_SYS_SDRAM_BASE		PHYS_SDRAM_1

#ifndef CONFIG_SYS_DEVTREE_OFFSET
#define CONFIG_SYS_DEVTREE_OFFSET	"0x43000000"
#endif
#define SYS_DEVTREE_OFFSET		"devtree_addr=" CONFIG_SYS_DEVTREE_OFFSET "\0"
#ifndef CONFIG_SYS_DEVICETREE
#define CONFIG_SYS_DEVICETREE		"dvf101-ht818.dtb"
#endif
#define SYS_DEVICETREE			"devicetree=" CONFIG_SYS_DEVICETREE "\0"

#ifndef CONFIG_SYS_INITRD_OFFSET
#define CONFIG_SYS_INITRD_OFFSET	"0x44000000"
#endif
#define SYS_INITRD_OFFSET		"initrd_addr=" CONFIG_SYS_INITRD_OFFSET "\0"
#ifndef CONFIG_SYS_INITRD
#define CONFIG_SYS_INITRD		"rootfs.cpio.uboot"
#endif
#define SYS_INITRD			"initrd=" CONFIG_SYS_INITRD "\0"

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN		(2 * SZ_1M)

/*
 * Stack sizes - set up in start.S using sizes below
 */
#define CONFIG_SYS_INIT_SP_ADDR		(CONFIG_SYS_TEXT_BASE - SZ_4K - \
                                         GENERATED_GBL_DATA_SIZE)

#define CONFIG_SYS_DCACHE_OFF
#define CONFIG_SYS_DCACHE_INVALIDATE

#define CONFIG_ARCH_CPU_INIT
#define CONFIG_BOARD_EARLY_INIT_F
#define CONFIG_BOARD_LATE_INIT
#define CFG_PBSIZE			200

#define CONFIG_ARMCORTEXA9
#define CONFIG_HAVE_CCU

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO
#define CONFIG_VERSION_VARIABLE

#define CONFIG_SYS_ALT_MEMTEST
#define CONFIG_SYS_MEMTEST_START	0x41000000
#define CONFIG_SYS_MEMTEST_END		0x410fffff
#define CONFIG_SYS_MEMTEST_SCRATCH	0x40000000

#define CONFIG_INITRD_TAG		1
#define CONFIG_CMDLINE_TAG /* pass cmdline  */
#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_OF_LIBFDT

#define CONFIG_DVF101_RAM_SIZE_OVERWRITE (256 * 1024 * 1024)

#define CONFIG_SYS_HZ			1000

/*
 * Hardware drivers
 */
#ifdef CONFIG_NETWORKING
	#define CONFIG_PHYLIB
	#define CONFIG_CMD_MII
	#define CONFIG_DESIGNWARE_ETH_QOS
	#define CONFIG_RANDOM_MACADDR
#endif

/*
 * NS16550 Configuration
 */
#define CFG_DSPG_DMW_SERIAL

#define CONFIG_SERIAL_MULTI

#define CONFIG_CMDLINE_EDITING		1
#define CONFIG_AUTO_COMPLETE		1
#define CONFIG_MX_CYCLIC		1

#define CONFIG_CONS_INDEX		0

#define CONFIG_BAUDRATE			115200
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define CONFIG_BOOTP_MASK		CONFIG_BOOTP_DEFAULT

#define CONFIG_SYS_HUSH_PARSER		1
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "

#include <config_cmd_default.h>
#define CONFIG_CMD_IMI
#define CONFIG_CMD_BDI
#define CONFIG_CMD_MEMORY
#define CONFIG_CMD_ENV
#define CONFIG_CMD_RUN
#define CONFIG_CMD_ECHO
#define CONFIG_CMD_ITEST
#define CONFIG_CMD_REV
#define CONFIG_CMD_GREPENV
#define CONFIG_CMD_DEFENV
#define CONFIG_CMD_MD5SUM
#define CONFIG_MD5
#define CONFIG_CMD_SETEXPR
#ifdef CONFIG_NETWORKING
	#define CONFIG_CMD_NET
	#define CONFIG_CMD_DHCP
	#define CONFIG_CMD_PING
#endif
#if defined(CONFIG_MMC) && defined(CONFIG_EMMC)
	#error "Do not define MMC and EMMC at the same time"
#endif
#ifdef CONFIG_MMC
	#define CONFIG_DOS_PARTITION
	#define CONFIG_GENERIC_MMC
	#define CONFIG_SYNDWC_MMC
	#define CONFIG_CMD_MMC
	#define CONFIG_CMD_FAT
	#define CONFIG_SYS_SYNDWC_BASE CONFIG_SD_MMC2_BASE
#endif

#define CONFIG_BOOTDELAY		1
#define CONFIG_BOOTABORTKEY		0x1b /* ascii for escape */
#define CONFIG_TFTP_PORT		1

#define CONFIG_NET_RETRY_COUNT		7

#define CONFIG_BOOTCOMMAND \
	"gsboot"

#ifdef DW_USBTTY_AS_DEFAULT
#define USBTTY_ENV_SETTINGS \
	"stdin=usbtty\0"  \
	"stdout=usbtty\0" \
	"stderr=usbtty\0"
#elif defined(CONFIG_ARM_DCC)
#define USBTTY_ENV_SETTINGS \
	"stdin=dcc\0"  \
	"stdout=dcc\0" \
	"stderr=dcc\0"
#else
#define USBTTY_ENV_SETTINGS
#endif

#define CONFIG_EXTRA_ENV_SETTINGS \
	SYS_DEVTREE_OFFSET            \
	SYS_DEVICETREE                \
	"ethaddr=00:0B:82:00:00:00\0" \
	"ipaddr=192.168.0.240\0"      \
	"serverip=192.168.0.100\0"    \
	"mtdparts=" MTDPARTS_DEFAULT  \
	"chk_parts=test -n ${mtdparts} || mtdparts default\0"             \
	"dfu_manifest=run bootm\0"                                        \
	"console=console=ttyS0\0"                                         \
	"uart_locked=uart_locked=0\0"                                     \
	"rootfspart=system\0"                                             \
	"linux_boot=" KERNEL_BOOTCMD                                      \
	"linux_bootargs=" KERNEL_BOOTARGS                                 \
	"bootm=setenv bootargs ${console} ${linux_bootargs} ${mtdparts} " \
		"&& bootm ${loadaddr} - ${devtree_addr}\0"                    \
	"bootl=setenv bootargs ${console} ${linux_bootargs} ${mtdparts} " \
		"&& bootm ${loadaddr_hdr} - ${devtree_addr}\0"                \
	"linux_boot_nfs=tftp ${loadaddr} uImage "                         \
		"&& tftp ${devtree_addr} ${devicetree} "                      \
		"&& run bootm\0"                                              \
	"linux_bootargs_nfs=root=/dev/nfs ip=dhcp "                       \
		"nfsroot=192.168.0.1:/opt/nfs/rootfs.ht818\0"                 \
	"setnfsmode=setenv linux_boot ${linux_boot_nfs} "                 \
		"&& setenv linux_bootargs ${linux_bootargs_nfs} "             \
		"&& save\0"                                                   \
	"bootram=setenv bootargs ${console} ${linux_bootargs} ${mtdparts} "\
		"&& bootm ${loadaddr} ${initrd_addr} ${devtree_addr}\0"       \
	"clearenv=nand erase.part uboot_env\0"                    \
	"clearnvram=nand erase.part nvram\0"                      \
	"tftpfactcfg=load_cfg && tftp ${MODEL}_${PNSTR}.cfg\0"    \
	"tftpbootdev=load_cfg && tftp ${GSMODEL}bootdev.bin\0"    \
	"tftpcoredev=load_cfg && tftp ${GSMODEL}coredev.bin\0"    \
	"tftpbasedev=load_cfg && tftp ${GSMODEL}basedev.bin\0"    \
	"tftpprogdev=load_cfg && tftp ${GSMODEL}progdev.bin\0"    \
	"tftpsystemubi=load_cfg && tftp ${GSMODEL}system.ubi\0"   \
	"loadfactcfg=nand erase.part bastic_fact && nand write ${loadaddr} bastic_fact ${filesize}\0"   \
	"loadboot=nand erase.part uboot && nand write ${loadaddr} uboot ${filesize}\0"                  \
	"loadboot2=nand erase.part uboot2 && nand write ${loadaddr} uboot2 ${filesize}\0"                \
	"loadsystem=nand erase.part system && nand write ${loadaddr} system ${filesize}\0"              \
	"loadsystem2=nand erase.part system2 && nand write ${loadaddr} system2 ${filesize}\0"           \
	"updatefact=run tftpfactcfg && nand erase.part bastic_fact && nand write ${loadaddr} bastic_fact ${filesize}\0" \
	"updateboot=run tftpbootdev loadboot loadboot2\0"           \
	"updatesystem=run tftpsystemubi loadsystem loadsystem2\0"   \
	"updatecore=run tftpcoredev ubipart_system ubiload_core ubipart_system2 ubiload_core\0"    \
	"updatebase=run tftpbasedev ubipart_system ubiload_base ubipart_system2 ubiload_base\0"    \
	"updateprog=run tftpprogdev ubipart_system ubiload_prog ubipart_system2 ubiload_prog\0"    \
	"updateall=run updateboot updatesystem\0" \
	"ubipart_system=ubi part system\0"               \
	"ubipart_system2=ubi part system2\0"             \
	"ubivoladd_core=ubi create core 400000 s\0"  \
	"ubivoladd_base=ubi create base F00000 s\0"  \
	"ubivoladd_prog=ubi create prog 1800000 s\0" \
	"ubivoladd_lang=ubi create lang 200000 s\0"  \
	"ubivoladd_oem=ubi create oem 100000 s\0"    \
	"ubivoladd_all=run ubivoladd_core ubivoladd_base ubivoladd_prog ubivoladd_lang ubivoladd_oem\0" \
	"ubivoladdall=run ubipart_system ubivoladd_all ubipart_system2 ubivoladd_all\0"                 \
	"ubiload_core=ubi check core && ubi write ${loadaddr} core ${filesize}\0" \
	"ubiload_base=ubi check base && ubi write ${loadaddr} base ${filesize}\0" \
	"ubiload_prog=ubi check prog && ubi write ${loadaddr} prog ${filesize}\0" \
	"ubiload_lang=ubi check lang && ubi write ${loadaddr} lang ${filesize}\0" \
	"ubiload_oem=ubi check oem && ubi write ${loadaddr} oem ${filesize}\0"    \
	"ubiupdate_core=run tftpcoredev ubiload_core\0"     \
	"ubiupdate_base=run tftpbasedev ubiload_base\0"     \
	"ubiupdate_prog=run tftpprogdev ubiload_prog\0"     \
	"ubiupdate_lang=run tftplangdev ubiload_lang\0"     \
	"ubiupdate_oem=run tftpoemdev ubiload_oem\0"        \
	"ubiupdate_all=run ubiupdate_core ubiupdate_base ubiupdate_prog ubiupdate_lang ubiupdate_oem\0" \
	"ubiupdateall=run ubipart_system ubiupdate_all ubipart_system2 ubiupdate_all\0" \
	USBTTY_ENV_SETTINGS

/*
 * Static configuration when assigning fixed address
 */
#define CONFIG_BOOTFILE			"uImage"
#define CONFIG_LOADADDR			CONFIG_SYS_LOAD_ADDR
#define CONFIG_LOADADDR_HEADER	CONFIG_SYS_LOAD_ADDR_SKIP_HEADER

#ifdef CONFIG_NAND
	/*
	 * Flash configuration
	 */
	#define CONFIG_CMD_MTDPARTS
	#define CONFIG_MTD_DEVICE
	#define CONFIG_MTD_PARTITIONS
#elif defined(CONFIG_EMMC)
	#define CONFIG_DOS_PARTITION
	#define CONFIG_EFI_PARTITION
	#define CONFIG_CMD_DISK
	#define CONFIG_CMD_GPTPART
	#define CONFIG_CMD_EXT4
	#define CONFIG_FS_EXT4
#endif

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP		/* undef to save memory */
#define CONFIG_SYS_PROMPT		"HT818 # "
#define CONFIG_SYS_CBSIZE		1024

/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE + \
					 sizeof(CONFIG_SYS_PROMPT) + 16)
/* max number of command args */
#define CONFIG_SYS_MAXARGS		64
/* boot argument buffer size */
#define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE

#define CONFIG_MTD_NAND_VERIFY_WRITE
#define CONFIG_LIB_RAND
#define CONFIG_CMD_MEMTEST

/* Secure boot */
/* #define CONFIG_DVF_SECURE_BOOT */
/* #define CONFIG_DVF_SECURE_BOOT_DEBUG */

#define CONFIG_SYS_GENERIC_BOARD

#endif
