#ifndef __CONFIG_EVB99_NAND_H
#define __CONFIG_EVB99_NAND_H

#define CONFIG_IDENT_STRING		" lp99-nand"

#define CONFIG_SDMMC_LOC_VAL	0
#define CONFIG_SDMMC_4BIT

/*
 * Configure which UART to use for stdio
 * 0: choose UART1 or UART2 according to straps
 * 1: UART1
 * 2: UART2
 */
#define CONFIG_USE_UART		0

/* Only relevant when using UART2 */
#define CONFIG_UART2_LOC_VAL	0

#define MTDIDS_DEFAULT		"nand0=dmw_nand"
#define MTDPARTS_DEFAULT	"mtdparts="       \
					"dmw_nand:"           \
					"2m(u-boot),"         \
					"1m(u-boot-env),"     \
					"1m(devtree),"        \
					"5m(kernel),"         \
					"-(rootfs)"

#define NAND_BOOTARGS \
	"ubi.mtd=rootfs root=ubi0:rootfs rootfstype=ubifs nodebug\0"

#define NAND_BOOTCMD \
	"run chk_parts; nand read ${loadaddr} kernel && "                 \
	"nand read ${devtree_addr} devtree && run bootm\0"

#define BOOTMEDIUM_ENV_SETTINGS \
	"linux_boot=" NAND_BOOTCMD                                        \
	"linux_boot_ubi=" NAND_BOOTCMD                                    \
	"linux_bootargs=" NAND_BOOTARGS                                   \
	"linux_bootargs_ubi=" NAND_BOOTARGS                               \
	"setubimode=setenv linux_boot ${linux_boot_ubi} "                 \
		"&& set linux_bootargs ${linux_bootargs_ubi} "            \
		"&& save\0"                                               \
	"chk_parts=test -n ${mtdparts} || mtdparts default\0"             \
	"dfu_manifest_2=run chk_parts; run dfu_manifest_u-boot-env\0"     \
	"dfu_manifest_u-boot=run u-boot-upgrade\0"                        \
	"dfu_manifest_u-boot-env=nand erase.part u-boot-env "             \
		"&& nand write ${loadaddr} u-boot-env\0"                  \
	"dfu_manifest_devtree=run devtree-upgrade\0"                      \
	"dfu_manifest_kernel=nand erase.part kernel "                     \
		"&& nand write.jffs2 ${loadaddr} kernel\0"                \
	"dfu_manifest_system=nand erase.part rootfs "                    \
		"&& ubi part rootfs "										\
		"&& ubi create rootfs "										\
		"&& ubi write ${loadaddr} rootfs ${filesize} "				\
		"; reset\0"													\
	"u-boot-upgrade=run chk_parts; "                                  \
		"nand erase.part u-boot "                                 \
		"&& nand write ${loadaddr} u-boot "                       \
		"&& save "                                                \
		"&& echo Done. Please do a power cycle to reset board\0"  \
	"devtree-upgrade=run chk_parts; nand erase.part devtree "         \
		"&& nand write ${loadaddr} devtree "                      \
		"&& save "                                                \
		"&& echo Done.\0"                                         \
	"bgvalue=100\0"

#define CONFIG_LZO    /* needed for UBI */
#define CONFIG_RBTREE /* needed for UBI */

#define CONFIG_SYS_NO_FLASH
#define CONFIG_SYS_NAND_BASE		0x05400000
#define CONFIG_SYS_MAX_NAND_DEVICE	1
#define CONFIG_SYS_NAND_ONFI_DETECTION
#define CONFIG_SYS_NAND_SELF_INIT
#define NAND_MAX_CHIPS			1 /* needed for u-boot bbt */

#define CONFIG_NAND_DMW96
#define CONFIG_CMD_NAND
#define CONFIG_CMD_UBI
#define CONFIG_CMD_UBIFS
#define CONFIG_CMD_MTDPROTECT

/* Environment settings */
#define CONFIG_ENV_OVERWRITE
#define CONFIG_ENV_IS_IN_NAND
#define CONFIG_ENV_OFFSET		(  2 << 20) /* 2M */
#define CONFIG_ENV_SIZE			(256 << 10) /* 256k */
#define CONFIG_ENV_RANGE		(  1 << 20) /* 1M */

#include <asm/arch/gpio.h>
#include <linux/sizes.h>

/*
 * Physical Memory Map
 */
#define CONFIG_NR_DRAM_BANKS		1
#define PHYS_SDRAM_1			0x40000000
#define CONFIG_SYS_SDRAM_BASE		PHYS_SDRAM_1

#define CONFIG_SYS_TEXT_BASE		0x41000000
#define CONFIG_SYS_LOAD_ADDR		0x42000000

/*
 * Size of malloc() pool
 */
#define CONFIG_SYS_MALLOC_LEN		(2 * SZ_1M)

/*
 * Stack sizes - set up in start.S using sizes below
 */
#define CONFIG_SYS_INIT_SP_ADDR		(CONFIG_SYS_SDRAM_BASE + SZ_4K - \
                                         GENERATED_GBL_DATA_SIZE)
#define CONFIG_STACKSIZE		(SZ_128K)	/* regular stack */

#define CONFIG_SYS_DCACHE_OFF

#define CONFIG_ARCH_CPU_INIT
#define CONFIG_BOARD_EARLY_INIT_F
#define CONFIG_BOARD_LATE_INIT
#define CFG_PBSIZE			200

#define CONFIG_ARM926EJS
#define CONFIG_HAVE_CCU

#define CONFIG_DISPLAY_CPUINFO
#define CONFIG_DISPLAY_BOARDINFO
#define CONFIG_VERSION_VARIABLE

#define CONFIG_SYS_ALT_MEMTEST
#define CONFIG_SYS_MEMTEST_START	0x100000
#define CONFIG_SYS_MEMTEST_END		0x10000000

#define CONFIG_INITRD_TAG		1
#define CONFIG_CMDLINE_TAG /* pass cmdline  */
#define CONFIG_SETUP_MEMORY_TAGS
#define CONFIG_OF_LIBFDT

//#define CONFIG_LCD
#define LCD_BPP				LCD_COLOR16
//#define CONFIG_CMD_BMP
#define CONFIG_BMP_16BP

/* #define VIDEO_FB_LITTLE_ENDIAN */
/* #define VIDEO_FB_16BPP_PIXEL_SWAP */
#define VIDEO_FB_16BPP_WORD_SWAP

#define CONFIG_SYS_HZ			1000

#define CONFIG_MMC
#define CONFIG_DOS_PARTITION
#define CONFIG_GENERIC_MMC
#define CONFIG_SYNDWC_MMC

/*
 * NS16550 Configuration
 */
#define CFG_DSPG_DMW_SERIAL

#define CONFIG_CMDLINE_EDITING		1
#define CONFIG_AUTO_COMPLETE		1
#define CONFIG_MX_CYCLIC		1

#define CONFIG_CONS_INDEX		0

#define CONFIG_BAUDRATE			115200
#define CONFIG_SYS_BAUDRATE_TABLE	{ 9600, 19200, 38400, 57600, 115200 }

#define CONFIG_BOOTP_MASK		CONFIG_BOOTP_DEFAULT

#define CONFIG_SYS_HUSH_PARSER		1
#define CONFIG_SYS_PROMPT_HUSH_PS2	"> "

#include <config_cmd_default.h>
#define CONFIG_CMD_IMI
#define CONFIG_CMD_BDI
#define CONFIG_CMD_MEMORY
#define CONFIG_CMD_ENV
#define CONFIG_CMD_RUN
#define CONFIG_CMD_ECHO
#define CONFIG_CMD_ITEST
#define CONFIG_CMD_REV
#define CONFIG_CMD_MMC
#define CONFIG_CMD_FAT
#define CONFIG_CMD_GREPENV
#define CONFIG_CMD_DEFENV

#define CONFIG_BOOTDELAY		4
#define CONFIG_BOOTABORTKEY		0x1b /* ascii for escape */
#define CONFIG_TFTP_PORT		1

#define CONFIG_NET_RETRY_COUNT		7

#define CONFIG_BOOTCOMMAND \
	"if mmc rescan; then " \
		"if run loadbootscript; then " \
			"echo \"Boot script found on SD card. Do 'run bootscript' to execute\"; " \
			/* "run bootscript; " */ \
		"else " \
			"run linux_boot; " \
		"fi; " \
	"else run linux_boot; fi"

#ifdef DW_USBTTY_AS_DEFAULT
#define USBTTY_ENV_SETTINGS \
	"stdin=usbtty\0"  \
	"stdout=usbtty\0" \
	"stderr=usbtty\0"
#elif defined(CONFIG_ARM_DCC)
#define USBTTY_ENV_SETTINGS \
	"stdin=dcc\0"  \
	"stdout=dcc\0" \
	"stderr=dcc\0"
#else
#define USBTTY_ENV_SETTINGS
#endif

#define CONFIG_EXTRA_ENV_SETTINGS \
	"devtree_addr=0x43000000\0"                                       \
	"loadbootscript=fatload mmc 0 ${loadaddr} boot.scr\0"             \
	"bootscript=echo Running bootscript from mmc ...; "               \
		"source ${loadaddr}\0"                                    \
	"dfu_manifest=run bootm\0"                                        \
	"console=console=ttyS0\0"                                         \
	"bootm=set bootargs ${console} ${linux_bootargs} ${mtdparts} "    \
		"&& bootm ${loadaddr} - ${devtree_addr}\0"                \
	"linux_boot_nfs=tftp ${loadaddr} uImage "                         \
		"&& tftp ${devtree_addr} dvf99-evb.dtb "                  \
		"&& run bootm\0"                                          \
	"linux_bootargs_nfs=root=/dev/nfs ip=dhcp "                       \
		"nfsroot=192.168.0.1:/\0"                                 \
	"setnfsmode=setenv linux_boot ${linux_boot_nfs} "                 \
		"&& set linux_bootargs ${linux_bootargs_nfs} "            \
		"&& save\0"                                               \
	BOOTMEDIUM_ENV_SETTINGS                                           \
	USBTTY_ENV_SETTINGS

/*
 * Static configuration when assigning fixed address
 */
#define CONFIG_ETHADDR			54:89:07:9B:8A:0D
#define CONFIG_NETMASK			255.255.255.0
#define CONFIG_IPADDR			192.168.0.240
#define CONFIG_SERVERIP			192.168.0.1
#define CONFIG_BOOTFILE			"uImage"
#define CONFIG_LOADADDR			CONFIG_SYS_LOAD_ADDR

/*
 * Flash configuration
 */
#define CONFIG_CMD_MTDPARTS
#define CONFIG_MTD_DEVICE
#define CONFIG_MTD_PARTITIONS

/*
 * Miscellaneous configurable options
 */
#define CONFIG_SYS_LONGHELP		/* undef to save memory */
#define CONFIG_SYS_PROMPT		"DVF99 # "
#define CONFIG_SYS_CBSIZE		1024

/* Print Buffer Size */
#define CONFIG_SYS_PBSIZE		(CONFIG_SYS_CBSIZE + \
					 sizeof(CONFIG_SYS_PROMPT) + 16)
/* max number of command args */
#define CONFIG_SYS_MAXARGS		64
/* boot argument buffer size */
#define CONFIG_SYS_BARGSIZE		CONFIG_SYS_CBSIZE

#endif
