/*
 * NVRAM variable manipulation
 *
 * Copyright 2005, Broadcom Corporation
 * All Rights Reserved.
 *
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: nvram.h,v 1.1 2013/02/05 23:28:00 sfrick Exp $
 *
 * SCF - ADM5120 port - 6/30/2006
 *     - removed broadcom specific portions and OS indepent layer
 *     - removed MTD dependence, ported to raw flash access routines
 *     - added zlib support and checksumming for nvram file
 *
 */

#ifndef __NVRAM_H__
#define __NVRAM_H__

/*
 * Initialize NVRAM access. May be unnecessary or undefined on certain
 * platforms.
 */
int dev_nvram_init(void);

/*
 * Get the value of an NVRAM variable. The pointer returned may be
 * invalid after a set.
 * @param	name	name of variable to get
 * @return	value of variable or NULL if undefined
 */
char* nvram_get(const char *name);

int nvram_getall(char *buf, int count);
int nvram_unset(const char *name);
int nvram_set(const char *name, const char *value);
void nvram_commit(void);

#if defined( CONFIG_HT818 )
	#define NVRAM_SPACE           0x4000
	#define NVRAM_BUFFER_SPACE    (32 * NVRAM_SPACE)
#else
    #error NO PRODUCT DEFINED
#endif
#endif /* _nvram_h_ */
